@@ -53,9 +53,8 @@ static const char *bc_get_joint_name(T *node)
 static EditBone *get_edit_bone(bArmature * armature, char *name) {
 	EditBone  *eBone;
 
-	eBone = (EditBone *)armature->edbo->first;
-	for (eBone; eBone; eBone = eBone->next) {
-		if (!strcmp(name, eBone->name))
+	for (eBone = (EditBone *)armature->edbo->first; eBone; eBone = eBone->next) {
+		if (STREQ(name, eBone->name))
 			return eBone;
 	}
 
@@ -66,7 +65,7 @@ static EditBone *get_edit_bone(bArmature * armature, char *name) {
 
 
 ArmatureImporter::ArmatureImporter(UnitConverter *conv, MeshImporterBase *mesh, Scene *sce, const ImportSettings *import_settings) :
-    import_settings(import_settings),
+	import_settings(import_settings),
 	unit_converter(conv),
 	TransformReader(conv), 
 	scene(sce), 
@@ -259,7 +258,7 @@ void ArmatureImporter::connect_bone_chains(bArmature *armature, Bone *parentbone
 	Bone *child = (Bone *)parentbone->childbase.first;
 	if (child && (import_settings->find_chains || child->next==NULL) )
 	{
-		for (child; child; child = child->next) {
+		for (; child; child = child->next) {
 			BoneExtended *be = extended_bones[child->name];
 			if (be != NULL) {
 				if (be->get_chain_length() <= clip) {
@@ -589,9 +588,11 @@ void ArmatureImporter::create_armature_bones(SkinInfo& skin)
 	/* and step back to edit mode to fix the leaf nodes */
 	ED_armature_to_edit(armature);
 
-	connect_bone_chains(armature, (Bone *)armature->bonebase.first, UNLIMITED_CHAIN_MAX);
-	fix_leaf_bones(armature, (Bone *)armature->bonebase.first);
-
+	if (armature->bonebase.first) {
+		/* Do this only if Armature has bones */
+		connect_bone_chains(armature, (Bone *)armature->bonebase.first, UNLIMITED_CHAIN_MAX);
+		fix_leaf_bones(armature, (Bone *)armature->bonebase.first);
+	}
 	// exit armature edit mode
 	ED_armature_from_edit(armature);
 	ED_armature_edit_free(armature);
