@@ -35,6 +35,7 @@
 #include "BKE_context.h"
 #include "BKE_mask.h"
 
+#include "DNA_mask_types.h"
 #include "DNA_scene_types.h"
 
 #include "WM_api.h"
@@ -69,7 +70,7 @@ int ED_maskedit_poll(bContext *C)
 				return ED_space_image_maskedit_poll(C);
 		}
 	}
-	return FALSE;
+	return false;
 }
 
 int ED_maskedit_mask_poll(bContext *C)
@@ -85,7 +86,7 @@ int ED_maskedit_mask_poll(bContext *C)
 				return ED_space_image_maskedit_mask_poll(C);
 		}
 	}
-	return FALSE;
+	return false;
 }
 
 /********************** registration *********************/
@@ -324,15 +325,13 @@ void ED_mask_pixelspace_factor(ScrArea *sa, ARegion *ar, float *scalex, float *s
 			case SPACE_CLIP:
 			{
 				SpaceClip *sc = sa->spacedata.first;
-				int width, height;
-				float zoomx, zoomy, aspx, aspy;
+				float aspx, aspy;
 
-				ED_space_clip_get_size(sc, &width, &height);
-				ED_space_clip_get_zoom(sc, ar, &zoomx, &zoomy);
+				UI_view2d_scale_get(&ar->v2d, scalex, scaley);
 				ED_space_clip_get_aspect(sc, &aspx, &aspy);
 
-				*scalex = ((float)width * aspx) * zoomx;
-				*scaley = ((float)height * aspy) * zoomy;
+				*scalex *= aspx;
+				*scaley *= aspy;
 				break;
 			}
 			case SPACE_SEQ:
@@ -343,15 +342,13 @@ void ED_mask_pixelspace_factor(ScrArea *sa, ARegion *ar, float *scalex, float *s
 			case SPACE_IMAGE:
 			{
 				SpaceImage *sima = sa->spacedata.first;
-				int width, height;
-				float zoomx, zoomy, aspx, aspy;
+				float aspx, aspy;
 
-				ED_space_image_get_size(sima, &width, &height);
-				ED_space_image_get_zoom(sima, ar, &zoomx, &zoomy);
+				UI_view2d_scale_get(&ar->v2d, scalex, scaley);
 				ED_space_image_get_aspect(sima, &aspx, &aspy);
 
-				*scalex = ((float)width * aspx) * zoomx;
-				*scaley = ((float)height * aspy) * zoomy;
+				*scalex *= aspx;
+				*scaley *= aspy;
 				break;
 			}
 			default:
@@ -367,6 +364,92 @@ void ED_mask_pixelspace_factor(ScrArea *sa, ARegion *ar, float *scalex, float *s
 	}
 }
 
+void ED_mask_cursor_location_get(ScrArea *sa, float cursor[2])
+{
+	if (sa) {
+		switch (sa->spacetype) {
+			case SPACE_CLIP:
+			{
+				SpaceClip *space_clip = sa->spacedata.first;
+				copy_v2_v2(cursor, space_clip->cursor);
+				break;
+			}
+			case SPACE_SEQ:
+			{
+				zero_v2(cursor);
+				break;
+			}
+			case SPACE_IMAGE:
+			{
+				SpaceImage *space_image = sa->spacedata.first;
+				copy_v2_v2(cursor, space_image->cursor);
+				break;
+			}
+			default:
+				/* possible other spaces from which mask editing is available */
+				BLI_assert(0);
+				zero_v2(cursor);
+				break;
+		}
+	}
+	else {
+		BLI_assert(0);
+		zero_v2(cursor);
+	}
+}
+
+bool ED_mask_selected_minmax(const bContext *C, float min[2], float max[2])
+{
+	Mask *mask = CTX_data_edit_mask(C);
+	MaskLayer *mask_layer;
+	bool ok = false;
+	INIT_MINMAX2(min, max);
+	for (mask_layer = mask->masklayers.first;
+	     mask_layer != NULL;
+	     mask_layer = mask_layer->next)
+	{
+		MaskSpline *spline;
+		if (mask_layer->restrictflag & (MASK_RESTRICT_VIEW | MASK_RESTRICT_SELECT)) {
+			continue;
+		}
+		for (spline = mask_layer->splines.first;
+		     spline != NULL;
+		     spline = spline->next)
+		{
+			MaskSplinePoint *points_array = BKE_mask_spline_point_array(spline);
+			int i;
+			for (i = 0; i < spline->tot_point; i++) {
+				MaskSplinePoint *point = &spline->points[i];
+				MaskSplinePoint *deform_point = &points_array[i];
+				BezTriple *bezt = &point->bezt;
+				float handle[2];
+				if (!MASKPOINT_ISSEL_ANY(point)) {
+					continue;
+				}
+				if (bezt->f2 & SELECT) {
+					minmax_v2v2_v2(min, max, deform_point->bezt.vec[1]);
+				}
+				if (BKE_mask_point_handles_mode_get(point) == MASK_HANDLE_MODE_STICK) {
+					BKE_mask_point_handle(deform_point, MASK_WHICH_HANDLE_STICK, handle);
+					minmax_v2v2_v2(min, max, handle);
+				}
+				else {
+					if ((bezt->f1 & SELECT) && (bezt->h1 != HD_VECT)) {
+						BKE_mask_point_handle(deform_point, MASK_WHICH_HANDLE_LEFT, handle);
+						minmax_v2v2_v2(min, max, handle);
+					}
+					if ((bezt->f3 & SELECT) && (bezt->h2 != HD_VECT)) {
+						BKE_mask_point_handle(deform_point, MASK_WHICH_HANDLE_RIGHT, handle);
+						minmax_v2v2_v2(min, max, handle);
+					}
+				}
+				ok = true;
+			}
+		}
+	}
+	return ok;
+}
+
 /********************** registration *********************/
 
 void ED_operatortypes_mask(void)
@@ -380,6 +463,8 @@ void ED_operatortypes_mask(void)
 	/* add */
 	WM_operatortype_append(MASK_OT_add_vertex);
 	WM_operatortype_append(MASK_OT_add_feather_vertex);
+	WM_operatortype_append(MASK_OT_primitive_circle_add);
+	WM_operatortype_append(MASK_OT_primitive_square_add);
 
 	/* geometry */
 	WM_operatortype_append(MASK_OT_switch_direction);
@@ -394,6 +479,8 @@ void ED_operatortypes_mask(void)
 	WM_operatortype_append(MASK_OT_select_circle);
 	WM_operatortype_append(MASK_OT_select_linked_pick);
 	WM_operatortype_append(MASK_OT_select_linked);
+	WM_operatortype_append(MASK_OT_select_more);
+	WM_operatortype_append(MASK_OT_select_less);
 
 	/* hide/reveal */
 	WM_operatortype_append(MASK_OT_hide_view_clear);
@@ -404,6 +491,7 @@ void ED_operatortypes_mask(void)
 
 	/* shape */
 	WM_operatortype_append(MASK_OT_slide_point);
+	WM_operatortype_append(MASK_OT_slide_spline_curvature);
 	WM_operatortype_append(MASK_OT_cyclic_toggle);
 	WM_operatortype_append(MASK_OT_handle_type_set);
 
@@ -419,6 +507,13 @@ void ED_operatortypes_mask(void)
 
 	/* layers */
 	WM_operatortype_append(MASK_OT_layer_move);
+
+	/* duplicate */
+	WM_operatortype_append(MASK_OT_duplicate);
+
+	/* clipboard */
+	WM_operatortype_append(MASK_OT_copy_splines);
+	WM_operatortype_append(MASK_OT_paste_splines);
 }
 
 void ED_keymap_mask(wmKeyConfig *keyconf)
@@ -431,6 +526,9 @@ void ED_keymap_mask(wmKeyConfig *keyconf)
 
 	WM_keymap_add_item(keymap, "MASK_OT_new", NKEY, KM_PRESS, KM_ALT, 0);
 
+	/* add menu */
+	WM_keymap_add_menu(keymap, "MASK_MT_add", AKEY, KM_PRESS, KM_SHIFT, 0);
+
 	/* mask mode supports PET now */
 	ED_keymap_proportional_cycle(keyconf, keymap);
 	ED_keymap_proportional_maskmode(keyconf, keymap);
@@ -443,13 +541,13 @@ void ED_keymap_mask(wmKeyConfig *keyconf)
 
 	/* selection */
 	kmi = WM_keymap_add_item(keymap, "MASK_OT_select", SELECTMOUSE, KM_PRESS, 0, 0);
-	RNA_boolean_set(kmi->ptr, "extend", FALSE);
-	RNA_boolean_set(kmi->ptr, "deselect", FALSE);
-	RNA_boolean_set(kmi->ptr, "toggle", FALSE);
+	RNA_boolean_set(kmi->ptr, "extend", false);
+	RNA_boolean_set(kmi->ptr, "deselect", false);
+	RNA_boolean_set(kmi->ptr, "toggle", false);
 	kmi = WM_keymap_add_item(keymap, "MASK_OT_select", SELECTMOUSE, KM_PRESS, KM_SHIFT, 0);
-	RNA_boolean_set(kmi->ptr, "extend", FALSE);
-	RNA_boolean_set(kmi->ptr, "deselect", FALSE);
-	RNA_boolean_set(kmi->ptr, "toggle", TRUE);
+	RNA_boolean_set(kmi->ptr, "extend", false);
+	RNA_boolean_set(kmi->ptr, "deselect", false);
+	RNA_boolean_set(kmi->ptr, "toggle", true);
 
 	kmi = WM_keymap_add_item(keymap, "MASK_OT_select_all", AKEY, KM_PRESS, 0, 0);
 	RNA_enum_set(kmi->ptr, "action", SEL_TOGGLE);
@@ -458,35 +556,39 @@ void ED_keymap_mask(wmKeyConfig *keyconf)
 
 	WM_keymap_add_item(keymap, "MASK_OT_select_linked", LKEY, KM_PRESS, KM_CTRL, 0);
 	kmi = WM_keymap_add_item(keymap, "MASK_OT_select_linked_pick", LKEY, KM_PRESS, 0, 0);
-	RNA_boolean_set(kmi->ptr, "deselect", FALSE);
+	RNA_boolean_set(kmi->ptr, "deselect", false);
 	kmi = WM_keymap_add_item(keymap, "MASK_OT_select_linked_pick", LKEY, KM_PRESS, KM_SHIFT, 0);
-	RNA_boolean_set(kmi->ptr, "deselect", TRUE);
+	RNA_boolean_set(kmi->ptr, "deselect", true);
 
 	WM_keymap_add_item(keymap, "MASK_OT_select_border", BKEY, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "MASK_OT_select_circle", CKEY, KM_PRESS, 0, 0);
 
 	kmi = WM_keymap_add_item(keymap, "MASK_OT_select_lasso", EVT_TWEAK_A, KM_ANY, KM_CTRL | KM_ALT, 0);
-	RNA_boolean_set(kmi->ptr, "deselect", FALSE);
+	RNA_boolean_set(kmi->ptr, "deselect", false);
 	kmi = WM_keymap_add_item(keymap, "MASK_OT_select_lasso", EVT_TWEAK_A, KM_ANY, KM_CTRL | KM_SHIFT | KM_ALT, 0);
-	RNA_boolean_set(kmi->ptr, "deselect", TRUE);
+	RNA_boolean_set(kmi->ptr, "deselect", true);
+
+	WM_keymap_add_item(keymap, "MASK_OT_select_more", PADPLUSKEY, KM_PRESS, KM_CTRL, 0);
+	WM_keymap_add_item(keymap, "MASK_OT_select_less", PADMINUS, KM_PRESS, KM_CTRL, 0);
 
 	/* hide/reveal */
 	WM_keymap_add_item(keymap, "MASK_OT_hide_view_clear", HKEY, KM_PRESS, KM_ALT, 0);
 	kmi = WM_keymap_add_item(keymap, "MASK_OT_hide_view_set", HKEY, KM_PRESS, 0, 0);
-	RNA_boolean_set(kmi->ptr, "unselected", FALSE);
+	RNA_boolean_set(kmi->ptr, "unselected", false);
 
 	kmi = WM_keymap_add_item(keymap, "MASK_OT_hide_view_set", HKEY, KM_PRESS, KM_SHIFT, 0);
-	RNA_boolean_set(kmi->ptr, "unselected", TRUE);
+	RNA_boolean_set(kmi->ptr, "unselected", true);
 
 	/* select clip while in maker view,
 	 * this matches View3D functionality where you can select an
 	 * object while in editmode to allow vertex parenting */
 	kmi = WM_keymap_add_item(keymap, "CLIP_OT_select", SELECTMOUSE, KM_PRESS, KM_CTRL, 0);
-	RNA_boolean_set(kmi->ptr, "extend", FALSE);
+	RNA_boolean_set(kmi->ptr, "extend", false);
 
 	/* shape */
 	WM_keymap_add_item(keymap, "MASK_OT_cyclic_toggle", CKEY, KM_PRESS, KM_ALT, 0);
 	WM_keymap_add_item(keymap, "MASK_OT_slide_point", ACTIONMOUSE, KM_PRESS, 0, 0);
+	WM_keymap_add_item(keymap, "MASK_OT_slide_spline_curvature", ACTIONMOUSE, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "MASK_OT_handle_type_set", VKEY, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "MASK_OT_normals_make_consistent", NKEY, KM_PRESS, KM_CTRL, 0);
 	// WM_keymap_add_item(keymap, "MASK_OT_feather_weight_clear", SKEY, KM_PRESS, KM_ALT, 0);
@@ -499,6 +601,11 @@ void ED_keymap_mask(wmKeyConfig *keyconf)
 	WM_keymap_add_item(keymap, "MASK_OT_shape_key_insert", IKEY, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "MASK_OT_shape_key_clear", IKEY, KM_PRESS, KM_ALT, 0);
 
+	/* duplicate */
+	WM_keymap_add_item(keymap, "MASK_OT_duplicate_move", DKEY, KM_PRESS, KM_SHIFT, 0);
+	WM_keymap_add_item(keymap, "MASK_OT_copy_splines", CKEY, KM_PRESS, KM_CTRL, 0);
+	WM_keymap_add_item(keymap, "MASK_OT_paste_splines", VKEY, KM_PRESS, KM_CTRL, 0);
+
 	/* for image editor only */
 	WM_keymap_add_item(keymap, "UV_OT_cursor_set", ACTIONMOUSE, KM_PRESS, 0, 0);
 
@@ -513,7 +620,6 @@ void ED_keymap_mask(wmKeyConfig *keyconf)
 
 void ED_operatormacros_mask(void)
 {
-	/* XXX: just for sample */
 	wmOperatorType *ot;
 	wmOperatorTypeMacro *otmacro;
 
@@ -521,13 +627,20 @@ void ED_operatormacros_mask(void)
 	                                  "Add new vertex and slide it", OPTYPE_UNDO | OPTYPE_REGISTER);
 	ot->description = "Add new vertex and slide it";
 	WM_operatortype_macro_define(ot, "MASK_OT_add_vertex");
-	otmacro = WM_operatortype_macro_define(ot, "TRANSFORM_OT_translate");
-	RNA_boolean_set(otmacro->ptr, "release_confirm", TRUE);
+	otmacro = WM_operatortype_macro_define(ot, "MASK_OT_slide_point");
+	RNA_boolean_set(otmacro->ptr, "is_new_point", true);
 
 	ot = WM_operatortype_append_macro("MASK_OT_add_feather_vertex_slide", "Add Feather Vertex and Slide",
 	                                  "Add new vertex to feather and slide it", OPTYPE_UNDO | OPTYPE_REGISTER);
 	ot->description = "Add new feather vertex and slide it";
 	WM_operatortype_macro_define(ot, "MASK_OT_add_feather_vertex");
 	otmacro = WM_operatortype_macro_define(ot, "MASK_OT_slide_point");
-	RNA_boolean_set(otmacro->ptr, "slide_feather", TRUE);
+	RNA_boolean_set(otmacro->ptr, "slide_feather", true);
+
+	ot = WM_operatortype_append_macro("MASK_OT_duplicate_move", "Add Duplicate", "Duplicate mask and move",
+	                                  OPTYPE_UNDO | OPTYPE_REGISTER);
+	WM_operatortype_macro_define(ot, "MASK_OT_duplicate");
+	otmacro = WM_operatortype_macro_define(ot, "TRANSFORM_OT_translate");
+	RNA_enum_set(otmacro->ptr, "proportional", 0);
+	RNA_boolean_set(otmacro->ptr, "mirror", false);
 }
