@@ -36,14 +36,16 @@
 #include "BLI_listbase.h"
 #include "BLI_utildefines.h"
 
+#include "BLT_translation.h"
+
 #include "DNA_armature_types.h"
 #include "DNA_lamp_types.h"
 #include "DNA_material_types.h"
 #include "DNA_node_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_world_types.h"
-#include "DNA_speaker_types.h"
 #include "DNA_brush_types.h"
+#include "DNA_linestyle_types.h"
 
 #include "BKE_context.h"
 #include "BKE_action.h"
@@ -53,9 +55,11 @@
 #include "BKE_particle.h"
 #include "BKE_screen.h"
 #include "BKE_texture.h"
+#include "BKE_linestyle.h"
 
 #include "RNA_access.h"
 
+#include "ED_buttons.h"
 #include "ED_armature.h"
 #include "ED_screen.h"
 #include "ED_physics.h"
@@ -138,6 +142,30 @@ static int buttons_context_path_world(ButsContextPath *path)
 	return 0;
 }
 
+static int buttons_context_path_linestyle(ButsContextPath *path)
+{
+	Scene *scene;
+	FreestyleLineStyle *linestyle;
+	PointerRNA *ptr = &path->ptr[path->len - 1];
+
+	/* if we already have a (pinned) linestyle, we're done */
+	if (RNA_struct_is_a(ptr->type, &RNA_FreestyleLineStyle)) {
+		return 1;
+	}
+	/* if we have a scene, use the lineset's linestyle */
+	else if (buttons_context_path_scene(path)) {
+		scene = path->ptr[path->len - 1].data;
+		linestyle = BKE_linestyle_active_from_scene(scene);
+		if (linestyle) {
+			RNA_id_pointer_create(&linestyle->id, &path->ptr[path->len]);
+			path->len++;
+			return 1;
+		}
+	}
+
+	/* no path to a linestyle possible */
+	return 0;
+}
 
 static int buttons_context_path_object(ButsContextPath *path)
 {
@@ -173,7 +201,7 @@ static int buttons_context_path_data(ButsContextPath *path, int type)
 
 	/* if we already have a data, we're done */
 	if (RNA_struct_is_a(ptr->type, &RNA_Mesh) && (type == -1 || type == OB_MESH)) return 1;
-	else if (RNA_struct_is_a(ptr->type, &RNA_Curve) && (type == -1 || ELEM3(type, OB_CURVE, OB_SURF, OB_FONT))) return 1;
+	else if (RNA_struct_is_a(ptr->type, &RNA_Curve) && (type == -1 || ELEM(type, OB_CURVE, OB_SURF, OB_FONT))) return 1;
 	else if (RNA_struct_is_a(ptr->type, &RNA_Armature) && (type == -1 || type == OB_ARMATURE)) return 1;
 	else if (RNA_struct_is_a(ptr->type, &RNA_MetaBall) && (type == -1 || type == OB_MBALL)) return 1;
 	else if (RNA_struct_is_a(ptr->type, &RNA_Lattice) && (type == -1 || type == OB_LATTICE)) return 1;
@@ -203,14 +231,14 @@ static int buttons_context_path_modifier(ButsContextPath *path)
 	if (buttons_context_path_object(path)) {
 		ob = path->ptr[path->len - 1].data;
 
-		if (ob && ELEM5(ob->type, OB_MESH, OB_CURVE, OB_FONT, OB_SURF, OB_LATTICE))
+		if (ob && ELEM(ob->type, OB_MESH, OB_CURVE, OB_FONT, OB_SURF, OB_LATTICE))
 			return 1;
 	}
 
 	return 0;
 }
 
-static int buttons_context_path_material(ButsContextPath *path, int for_texture)
+static int buttons_context_path_material(ButsContextPath *path, bool for_texture, bool new_shading)
 {
 	Object *ob;
 	PointerRNA *ptr = &path->ptr[path->len - 1];
@@ -231,12 +259,15 @@ static int buttons_context_path_material(ButsContextPath *path, int for_texture)
 
 			if (for_texture && give_current_material_texture_node(ma))
 				return 1;
-			
-			ma = give_node_material(ma);
-			if (ma) {
-				RNA_id_pointer_create(&ma->id, &path->ptr[path->len]);
-				path->len++;
-			}			
+
+			if (!new_shading) {
+				/* Only try to get mat from node in case of old shading system (see T40331). */
+				ma = give_node_material(ma);
+				if (ma) {
+					RNA_id_pointer_create(&ma->id, &path->ptr[path->len]);
+					path->len++;
+				}
+			}
 			return 1;
 		}
 	}
@@ -351,7 +382,7 @@ static int buttons_context_path_brush(ButsContextPath *path)
 		scene = path->ptr[path->len - 1].data;
 
 		if (scene)
-			br = paint_brush(paint_get_active(scene));
+			br = BKE_paint_brush(BKE_paint_get_active(scene));
 
 		if (br) {
 			RNA_id_pointer_create((ID *)br, &path->ptr[path->len]);
@@ -385,7 +416,7 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 			if (GS(id->name) == ID_BR)
 				buttons_context_path_brush(path);
 			else if (GS(id->name) == ID_MA)
-				buttons_context_path_material(path, 0);
+				buttons_context_path_material(path, false, true);
 			else if (GS(id->name) == ID_WO)
 				buttons_context_path_world(path);
 			else if (GS(id->name) == ID_LA)
@@ -394,8 +425,10 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 				buttons_context_path_particle(path);
 			else if (GS(id->name) == ID_OB)
 				buttons_context_path_object(path);
+			else if (GS(id->name) == ID_LS)
+				buttons_context_path_linestyle(path);
 		}
-		
+
 		if (ct->texture) {
 			RNA_id_pointer_create(&ct->texture->id, &path->ptr[path->len]);
 			path->len++;
@@ -407,31 +440,18 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 		/* old shading system */
 		Material *ma;
 		Lamp *la;
-		Brush *br;
 		World *wo;
 		ParticleSystem *psys;
+		FreestyleLineStyle *ls;
 		Tex *tex;
 		PointerRNA *ptr = &path->ptr[path->len - 1];
-		int orig_len = path->len;
 
 		/* if we already have a (pinned) texture, we're done */
 		if (RNA_struct_is_a(ptr->type, &RNA_Texture)) {
 			return 1;
 		}
-		/* try brush */
-		if ((path->tex_ctx == SB_TEXC_BRUSH) && buttons_context_path_brush(path)) {
-			br = path->ptr[path->len - 1].data;
-			
-			if (br) {
-				tex = give_current_brush_texture(br);
-
-				RNA_id_pointer_create(&tex->id, &path->ptr[path->len]);
-				path->len++;
-				return 1;
-			}
-		}
 		/* try world */
-		if ((path->tex_ctx == SB_TEXC_WORLD) && buttons_context_path_world(path)) {
+		else if ((path->tex_ctx == SB_TEXC_WORLD) && buttons_context_path_world(path)) {
 			wo = path->ptr[path->len - 1].data;
 
 			if (wo && GS(wo->id.name) == ID_WO) {
@@ -443,7 +463,7 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 			}
 		}
 		/* try particles */
-		if ((path->tex_ctx == SB_TEXC_PARTICLES) && buttons_context_path_particle(path)) {
+		else if ((path->tex_ctx == SB_TEXC_PARTICLES) && buttons_context_path_particle(path)) {
 			if (path->ptr[path->len - 1].type == &RNA_ParticleSettings) {
 				ParticleSettings *part = path->ptr[path->len - 1].data;
 
@@ -465,7 +485,7 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 			}
 		}
 		/* try material */
-		if (buttons_context_path_material(path, 1)) {
+		else if ((path->tex_ctx == SB_TEXC_MATERIAL) && buttons_context_path_material(path, true, false)) {
 			ma = path->ptr[path->len - 1].data;
 
 			if (ma) {
@@ -477,7 +497,7 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 			}
 		}
 		/* try lamp */
-		if (buttons_context_path_data(path, OB_LAMP)) {
+		else if ((path->tex_ctx == SB_TEXC_LAMP) && buttons_context_path_data(path, OB_LAMP)) {
 			la = path->ptr[path->len - 1].data;
 
 			if (la) {
@@ -488,14 +508,13 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 				return 1;
 			}
 		}
-		/* try brushes again in case of no material, lamp, etc */
-		path->len = orig_len;
-		if (buttons_context_path_brush(path)) {
-			br = path->ptr[path->len - 1].data;
-			
-			if (br) {
-				tex = give_current_brush_texture(br);
-				
+		/* try linestyle */
+		else if ((path->tex_ctx == SB_TEXC_LINESTYLE) && buttons_context_path_linestyle(path)) {
+			ls = path->ptr[path->len - 1].data;
+
+			if (ls) {
+				tex = give_current_linestyle_texture(ls);
+
 				RNA_id_pointer_create(&tex->id, &path->ptr[path->len]);
 				path->len++;
 				return 1;
@@ -507,6 +526,32 @@ static int buttons_context_path_texture(ButsContextPath *path, ButsContextTextur
 	return 0;
 }
 
+#ifdef WITH_FREESTYLE
+static bool buttons_context_linestyle_pinnable(const bContext *C)
+{
+	Scene *scene = CTX_data_scene(C);
+	SceneRenderLayer *actsrl;
+	FreestyleConfig *config;
+	SpaceButs *sbuts;
+
+	/* if Freestyle is disabled in the scene */
+	if ((scene->r.mode & R_EDGE_FRS) == 0) {
+		return false;
+	}
+	/* if Freestyle is not in the Parameter Editor mode */
+	actsrl = BLI_findlink(&scene->r.layers, scene->r.actlay);
+	config = &actsrl->freestyleConfig;
+	if (config->mode != FREESTYLE_CONTROL_EDITOR_MODE) {
+		return false;
+	}
+	/* if the scene has already been pinned */
+	sbuts = CTX_wm_space_buts(C);
+	if (sbuts->pinid && sbuts->pinid == &scene->id) {
+		return false;
+	}
+	return true;
+}
+#endif
 
 static int buttons_context_path(const bContext *C, ButsContextPath *path, int mainb, int flag)
 {
@@ -540,6 +585,17 @@ static int buttons_context_path(const bContext *C, ButsContextPath *path, int ma
 		case BCONTEXT_RENDER:
 			found = buttons_context_path_scene(path);
 			break;
+		case BCONTEXT_RENDER_LAYER:
+#ifdef WITH_FREESTYLE
+			if (buttons_context_linestyle_pinnable(C)) {
+				found = buttons_context_path_linestyle(path);
+				if (found) {
+					break;
+				}
+			}
+#endif
+			found = buttons_context_path_scene(path);
+			break;
 		case BCONTEXT_WORLD:
 			found = buttons_context_path_world(path);
 			break;
@@ -558,7 +614,7 @@ static int buttons_context_path(const bContext *C, ButsContextPath *path, int ma
 			found = buttons_context_path_particle(path);
 			break;
 		case BCONTEXT_MATERIAL:
-			found = buttons_context_path_material(path, 0);
+			found = buttons_context_path_material(path, false, (sbuts->texuser != NULL));
 			break;
 		case BCONTEXT_TEXTURE:
 			found = buttons_context_path_texture(path, sbuts->texuser);
@@ -583,7 +639,7 @@ static int buttons_shading_context(const bContext *C, int mainb)
 {
 	Object *ob = CTX_data_active_object(C);
 
-	if (ELEM3(mainb, BCONTEXT_MATERIAL, BCONTEXT_WORLD, BCONTEXT_TEXTURE))
+	if (ELEM(mainb, BCONTEXT_MATERIAL, BCONTEXT_WORLD, BCONTEXT_TEXTURE))
 		return 1;
 	if (mainb == BCONTEXT_DATA && ob && ELEM(ob->type, OB_LAMP, OB_CAMERA))
 		return 1;
@@ -611,13 +667,16 @@ void buttons_context_compute(const bContext *C, SpaceButs *sbuts)
 	PointerRNA *ptr;
 	int a, pflag = 0, flag = 0;
 
-	buttons_texture_context_compute(C, sbuts);
-
 	if (!sbuts->path)
 		sbuts->path = MEM_callocN(sizeof(ButsContextPath), "ButsContextPath");
-	
+
 	path = sbuts->path;
-	
+
+	/* We need to set Scene path now! Else, buttons_texture_context_compute() might not get a valid scene. */
+	buttons_context_path(C, path, BCONTEXT_SCENE, pflag);
+
+	buttons_texture_context_compute(C, sbuts);
+
 	/* for each context, see if we can compute a valid path to it, if
 	 * this is the case, we know we have to display the button */
 	for (a = 0; a < BCONTEXT_TOT; a++) {
@@ -680,11 +739,12 @@ void buttons_context_compute(const bContext *C, SpaceButs *sbuts)
 /************************* Context Callback ************************/
 
 const char *buttons_context_dir[] = {
-	"world", "object", "mesh", "armature", "lattice", "curve",
+	"texture_slot", "scene", "world", "object", "mesh", "armature", "lattice", "curve",
 	"meta_ball", "lamp", "speaker", "camera", "material", "material_slot",
-	"texture", "texture_slot", "texture_user", "bone", "edit_bone",
+	"texture", "texture_user", "texture_user_property", "bone", "edit_bone",
 	"pose_bone", "particle_system", "particle_system_editable", "particle_settings",
-	"cloth", "soft_body", "fluid", "smoke", "collision", "brush", "dynamic_paint", NULL
+	"cloth", "soft_body", "fluid", "smoke", "collision", "brush", "dynamic_paint",
+	"line_style", NULL
 };
 
 int buttons_context(const bContext *C, const char *member, bContextDataResult *result)
@@ -697,9 +757,18 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 
 	/* here we handle context, getting data from precomputed path */
 	if (CTX_data_dir(member)) {
-		CTX_data_dir_set(result, buttons_context_dir);
+		/* in case of new shading system we skip texture_slot, complex python
+		 * UI script logic depends on checking if this is available */
+		if (sbuts->texuser)
+			CTX_data_dir_set(result, buttons_context_dir + 1);
+		else
+			CTX_data_dir_set(result, buttons_context_dir);
 		return 1;
 	}
+	else if (CTX_data_equals(member, "scene")) {
+		/* Do not return one here if scene not found in path, in this case we want to get default context scene! */
+		return set_pointer_type(path, result, &RNA_Scene);
+	}
 	else if (CTX_data_equals(member, "world")) {
 		set_pointer_type(path, result, &RNA_World);
 		return 1;
@@ -787,13 +856,27 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 
 		return 1;
 	}
+	else if (CTX_data_equals(member, "texture_user_property")) {
+		ButsContextTexture *ct = sbuts->texuser;
+
+		if (!ct)
+			return -1;  /* old shading system (found but not available) */
+
+		if (ct->user && ct->user->ptr.data) {
+			ButsTextureUser *user = ct->user;
+			CTX_data_pointer_set(result, NULL, &RNA_Property, user->prop);
+		}
+
+		return 1;
+	}
 	else if (CTX_data_equals(member, "texture_node")) {
 		ButsContextTexture *ct = sbuts->texuser;
 
 		if (ct) {
 			/* new shading system */
-			if (ct->user && ct->user->node)
+			if (ct->user && ct->user->node) {
 				CTX_data_pointer_set(result, &ct->user->ntree->id, &RNA_Node, ct->user->node);
+			}
 
 			return 1;
 		}
@@ -817,10 +900,17 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 		ButsContextTexture *ct = sbuts->texuser;
 		PointerRNA *ptr;
 
-		if (ct)
-			return 0;  /* new shading system */
+		/* Particles slots are used in both old and new textures handling. */
+		if ((ptr = get_pointer_type(path, &RNA_ParticleSystem))) {
+			ParticleSettings *part = ((ParticleSystem *)ptr->data)->part;
 
-		if ((ptr = get_pointer_type(path, &RNA_Material))) {
+			if (part)
+				CTX_data_pointer_set(result, &part->id, &RNA_ParticleSettingsTextureSlot, part->mtex[(int)part->texact]);
+		}
+		else if (ct) {
+			return 0;  /* new shading system */
+		}
+		else if ((ptr = get_pointer_type(path, &RNA_Material))) {
 			Material *ma = ptr->data;
 
 			/* if we have a node material, get slot from material in material node */
@@ -829,7 +919,7 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 				 * then that texture is in context directly, without a texture slot */
 				if (give_current_material_texture_node(ma))
 					return 0;
-				
+
 				ma = give_node_material(ma);
 				if (ma)
 					CTX_data_pointer_set(result, &ma->id, &RNA_MaterialTextureSlot, ma->mtex[(int)ma->texact]);
@@ -852,17 +942,11 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 			if (wo)
 				CTX_data_pointer_set(result, &wo->id, &RNA_WorldTextureSlot, wo->mtex[(int)wo->texact]);
 		}
-		else if ((ptr = get_pointer_type(path, &RNA_Brush))) { /* how to get this into context? */
-			Brush *br = ptr->data;
+		else if ((ptr = get_pointer_type(path, &RNA_FreestyleLineStyle))) {
+			FreestyleLineStyle *ls = ptr->data;
 
-			if (br)
-				CTX_data_pointer_set(result, &br->id, &RNA_BrushTextureSlot, &br->mtex);
-		}
-		else if ((ptr = get_pointer_type(path, &RNA_ParticleSystem))) {
-			ParticleSettings *part = ((ParticleSystem *)ptr->data)->part;
-
-			if (part)
-				CTX_data_pointer_set(result, &part->id, &RNA_ParticleSettingsTextureSlot, part->mtex[(int)part->texact]);
+			if (ls)
+				CTX_data_pointer_set(result, &ls->id, &RNA_LineStyleTextureSlot, ls->mtex[(int)ls->texact]);
 		}
 
 		return 1;
@@ -889,7 +973,7 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 		else
 			CTX_data_pointer_set(result, NULL, &RNA_ParticleSystem, NULL);
 		return 1;
-	}	
+	}
 	else if (CTX_data_equals(member, "particle_settings")) {
 		/* only available when pinned */
 		PointerRNA *ptr = get_pointer_type(path, &RNA_ParticleSettings);
@@ -900,7 +984,7 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 		}
 		else {
 			/* get settings from active particle system instead */
-			PointerRNA *ptr = get_pointer_type(path, &RNA_ParticleSystem);
+			ptr = get_pointer_type(path, &RNA_ParticleSystem);
 			
 			if (ptr && ptr->data) {
 				ParticleSettings *part = ((ParticleSystem *)ptr->data)->part;
@@ -976,6 +1060,10 @@ int buttons_context(const bContext *C, const char *member, bContextDataResult *r
 			return 1;
 		}
 	}
+	else if (CTX_data_equals(member, "line_style")) {
+		set_pointer_type(path, result, &RNA_FreestyleLineStyle);
+		return 1;
+	}
 	else {
 		return 0; /* not found */
 	}
@@ -1012,14 +1100,15 @@ void buttons_context_draw(const bContext *C, uiLayout *layout)
 	if (!path)
 		return;
 
-	row = uiLayoutRow(layout, TRUE);
+	row = uiLayoutRow(layout, true);
 	uiLayoutSetAlignment(row, UI_LAYOUT_ALIGN_LEFT);
 
 	block = uiLayoutGetBlock(row);
-	uiBlockSetEmboss(block, UI_EMBOSSN);
-	but = uiDefIconButBitC(block, ICONTOG, SB_PIN_CONTEXT, 0, ICON_UNPINNED, 0, 0, UI_UNIT_X, UI_UNIT_Y, &sbuts->flag, 0, 0, 0, 0, "Follow context or keep fixed datablock displayed");
-	uiButClearFlag(but, UI_BUT_UNDO); /* skip undo on screen buttons */
-	uiButSetFunc(but, pin_cb, NULL, NULL);
+	UI_block_emboss_set(block, UI_EMBOSS_NONE);
+	but = uiDefIconButBitC(block, UI_BTYPE_ICON_TOGGLE, SB_PIN_CONTEXT, 0, ICON_UNPINNED, 0, 0, UI_UNIT_X, UI_UNIT_Y, &sbuts->flag,
+	                       0, 0, 0, 0, TIP_("Follow context or keep fixed datablock displayed"));
+	UI_but_flag_disable(but, UI_BUT_UNDO); /* skip undo on screen buttons */
+	UI_but_func_set(but, pin_cb, NULL, NULL);
 
 	for (a = 0; a < path->len; a++) {
 		ptr = &path->ptr[a];
@@ -1032,11 +1121,11 @@ void buttons_context_draw(const bContext *C, uiLayout *layout)
 			name = RNA_struct_name_get_alloc(ptr, namebuf, sizeof(namebuf), NULL);
 
 			if (name) {
-				if (!ELEM(sbuts->mainb, BCONTEXT_RENDER, BCONTEXT_SCENE) && ptr->type == &RNA_Scene)
+				if (!ELEM(sbuts->mainb, BCONTEXT_RENDER, BCONTEXT_SCENE, BCONTEXT_RENDER_LAYER) && ptr->type == &RNA_Scene)
 					uiItemLDrag(row, ptr, "", icon);  /* save some space */
 				else
 					uiItemLDrag(row, ptr, name, icon);
-								 
+
 				if (name != namebuf)
 					MEM_freeN(name);
 			}
@@ -1057,7 +1146,8 @@ void buttons_context_register(ARegionType *art)
 
 	pt = MEM_callocN(sizeof(PanelType), "spacetype buttons panel context");
 	strcpy(pt->idname, "BUTTONS_PT_context");
-	strcpy(pt->label, "Context");
+	strcpy(pt->label, N_("Context"));  /* XXX C panels are not available through RNA (bpy.types)! */
+	strcpy(pt->translation_context, BLT_I18NCONTEXT_DEFAULT_BPYRNA);
 	pt->draw = buttons_panel_context;
 	pt->flag = PNL_NO_HEADER;
 	BLI_addtail(&art->paneltypes, pt);
@@ -1090,3 +1180,33 @@ ID *buttons_context_id_path(const bContext *C)
 
 	return NULL;
 }
+
+void ED_buttons_id_unref(SpaceButs *sbuts, const ID *id)
+{
+	if (sbuts->pinid == id) {
+		sbuts->pinid = NULL;
+		sbuts->flag &= ~SB_PIN_CONTEXT;
+	}
+
+	if (sbuts->path) {
+		ButsContextPath *path = sbuts->path;
+		int i;
+
+		for (i = 0; i < path->len; i++) {
+			if (path->ptr[i].id.data == id) {
+				break;
+			}
+		}
+
+		if (i == path->len) {
+			/* pass */
+		}
+		else if (i == 0) {
+			MEM_SAFE_FREE(sbuts->path);
+		}
+		else {
+			memset(&path->ptr[i], 0, sizeof(path->ptr[i]) * (path->len - i));
+			path->len = i;
+		}
+	}
+}
