@@ -102,6 +102,8 @@ short ED_fileselect_set_params(SpaceFile *sfile)
 		/* set path to most recently opened .blend */
 		BLI_split_dirfile(G.main->name, sfile->params->dir, sfile->params->file, sizeof(sfile->params->dir), sizeof(sfile->params->file));
 		sfile->params->filter_glob[0] = '\0';
+		/* set the default thumbnails size */
+		sfile->params->thumbnail_size = 128;
 	}
 
 	params = sfile->params;
@@ -117,10 +119,12 @@ short ED_fileselect_set_params(SpaceFile *sfile)
 
 		BLI_strncpy_utf8(params->title, RNA_struct_ui_name(op->type->srna), sizeof(params->title));
 
-		if (RNA_struct_find_property(op->ptr, "filemode"))
-			params->type = RNA_int_get(op->ptr, "filemode");
-		else
+		if ((prop = RNA_struct_find_property(op->ptr, "filemode"))) {
+			params->type = RNA_property_int_get(op->ptr, prop);
+		}
+		else {
 			params->type = FILE_SPECIAL;
+		}
 
 		if (is_filepath && RNA_struct_property_is_set_ex(op->ptr, "filepath", false)) {
 			char name[FILE_MAX];
@@ -158,30 +162,32 @@ short ED_fileselect_set_params(SpaceFile *sfile)
 
 		params->filter = 0;
 		if ((prop = RNA_struct_find_property(op->ptr, "filter_blender")))
-			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? BLENDERFILE : 0;
+			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? FILE_TYPE_BLENDER : 0;
+		if ((prop = RNA_struct_find_property(op->ptr, "filter_blenlib")))
+			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? FILE_TYPE_BLENDERLIB : 0;
 		if ((prop = RNA_struct_find_property(op->ptr, "filter_backup")))
-			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? BLENDERFILE_BACKUP : 0;
+			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? FILE_TYPE_BLENDER_BACKUP : 0;
 		if ((prop = RNA_struct_find_property(op->ptr, "filter_image")))
-			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? IMAGEFILE : 0;
+			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? FILE_TYPE_IMAGE : 0;
 		if ((prop = RNA_struct_find_property(op->ptr, "filter_movie")))
-			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? MOVIEFILE : 0;
+			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? FILE_TYPE_MOVIE : 0;
 		if ((prop = RNA_struct_find_property(op->ptr, "filter_python")))
-			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? PYSCRIPTFILE : 0;
+			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? FILE_TYPE_PYSCRIPT : 0;
 		if ((prop = RNA_struct_find_property(op->ptr, "filter_font")))
-			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? FTFONTFILE : 0;
+			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? FILE_TYPE_FTFONT : 0;
 		if ((prop = RNA_struct_find_property(op->ptr, "filter_sound")))
-			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? SOUNDFILE : 0;
+			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? FILE_TYPE_SOUND : 0;
 		if ((prop = RNA_struct_find_property(op->ptr, "filter_text")))
-			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? TEXTFILE : 0;
+			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? FILE_TYPE_TEXT : 0;
 		if ((prop = RNA_struct_find_property(op->ptr, "filter_folder")))
-			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? FOLDERFILE : 0;
+			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? FILE_TYPE_FOLDER : 0;
 		if ((prop = RNA_struct_find_property(op->ptr, "filter_btx")))
-			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? BTXFILE : 0;
+			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? FILE_TYPE_BTX : 0;
 		if ((prop = RNA_struct_find_property(op->ptr, "filter_collada")))
-			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? COLLADAFILE : 0;
+			params->filter |= RNA_property_boolean_get(op->ptr, prop) ? FILE_TYPE_COLLADA : 0;
 		if ((prop = RNA_struct_find_property(op->ptr, "filter_glob"))) {
 			RNA_property_string_get(op->ptr, prop, params->filter_glob);
-			params->filter |= (OPERATORFILE | FOLDERFILE);
+			params->filter |= (FILE_TYPE_OPERATOR | FILE_TYPE_FOLDER);
 		}
 		else {
 			params->filter_glob[0] = '\0';
@@ -196,6 +202,13 @@ short ED_fileselect_set_params(SpaceFile *sfile)
 			}
 		}
 
+		/* For now, always init filterid to 'all true' */
+		params->filter_id = FILTER_ID_AC | FILTER_ID_AR | FILTER_ID_BR | FILTER_ID_CA | FILTER_ID_CU | FILTER_ID_GD |
+		                    FILTER_ID_GR | FILTER_ID_IM | FILTER_ID_LA | FILTER_ID_LS | FILTER_ID_LT | FILTER_ID_MA |
+		                    FILTER_ID_MB | FILTER_ID_MC | FILTER_ID_ME | FILTER_ID_MSK | FILTER_ID_NT | FILTER_ID_OB |
+		                    FILTER_ID_PA | FILTER_ID_PAL | FILTER_ID_PC | FILTER_ID_SCE | FILTER_ID_SPK | FILTER_ID_SO |
+		                    FILTER_ID_TE | FILTER_ID_TXT | FILTER_ID_VF | FILTER_ID_WO;
+
 		if (U.uiflag & USER_HIDE_DOT) {
 			params->flag |= FILE_HIDE_DOT;
 		}
@@ -210,12 +223,20 @@ short ED_fileselect_set_params(SpaceFile *sfile)
 			params->flag |= RNA_boolean_get(op->ptr, "active_layer") ? FILE_ACTIVELAY : 0;
 		}
 
-		if (RNA_struct_find_property(op->ptr, "display_type"))
-			params->display = RNA_enum_get(op->ptr, "display_type");
+		if ((prop = RNA_struct_find_property(op->ptr, "display_type"))) {
+			params->display = RNA_property_enum_get(op->ptr, prop);
+		}
+
+		if ((prop = RNA_struct_find_property(op->ptr, "sort_method"))) {
+			params->sort = RNA_property_enum_get(op->ptr, prop);
+		}
+		else {
+			params->sort = FILE_SORT_ALPHA;
+		}
 
 		if (params->display == FILE_DEFAULTDISPLAY) {
 			if (U.uiflag & USER_SHOW_THUMBNAILS) {
-				if (params->filter & (IMAGEFILE | MOVIEFILE))
+				if (params->filter & (FILE_TYPE_IMAGE | FILE_TYPE_MOVIE))
 					params->display = FILE_IMGDISPLAY;
 				else
 					params->display = FILE_SHORTDISPLAY;
@@ -226,8 +247,10 @@ short ED_fileselect_set_params(SpaceFile *sfile)
 		}
 
 		if (is_relative_path) {
-			if (!RNA_struct_property_is_set_ex(op->ptr, "relative_path", false)) {
-				RNA_boolean_set(op->ptr, "relative_path", U.flag & USER_RELPATHS);
+			if ((prop = RNA_struct_find_property(op->ptr, "relative_path"))) {
+				if (!RNA_property_is_set_ex(op->ptr, prop, false)) {
+					RNA_property_boolean_set(op->ptr, prop, (U.flag & USER_RELPATHS) != 0);
+				}
 			}
 		}
 	}
@@ -237,12 +260,13 @@ short ED_fileselect_set_params(SpaceFile *sfile)
 		params->flag |= FILE_HIDE_DOT;
 		params->flag &= ~FILE_DIRSEL_ONLY;
 		params->display = FILE_SHORTDISPLAY;
+		params->sort = FILE_SORT_ALPHA;
 		params->filter = 0;
 		params->filter_glob[0] = '\0';
 	}
 
 	/* operator has no setting for this */
-	params->sort = FILE_SORT_ALPHA;
+	params->active_file = -1;
 
 
 	/* initialize the list with previous folders */
@@ -276,6 +300,18 @@ void ED_fileselect_reset_params(SpaceFile *sfile)
 	sfile->params->type = FILE_UNIX;
 	sfile->params->flag = 0;
 	sfile->params->title[0] = '\0';
+	sfile->params->active_file = -1;
+}
+
+/**
+ * Sets FileSelectParams->file (name of selected file)
+ */
+void fileselect_file_set(SpaceFile *sfile, const int index)
+{
+	const struct FileDirEntry *file = filelist_file(sfile->files, index);
+	if (file && file->relpath && file->relpath[0] && !(file->typeflag & FILE_TYPE_FOLDER)) {
+		BLI_strncpy(sfile->params->file, file->relpath, FILE_MAXFILE);
+	}
 }
 
 int ED_fileselect_layout_numfiles(FileLayout *layout, ARegion *ar)
@@ -391,63 +427,23 @@ void ED_fileselect_layout_tilepos(FileLayout *layout, int tile, int *x, int *y)
 	}
 }
 
-/* Shorten a string to a given width w. 
- * If front is set, shorten from the front,
- * otherwise shorten from the end. */
-float file_shorten_string(char *string, float w, int front)
-{	
-	char temp[FILE_MAX];
-	short shortened = 0;
-	float sw = 0;
-	float pad = 0;
-
-	if (w <= 0) {
-		*string = '\0';
-		return 0.0;
-	}
-
-	sw = file_string_width(string);
-	if (front == 1) {
-		const char *s = string;
-		BLI_strncpy(temp, "...", 4);
-		pad = file_string_width(temp);
-		while ((*s) && (sw + pad > w)) {
-			s++;
-			sw = file_string_width(s);
-			shortened = 1;
-		}
-		if (shortened) {
-			int slen = strlen(s);
-			BLI_strncpy(temp + 3, s, slen + 1);
-			temp[slen + 4] = '\0';
-			BLI_strncpy(string, temp, slen + 4);
-		}
-	}
-	else {
-		const char *s = string;
-		while (sw > w) {
-			int slen = strlen(string);
-			string[slen - 1] = '\0';
-			sw = file_string_width(s);
-			shortened = 1;
-		}
-
-		if (shortened) {
-			int slen = strlen(string);
-			if (slen > 3) {
-				BLI_strncpy(string + slen - 3, "...", 4);
-			}
-		}
-	}
-	
-	return sw;
-}
-
 float file_string_width(const char *str)
 {
 	uiStyle *style = UI_style_get();
+	float width;
+
 	UI_fontstyle_set(&style->widget);
-	return BLF_width(style->widget.uifont_id, str, BLF_DRAW_STR_DUMMY_MAX);
+	if (style->widget.kerning == 1) {  /* for BLF_width */
+		BLF_enable(style->widget.uifont_id, BLF_KERNING_DEFAULT);
+	}
+
+	width = BLF_width(style->widget.uifont_id, str, BLF_DRAW_STR_DUMMY_MAX);
+
+	if (style->widget.kerning == 1) {
+		BLF_disable(style->widget.uifont_id, BLF_KERNING_DEFAULT);
+	}
+
+	return width;
 }
 
 float file_font_pointsize(void)
@@ -466,37 +462,20 @@ float file_font_pointsize(void)
 #endif
 }
 
-static void column_widths(struct FileList *files, struct FileLayout *layout)
+static void column_widths(FileSelectParams *params, struct FileLayout *layout)
 {
 	int i;
-	int numfiles = filelist_numfiles(files);
+	const bool small_size = SMALL_SIZE_CHECK(params->thumbnail_size);
 
 	for (i = 0; i < MAX_FILE_COLUMN; ++i) {
 		layout->column_widths[i] = 0;
 	}
 
-	for (i = 0; (i < numfiles); ++i) {
-		struct direntry *file = filelist_file(files, i);
-		if (file) {
-			float len;
-			len = file_string_width(file->relname);
-			if (len > layout->column_widths[COLUMN_NAME]) layout->column_widths[COLUMN_NAME] = len;
-			len = file_string_width(file->date);
-			if (len > layout->column_widths[COLUMN_DATE]) layout->column_widths[COLUMN_DATE] = len;
-			len = file_string_width(file->time);
-			if (len > layout->column_widths[COLUMN_TIME]) layout->column_widths[COLUMN_TIME] = len;
-			len = file_string_width(file->size);
-			if (len > layout->column_widths[COLUMN_SIZE]) layout->column_widths[COLUMN_SIZE] = len;
-			len = file_string_width(file->mode1);
-			if (len > layout->column_widths[COLUMN_MODE1]) layout->column_widths[COLUMN_MODE1] = len;
-			len = file_string_width(file->mode2);
-			if (len > layout->column_widths[COLUMN_MODE2]) layout->column_widths[COLUMN_MODE2] = len;
-			len = file_string_width(file->mode3);
-			if (len > layout->column_widths[COLUMN_MODE3]) layout->column_widths[COLUMN_MODE3] = len;
-			len = file_string_width(file->owner);
-			if (len > layout->column_widths[COLUMN_OWNER]) layout->column_widths[COLUMN_OWNER] = len;
-		}
-	}
+	layout->column_widths[COLUMN_NAME] = ((float)params->thumbnail_size / 8.0f) * UI_UNIT_X;;
+	/* Biggest possible reasonable values... */
+	layout->column_widths[COLUMN_DATE] = file_string_width(small_size ? "23/08/89" : "23-Dec-89");
+	layout->column_widths[COLUMN_TIME] = file_string_width("23:59");
+	layout->column_widths[COLUMN_SIZE] = file_string_width(small_size ? "98.7 M" : "98.7 MiB");
 }
 
 void ED_fileselect_init_layout(struct SpaceFile *sfile, ARegion *ar)
@@ -516,14 +495,14 @@ void ED_fileselect_init_layout(struct SpaceFile *sfile, ARegion *ar)
 		return;
 	}
 
-	numfiles = filelist_numfiles(sfile->files);
+	numfiles = filelist_files_ensure(sfile->files);
 	textheight = (int)file_font_pointsize();
 	layout = sfile->layout;
 	layout->textheight = textheight;
 
 	if (params->display == FILE_IMGDISPLAY) {
-		layout->prv_w = 4.8f * UI_UNIT_X;
-		layout->prv_h = 4.8f * UI_UNIT_Y;
+		layout->prv_w = ((float)params->thumbnail_size / 20.0f) * UI_UNIT_X;
+		layout->prv_h = ((float)params->thumbnail_size / 20.0f) * UI_UNIT_Y;
 		layout->tile_border_x = 0.3f * UI_UNIT_X;
 		layout->tile_border_y = 0.3f * UI_UNIT_X;
 		layout->prv_border_x = 0.3f * UI_UNIT_X;
@@ -555,7 +534,7 @@ void ED_fileselect_init_layout(struct SpaceFile *sfile, ARegion *ar)
 		layout->height = (int)(BLI_rctf_size_y(&v2d->cur) - 2 * layout->tile_border_y);
 		layout->rows = layout->height / (layout->tile_h + 2 * layout->tile_border_y);
 
-		column_widths(sfile->files, layout);
+		column_widths(params, layout);
 
 		if (params->display == FILE_SHORTDISPLAY) {
 			maxlen = ICON_DEFAULT_WIDTH_SCALE + column_icon_space +
@@ -565,12 +544,6 @@ void ED_fileselect_init_layout(struct SpaceFile *sfile, ARegion *ar)
 		else {
 			maxlen = ICON_DEFAULT_WIDTH_SCALE + column_icon_space +
 			         (int)layout->column_widths[COLUMN_NAME] + column_space +
-#ifndef WIN32
-			         (int)layout->column_widths[COLUMN_MODE1] + column_space +
-			         (int)layout->column_widths[COLUMN_MODE2] + column_space +
-			         (int)layout->column_widths[COLUMN_MODE3] + column_space +
-			         (int)layout->column_widths[COLUMN_OWNER] + column_space +
-#endif
 			         (int)layout->column_widths[COLUMN_DATE] + column_space +
 			         (int)layout->column_widths[COLUMN_TIME] + column_space +
 			         (int)layout->column_widths[COLUMN_SIZE] + column_space;
@@ -597,27 +570,32 @@ FileLayout *ED_fileselect_get_layout(struct SpaceFile *sfile, ARegion *ar)
 	return sfile->layout;
 }
 
-void file_change_dir(bContext *C, int checkdir)
+void ED_file_change_dir(bContext *C, const bool checkdir)
 {
 	wmWindowManager *wm = CTX_wm_manager(C);
 	SpaceFile *sfile = CTX_wm_space_file(C);
+	ScrArea *sa = CTX_wm_area(C);
 
 	if (sfile->params) {
+		ED_fileselect_clear(wm, sa, sfile);
 
-		ED_fileselect_clear(wm, sfile);
+		/* Clear search string, it is very rare to want to keep that filter while changing dir,
+		 * and usually very annoying to keep it actually! */
+		sfile->params->filter_search[0] = '\0';
+		sfile->params->active_file = -1;
 
 		if (checkdir && !BLI_is_dir(sfile->params->dir)) {
 			BLI_strncpy(sfile->params->dir, filelist_dir(sfile->files), sizeof(sfile->params->dir));
 			/* could return but just refresh the current dir */
 		}
 		filelist_setdir(sfile->files, sfile->params->dir);
-		
+
 		if (folderlist_clear_next(sfile))
 			folderlist_free(sfile->folders_next);
 
 		folderlist_pushdir(sfile->folders_prev, sfile->params->dir);
 
-		file_draw_check_cb(C, NULL, NULL);
+		file_draw_check(C);
 	}
 }
 
@@ -626,18 +604,19 @@ int file_select_match(struct SpaceFile *sfile, const char *pattern, char *matche
 	int match = 0;
 	
 	int i;
-	struct direntry *file;
-	int n = filelist_numfiles(sfile->files);
+	FileDirEntry *file;
+	int n = filelist_files_ensure(sfile->files);
 
 	/* select any file that matches the pattern, this includes exact match 
 	 * if the user selects a single file by entering the filename
 	 */
 	for (i = 0; i < n; i++) {
 		file = filelist_file(sfile->files, i);
-		if (fnmatch(pattern, file->relname, 0) == 0) {
-			file->selflag |= SELECTED_FILE;
+		/* Do not check wether file is a file or dir here! Causes T44243 (we do accept dirs at this stage). */
+		if (fnmatch(pattern, file->relpath, 0) == 0) {
+			filelist_entry_select_set(sfile->files, file, FILE_SEL_ADD, FILE_SEL_SELECTED, CHECK_ALL);
 			if (!match) {
-				BLI_strncpy(matched_file, file->relname, FILE_MAX);
+				BLI_strncpy(matched_file, file->relpath, FILE_MAX);
 			}
 			match++;
 		}
@@ -666,7 +645,7 @@ int autocomplete_directory(struct bContext *C, char *str, void *UNUSED(arg_v))
 			AutoComplete *autocpl = UI_autocomplete_begin(str, FILE_MAX);
 
 			while ((de = readdir(dir)) != NULL) {
-				if (strcmp(".", de->d_name) == 0 || strcmp("..", de->d_name) == 0) {
+				if (FILENAME_IS_CURRPAR(de->d_name)) {
 					/* pass */
 				}
 				else {
@@ -685,13 +664,8 @@ int autocomplete_directory(struct bContext *C, char *str, void *UNUSED(arg_v))
 			closedir(dir);
 
 			match = UI_autocomplete_end(autocpl, str);
-			if (match) {
-				if (match == AUTOCOMPLETE_FULL_MATCH) {
-					BLI_add_slash(str);
-				}
-				else {
-					BLI_strncpy(sfile->params->dir, str, sizeof(sfile->params->dir));
-				}
+			if (match == AUTOCOMPLETE_FULL_MATCH) {
+				BLI_add_slash(str);
 			}
 		}
 	}
@@ -707,14 +681,12 @@ int autocomplete_file(struct bContext *C, char *str, void *UNUSED(arg_v))
 	/* search if str matches the beginning of name */
 	if (str[0] && sfile->files) {
 		AutoComplete *autocpl = UI_autocomplete_begin(str, FILE_MAX);
-		int nentries = filelist_numfiles(sfile->files);
+		int nentries = filelist_files_ensure(sfile->files);
 		int i;
 
 		for (i = 0; i < nentries; ++i) {
-			struct direntry *file = filelist_file(sfile->files, i);
-			if (file && (S_ISREG(file->type) || S_ISDIR(file->type))) {
-				UI_autocomplete_update_name(autocpl, file->relname);
-			}
+			FileDirEntry *file = filelist_file(sfile->files, i);
+			UI_autocomplete_update_name(autocpl, file->relpath);
 		}
 		match = UI_autocomplete_end(autocpl, str);
 	}
@@ -722,20 +694,20 @@ int autocomplete_file(struct bContext *C, char *str, void *UNUSED(arg_v))
 	return match;
 }
 
-void ED_fileselect_clear(struct wmWindowManager *wm, struct SpaceFile *sfile)
+void ED_fileselect_clear(wmWindowManager *wm, ScrArea *sa, SpaceFile *sfile)
 {
 	/* only NULL in rare cases - [#29734] */
 	if (sfile->files) {
-		thumbnails_stop(wm, sfile->files);
+		filelist_readjob_stop(wm, sa);
 		filelist_freelib(sfile->files);
-		filelist_free(sfile->files);
+		filelist_clear(sfile->files);
 	}
 
-	sfile->params->active_file = -1;
+	sfile->params->highlight_file = -1;
 	WM_main_add_notifier(NC_SPACE | ND_SPACE_FILE_LIST, NULL);
 }
 
-void ED_fileselect_exit(struct wmWindowManager *wm, struct SpaceFile *sfile)
+void ED_fileselect_exit(wmWindowManager *wm, ScrArea *sa, SpaceFile *sfile)
 {
 	if (!sfile) return;
 	if (sfile->op) {
@@ -747,7 +719,8 @@ void ED_fileselect_exit(struct wmWindowManager *wm, struct SpaceFile *sfile)
 	folderlist_free(sfile->folders_next);
 	
 	if (sfile->files) {
-		ED_fileselect_clear(wm, sfile);
+		ED_fileselect_clear(wm, sa, sfile);
+		filelist_free(sfile->files);
 		MEM_freeN(sfile->files);
 		sfile->files = NULL;
 	}
