@@ -39,7 +39,6 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_math.h"
-#include "BLI_rand.h"
 #include "BLI_utildefines.h"
 
 #include "BKE_context.h"
@@ -183,14 +182,17 @@ static void graph_init(struct wmWindowManager *UNUSED(wm), ScrArea *sa)
 {
 	SpaceIpo *sipo = (SpaceIpo *)sa->spacedata.first;
 	
-	/* init dopesheet data if non-existant (i.e. for old files) */
+	/* init dopesheet data if non-existent (i.e. for old files) */
 	if (sipo->ads == NULL) {
 		sipo->ads = MEM_callocN(sizeof(bDopeSheet), "GraphEdit DopeSheet");
 		sipo->ads->source = (ID *)(G.main->scene.first); // FIXME: this is a really nasty hack here for now...
 	}
 	
 	/* force immediate init of any invalid F-Curve colors */
-	sipo->flag |= SIPO_TEMP_NEEDCHANSYNC;
+	/* XXX: but, don't do SIPO_TEMP_NEEDCHANSYNC (i.e. channel select state sync)
+	 * as this is run on each region resize; setting this here will cause selection
+	 * state to be lost on area/region resizing. [#35744]
+	 */
 	ED_area_tag_refresh(sa);
 }
 
@@ -242,6 +244,8 @@ static void graph_main_area_draw(const bContext *C, ARegion *ar)
 	grid = UI_view2d_grid_calc(CTX_data_scene(C), v2d, unitx, V2D_GRID_NOCLAMP, unity, V2D_GRID_NOCLAMP, ar->winx, ar->winy);
 	UI_view2d_grid_draw(v2d, grid, V2D_GRIDLINES_ALL);
 	
+	ED_region_draw_cb_draw(C, ar, REGION_DRAW_PRE_VIEW);
+
 	/* draw data */
 	if (ANIM_animdata_get_context(C, &ac)) {
 		/* draw ghost curves */
@@ -252,7 +256,7 @@ static void graph_main_area_draw(const bContext *C, ARegion *ar)
 		graph_draw_curves(&ac, sipo, ar, grid, 1);
 		
 		/* XXX the slow way to set tot rect... but for nice sliders needed (ton) */
-		get_graph_keyframe_extents(&ac, &v2d->tot.xmin, &v2d->tot.xmax, &v2d->tot.ymin, &v2d->tot.ymax, FALSE, TRUE);
+		get_graph_keyframe_extents(&ac, &v2d->tot.xmin, &v2d->tot.xmax, &v2d->tot.ymin, &v2d->tot.ymax, false, true);
 		/* extra offset so that these items are visible */
 		v2d->tot.xmin -= 10.0f;
 		v2d->tot.xmax += 10.0f;
@@ -289,12 +293,16 @@ static void graph_main_area_draw(const bContext *C, ARegion *ar)
 	
 	/* markers */
 	UI_view2d_view_orthoSpecial(ar, v2d, 1);
-	draw_markers_time(C, 0);
+	ED_markers_draw(C, DRAW_MARKERS_MARGIN);
 	
 	/* preview range */
 	UI_view2d_view_ortho(v2d);
-	ANIM_draw_previewrange(C, v2d);
+	ANIM_draw_previewrange(C, v2d, 0);
 	
+	/* callback */
+	UI_view2d_view_ortho(v2d);
+	ED_region_draw_cb_draw(C, ar, REGION_DRAW_POST_VIEW);
+
 	/* reset view matrix */
 	UI_view2d_view_restore(C);
 	
@@ -309,6 +317,12 @@ static void graph_channel_area_init(wmWindowManager *wm, ARegion *ar)
 {
 	wmKeyMap *keymap;
 	
+	/* make sure we keep the hide flags */
+	ar->v2d.scroll |= (V2D_SCROLL_RIGHT | V2D_SCROLL_BOTTOM);
+	ar->v2d.scroll &= ~(V2D_SCROLL_LEFT | V2D_SCROLL_TOP);	/* prevent any noise of past */
+	ar->v2d.scroll |= V2D_SCROLL_HORIZONTAL_HIDE;
+	ar->v2d.scroll |= V2D_SCROLL_VERTICAL_HIDE;
+	
 	UI_view2d_region_reinit(&ar->v2d, V2D_COMMONVIEW_LIST, ar->winx, ar->winy);
 	
 	/* own keymap */
@@ -370,10 +384,10 @@ static void graph_buttons_area_init(wmWindowManager *wm, ARegion *ar)
 
 static void graph_buttons_area_draw(const bContext *C, ARegion *ar)
 {
-	ED_region_panels(C, ar, 1, NULL, -1);
+	ED_region_panels(C, ar, NULL, -1, true);
 }
 
-static void graph_region_listener(ARegion *ar, wmNotifier *wmn)
+static void graph_region_listener(bScreen *UNUSED(sc), ScrArea *UNUSED(sa), ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
 	switch (wmn->category) {
@@ -422,12 +436,13 @@ static void graph_region_listener(ARegion *ar, wmNotifier *wmn)
 		default:
 			if (wmn->data == ND_KEYS)
 				ED_region_tag_redraw(ar);
+			break;
 				
 	}
 }
 
 /* editor level listener */
-static void graph_listener(ScrArea *sa, wmNotifier *wmn)
+static void graph_listener(bScreen *UNUSED(sc), ScrArea *sa, wmNotifier *wmn)
 {
 	SpaceIpo *sipo = (SpaceIpo *)sa->spacedata.first;
 	
@@ -441,7 +456,7 @@ static void graph_listener(ScrArea *sa, wmNotifier *wmn)
 				ED_area_tag_refresh(sa);
 			break;
 		case NC_SCENE:
-			switch (wmn->data) {	
+			switch (wmn->data) {
 				case ND_OB_ACTIVE:  /* selection changed, so force refresh to flush (needs flag set to do syncing)  */
 				case ND_OB_SELECT:
 					sipo->flag |= SIPO_TEMP_NEEDCHANSYNC;
@@ -461,7 +476,7 @@ static void graph_listener(ScrArea *sa, wmNotifier *wmn)
 					ED_area_tag_refresh(sa);
 					break;
 				case ND_TRANSFORM:
-					break; /*do nothing*/					
+					break; /*do nothing*/
 					
 				default: /* just redrawing the view will do */
 					ED_area_tag_redraw(sa);
@@ -504,15 +519,13 @@ static void graph_refresh(const bContext *C, ScrArea *sa)
 	switch (sipo->mode) {
 		case SIPO_MODE_ANIMATION: /* all animation */
 		{
-			
+			break;
 		}
-		break;
 		
 		case SIPO_MODE_DRIVERS: /* drivers only  */
 		{
-		
+			break;
 		}
-		break;
 	}
 	
 	/* region updates? */
@@ -535,6 +548,8 @@ static void graph_refresh(const bContext *C, ScrArea *sa)
 		int filter;
 		int i;
 		
+		UI_SetTheme(SPACE_IPO, RGN_TYPE_WINDOW);
+
 		/* build list of F-Curves which will be visible as channels in channel-region
 		 *  - we don't include ANIMFILTER_CURVEVISIBLE filter, as that will result in a
 		 *    mismatch between channel-colors and the drawn curves
@@ -549,11 +564,12 @@ static void graph_refresh(const bContext *C, ScrArea *sa)
 			/* set color of curve here */
 			switch (fcu->color_mode) {
 				case FCURVE_COLOR_CUSTOM:
+				{
 					/* User has defined a custom color for this curve already (we assume it's not going to cause clashes with text colors),
 					 * which should be left alone... Nothing needs to be done here.
 					 */
 					break;
-					
+				}
 				case FCURVE_COLOR_AUTO_RGB:
 				{
 					/* F-Curve's array index is automatically mapped to RGB values. This works best of 3-value vectors. 
@@ -563,22 +579,21 @@ static void graph_refresh(const bContext *C, ScrArea *sa)
 					
 					switch (fcu->array_index) {
 						case 0:
-							col[0] = 1.0f; col[1] = 0.0f; col[2] = 0.0f;
+							UI_GetThemeColor3fv(TH_AXIS_X, col);
 							break;
 						case 1:
-							col[0] = 0.0f; col[1] = 1.0f; col[2] = 0.0f;
+							UI_GetThemeColor3fv(TH_AXIS_Y, col);
 							break;
 						case 2:
-							col[0] = 0.0f; col[1] = 0.0f; col[2] = 1.0f;
+							UI_GetThemeColor3fv(TH_AXIS_Z, col);
 							break;
 						default:
 							/* 'unknown' color - bluish so as to not conflict with handles */
 							col[0] = 0.3f; col[1] = 0.8f; col[2] = 1.0f;
 							break;
 					}
+					break;
 				}
-				break;
-				
 				case FCURVE_COLOR_AUTO_RAINBOW:
 				default:
 				{
@@ -586,13 +601,13 @@ static void graph_refresh(const bContext *C, ScrArea *sa)
 					 * of current item index + total items to determine some RGB color
 					 */
 					getcolor_fcurve_rainbow(i, items, fcu->color);
+					break;
 				}
-				break;
 			}
 		}
 		
 		/* free temp list */
-		BLI_freelistN(&anim_data);
+		ANIM_animdata_freelist(&anim_data);
 	}
 }
 
