@@ -38,12 +38,13 @@
 #include "BLI_blenlib.h"
 #include "BLI_utildefines.h"
 
-#include "BLF_translation.h"
+#include "BLT_translation.h"
 
 #include "BKE_context.h"
 #include "BKE_image.h"
 #include "BKE_node.h"
 #include "BKE_screen.h"
+#include "BKE_scene.h"
 
 #include "RE_pipeline.h"
 
@@ -64,6 +65,7 @@
 #include "image_intern.h"
 
 #define B_NOP -1
+#define MAX_IMAGE_INFO_LEN  128
 
 /* proto */
 
@@ -81,9 +83,9 @@ static void image_info(Scene *scene, ImageUser *iuser, Image *ima, ImBuf *ibuf,
 	else {
 		if (ima->source == IMA_SRC_MOVIE) {
 			ofs += BLI_strncpy_rlen(str + ofs, IFACE_("Movie"), len - ofs);
-			if (ima->anim)
+			if (BKE_image_has_anim(ima))
 				ofs += BLI_snprintf(str + ofs, len - ofs, IFACE_(" %d frs"),
-				                    IMB_anim_get_duration(ima->anim, IMB_TC_RECORD_RUN));
+				                    IMB_anim_get_duration(((ImageAnim *)ima->anims.first)->anim, IMB_TC_RECORD_RUN));
 		}
 		else {
 			ofs += BLI_strncpy_rlen(str, IFACE_("Image"), len - ofs);
@@ -310,10 +312,11 @@ static void ui_imageuser_slot_menu(bContext *UNUSED(C), uiLayout *layout, void *
 
 static const char *ui_imageuser_layer_fake_name(RenderResult *rr)
 {
-	if (rr->rectf) {
+	RenderView *rv = RE_RenderViewGetById(rr, 0);
+	if (rv->rectf) {
 		return IFACE_("Composite");
 	}
-	else if (rr->rect32) {
+	else if (rv->rect32) {
 		return IFACE_("Sequence");
 	}
 	else {
@@ -357,7 +360,7 @@ static void ui_imageuser_layer_menu(bContext *UNUSED(C), uiLayout *layout, void
 
 	for (rl = rr->layers.last; rl; rl = rl->prev, nr--) {
 final:
-		uiDefButS(block, UI_BTYPE_BUT_MENU, B_NOP, IFACE_(rl->name), 0, 0,
+		uiDefButS(block, UI_BTYPE_BUT_MENU, B_NOP, rl->name, 0, 0,
 		          UI_UNIT_X * 5, UI_UNIT_X, &iuser->layer, (float) nr, 0.0, 0, -1, "");
 	}
 
@@ -374,7 +377,7 @@ final:
 
 static const char *ui_imageuser_pass_fake_name(RenderLayer *rl)
 {
-	if (rl == NULL || rl->rectf) {
+	if (rl == NULL) {
 		return IFACE_("Combined");
 	}
 	else {
@@ -382,9 +385,9 @@ static const char *ui_imageuser_pass_fake_name(RenderLayer *rl)
 	}
 }
 
-static void ui_imageuser_pass_menu(bContext *UNUSED(C), uiLayout *layout, void *ptrpair_p)
+static void ui_imageuser_pass_menu(bContext *UNUSED(C), uiLayout *layout, void *rnd_pt)
 {
-	void **rnd_data = ptrpair_p;
+	void **rnd_data = rnd_pt;
 	uiBlock *block = uiLayoutGetBlock(layout);
 	Image *image = rnd_data[0];
 	ImageUser *iuser = rnd_data[1];
@@ -397,6 +400,7 @@ static void ui_imageuser_pass_menu(bContext *UNUSED(C), uiLayout *layout, void *
 	RenderPass *rpass;
 	const char *fake_name;
 	int nr;
+	int passflag = 0;
 
 	/* may have been freed since drawing */
 	rr = BKE_image_acquire_renderresult(scene, image);
@@ -414,47 +418,114 @@ static void ui_imageuser_pass_menu(bContext *UNUSED(C), uiLayout *layout, void *
 
 	uiItemS(layout);
 
-	nr = (rl ? BLI_listbase_count(&rl->passes) : 0) - 1;
+	nr = 0;
 	fake_name = ui_imageuser_pass_fake_name(rl);
 
 	if (fake_name) {
-		BLI_strncpy(rpass_fake.name, fake_name, sizeof(rpass_fake.name));
+		BLI_strncpy(rpass_fake.internal_name, fake_name, sizeof(rpass_fake.internal_name));
 		nr += 1;
 	}
 
 	/* rendered results don't have a Combined pass */
-	for (rpass = rl ? rl->passes.last : NULL; rpass; rpass = rpass->prev, nr--) {
+	/* multiview: the ordering must be ascending, so the left-most pass is always the one picked */
+	for (rpass = rl ? rl->passes.first : NULL; rpass; rpass = rpass->next, nr++) {
+
+		/* just show one pass of each kind */
+		if (passflag & rpass->passtype)
+			continue;
+
+		passflag |= rpass->passtype;
+
 final:
-		uiDefButS(block, UI_BTYPE_BUT_MENU, B_NOP, IFACE_(rpass->name), 0, 0,
+		uiDefButS(block, UI_BTYPE_BUT_MENU, B_NOP, IFACE_(rpass->internal_name), 0, 0,
 		          UI_UNIT_X * 5, UI_UNIT_X, &iuser->pass, (float) nr, 0.0, 0, -1, "");
 	}
 
 	if (fake_name) {
 		fake_name = NULL;
 		rpass = &rpass_fake;
+		nr = 0;
 		goto final;
 	}
 
-	BLI_assert(nr == -1);
+	BKE_image_release_renderresult(scene, image);
+}
+
+/**************************** view menus *****************************/
+static void ui_imageuser_view_menu_rr(bContext *UNUSED(C), uiLayout *layout, void *rnd_pt)
+{
+	void **rnd_data = rnd_pt;
+	uiBlock *block = uiLayoutGetBlock(layout);
+	Image *image = rnd_data[0];
+	ImageUser *iuser = rnd_data[1];
+	RenderResult *rr;
+	RenderView *rview;
+	int nr;
+	Scene *scene = iuser->scene;
+
+	/* may have been freed since drawing */
+	rr = BKE_image_acquire_renderresult(scene, image);
+	if (UNLIKELY(rr == NULL)) {
+		return;
+	}
+
+	UI_block_layout_set_current(block, layout);
+	uiLayoutColumn(layout, false);
+
+	uiDefBut(block, UI_BTYPE_LABEL, 0, IFACE_("View"),
+	         0, 0, UI_UNIT_X * 5, UI_UNIT_Y, NULL, 0.0, 0.0, 0, 0, "");
+
+	uiItemS(layout);
+
+	nr = (rr ? BLI_listbase_count(&rr->views) : 0) - 1;
+	for (rview = rr ? rr->views.last : NULL; rview; rview = rview->prev, nr--) {
+		uiDefButS(block, UI_BTYPE_BUT_MENU, B_NOP, IFACE_(rview->name), 0, 0,
+		          UI_UNIT_X * 5, UI_UNIT_X, &iuser->view, (float) nr, 0.0, 0, -1, "");
+	}
 
 	BKE_image_release_renderresult(scene, image);
 }
 
+static void ui_imageuser_view_menu_multiview(bContext *UNUSED(C), uiLayout *layout, void *rnd_pt)
+{
+	void **rnd_data = rnd_pt;
+	uiBlock *block = uiLayoutGetBlock(layout);
+	Image *image = rnd_data[0];
+	ImageUser *iuser = rnd_data[1];
+	int nr;
+	ImageView *iv;
+
+	UI_block_layout_set_current(block, layout);
+	uiLayoutColumn(layout, false);
+
+	uiDefBut(block, UI_BTYPE_LABEL, 0, IFACE_("View"),
+	         0, 0, UI_UNIT_X * 5, UI_UNIT_Y, NULL, 0.0, 0.0, 0, 0, "");
+
+	uiItemS(layout);
+
+	nr = BLI_listbase_count(&image->views) - 1;
+	for (iv = image->views.last; iv; iv = iv->prev, nr--) {
+		uiDefButS(block, UI_BTYPE_BUT_MENU, B_NOP, IFACE_(iv->name), 0, 0,
+		          UI_UNIT_X * 5, UI_UNIT_X, &iuser->view, (float) nr, 0.0, 0, -1, "");
+	}
+}
+
 /* 5 layer button callbacks... */
 static void image_multi_cb(bContext *C, void *rr_v, void *iuser_v) 
 {
 	ImageUser *iuser = iuser_v;
 
-	BKE_image_multilayer_index(rr_v, iuser); 
+	BKE_image_multilayer_index(rr_v, iuser);
 	WM_event_add_notifier(C, NC_IMAGE | ND_DRAW, NULL);
 }
+
 static void image_multi_inclay_cb(bContext *C, void *rr_v, void *iuser_v) 
 {
 	RenderResult *rr = rr_v;
 	ImageUser *iuser = iuser_v;
 	int tot = BLI_listbase_count(&rr->layers);
 
-	if (rr->rectf || rr->rect32)
+	if (RE_HasFakeLayer(rr))
 		tot++;  /* fake compo/sequencer layer */
 
 	if (iuser->layer < tot - 1) {
@@ -477,30 +548,74 @@ static void image_multi_incpass_cb(bContext *C, void *rr_v, void *iuser_v)
 {
 	RenderResult *rr = rr_v;
 	ImageUser *iuser = iuser_v;
-	RenderLayer *rl = BLI_findlink(&rr->layers, iuser->layer);
+	RenderLayer *rl;
+	RenderPass *rp;
+	int layer = iuser->layer;
+
+	if (RE_HasFakeLayer(rr))
+		layer -= 1;
+
+	rl = BLI_findlink(&rr->layers, layer);
 
 	if (rl) {
-		int tot = BLI_listbase_count(&rl->passes);
+		RenderPass *rpass = BLI_findlink(&rl->passes, iuser->pass);
+		int rp_index = iuser->pass + 1;
 
-		if (rr->rectf || rr->rect32)
-			tot++;  /* fake compo/sequencer layer */
+		if (rpass == NULL)
+			return;
 
-		if (iuser->pass < tot - 1) {
-			iuser->pass++;
-			BKE_image_multilayer_index(rr, iuser); 
-			WM_event_add_notifier(C, NC_IMAGE | ND_DRAW, NULL);
+		for (rp = rpass->next; rp; rp = rp->next, rp_index++) {
+			if (rp->passtype != rpass->passtype) {
+				iuser->pass = rp_index;
+				BKE_image_multilayer_index(rr, iuser);
+				WM_event_add_notifier(C, NC_IMAGE | ND_DRAW, NULL);
+				break;
+			}
 		}
 	}
 }
 static void image_multi_decpass_cb(bContext *C, void *rr_v, void *iuser_v) 
 {
+	RenderResult *rr = rr_v;
+	ImageUser *iuser = iuser_v;
+	RenderLayer *rl;
+	RenderPass *rp;
+	int layer = iuser->layer;
+
+	if (iuser->pass == 0)
+		return;
+
+	if (RE_HasFakeLayer(rr))
+		layer -= 1;
+
+	rl = BLI_findlink(&rr->layers, layer);
+
+	if (rl) {
+		RenderPass *rpass = BLI_findlink(&rl->passes, iuser->pass);
+		int rp_index = 0;
+
+		if (rpass == NULL)
+			return;
+
+		for (rp = rl->passes.first; rp; rp = rp->next, rp_index++) {
+			if (rp->passtype == rpass->passtype) {
+				iuser->pass = rp_index - 1;
+				BKE_image_multilayer_index(rr, iuser);
+				WM_event_add_notifier(C, NC_IMAGE | ND_DRAW, NULL);
+				break;
+			}
+		}
+	}
+}
+
+/* 5 view button callbacks... */
+static void image_multiview_cb(bContext *C, void *ima_v, void *iuser_v)
+{
+	Image *ima = ima_v;
 	ImageUser *iuser = iuser_v;
 
-	if (iuser->pass > 0) {
-		iuser->pass--;
-		BKE_image_multilayer_index(rr_v, iuser); 
-		WM_event_add_notifier(C, NC_IMAGE | ND_DRAW, NULL);
-	}
+	BKE_image_multiview_index(ima, iuser);
+	WM_event_add_notifier(C, NC_IMAGE | ND_DRAW, NULL);
 }
 
 #if 0
@@ -522,13 +637,14 @@ static void image_user_change(bContext *C, void *iuser_v, void *unused)
 
 static void uiblock_layer_pass_buttons(uiLayout *layout, Image *image, RenderResult *rr, ImageUser *iuser, int w, short *render_slot)
 {
-	static void *rnd_pt[3];  /* XXX, workaround */
+	static void *rnd_pt[4];  /* XXX, workaround */
 	uiBlock *block = uiLayoutGetBlock(layout);
 	uiBut *but;
 	RenderLayer *rl = NULL;
-	int wmenu1, wmenu2, wmenu3;
+	int wmenu1, wmenu2, wmenu3, wmenu4;
 	const char *fake_name;
-	const char *display_name;
+	const char *display_name = "";
+	const bool show_stereo = (iuser->flag & IMA_SHOW_STEREO) != 0;
 
 	uiLayoutRow(layout, true);
 
@@ -536,6 +652,7 @@ static void uiblock_layer_pass_buttons(uiLayout *layout, Image *image, RenderRes
 	wmenu1 = (2 * w) / 5;
 	wmenu2 = (3 * w) / 5;
 	wmenu3 = (3 * w) / 6;
+	wmenu4 = (3 * w) / 6;
 	
 	rnd_pt[0] = image;
 	rnd_pt[1] = iuser;
@@ -557,6 +674,7 @@ static void uiblock_layer_pass_buttons(uiLayout *layout, Image *image, RenderRes
 
 	if (rr) {
 		RenderPass *rpass;
+		RenderView *rview;
 		int rpass_index;
 
 		/* layer */
@@ -565,29 +683,63 @@ static void uiblock_layer_pass_buttons(uiLayout *layout, Image *image, RenderRes
 		rl = BLI_findlink(&rr->layers, rpass_index);
 		rnd_pt[2] = SET_INT_IN_POINTER(rpass_index);
 
-		display_name = rl ? rl->name : (fake_name ? fake_name : "");
-		but = uiDefMenuBut(block, ui_imageuser_layer_menu, rnd_pt, display_name, 0, 0, wmenu2, UI_UNIT_Y, TIP_("Select Layer"));
-		UI_but_func_set(but, image_multi_cb, rr, iuser);
-		UI_but_type_set_menu_from_pulldown(but);
-
+		if (RE_layers_have_name(rr)) {
+			display_name = rl ? rl->name : (fake_name ? fake_name : "");
+			but = uiDefMenuBut(block, ui_imageuser_layer_menu, rnd_pt, display_name,
+			                   0, 0, wmenu2, UI_UNIT_Y, TIP_("Select Layer"));
+			UI_but_func_set(but, image_multi_cb, rr, iuser);
+			UI_but_type_set_menu_from_pulldown(but);
+		}
 
 		/* pass */
 		fake_name = ui_imageuser_pass_fake_name(rl);
 		rpass = (rl ? BLI_findlink(&rl->passes, iuser->pass  - (fake_name ? 1 : 0)) : NULL);
 
-		display_name = rpass ? rpass->name : (fake_name ? fake_name : "");
-		but = uiDefMenuBut(block, ui_imageuser_pass_menu, rnd_pt, display_name, 0, 0, wmenu3, UI_UNIT_Y, TIP_("Select Pass"));
+		display_name = rpass ? rpass->internal_name : (fake_name ? fake_name : "");
+		but = uiDefMenuBut(block, ui_imageuser_pass_menu, rnd_pt, IFACE_(display_name),
+		                   0, 0, wmenu3, UI_UNIT_Y, TIP_("Select Pass"));
 		UI_but_func_set(but, image_multi_cb, rr, iuser);
 		UI_but_type_set_menu_from_pulldown(but);
+
+		/* view */
+		if (BLI_listbase_count_ex(&rr->views, 2) > 1 &&
+		    ((!show_stereo) || (!RE_RenderResult_is_stereo(rr))))
+		{
+			rview = BLI_findlink(&rr->views, iuser->view);
+			display_name = rview ? rview->name : "";
+
+			but = uiDefMenuBut(block, ui_imageuser_view_menu_rr, rnd_pt, display_name, 0, 0, wmenu4, UI_UNIT_Y, TIP_("Select View"));
+			UI_but_func_set(but, image_multi_cb, rr, iuser);
+			UI_but_type_set_menu_from_pulldown(but);
+		}
+	}
+
+	/* stereo image */
+	else if (((image->flag & IMA_IS_STEREO) && (!show_stereo)) ||
+	         ((image->flag & IMA_IS_MULTIVIEW) && ((image->flag & IMA_IS_STEREO) == 0)))
+	{
+		ImageView *iv;
+		int nr = 0;
+
+		for (iv = image->views.first; iv; iv = iv->next) {
+			if (nr++ == iuser->view) {
+				display_name = iv->name;
+				break;
+			}
+		}
+
+		but = uiDefMenuBut(block, ui_imageuser_view_menu_multiview, rnd_pt, display_name, 0, 0, wmenu1, UI_UNIT_Y, TIP_("Select View"));
+		UI_but_func_set(but, image_multiview_cb, image, iuser);
+		UI_but_type_set_menu_from_pulldown(but);
 	}
 }
 
-static void uiblock_layer_pass_arrow_buttons(uiLayout *layout, Image *image, RenderResult *rr, ImageUser *iuser, short *render_slot)
+static void uiblock_layer_pass_arrow_buttons(uiLayout *layout, Image *image, RenderResult *rr, ImageUser *iuser,
+                                             int menus_width, short *render_slot)
 {
 	uiBlock *block = uiLayoutGetBlock(layout);
 	uiLayout *row;
 	uiBut *but;
-	const float dpi_fac = UI_DPI_FAC;
 	
 	row = uiLayoutRow(layout, true);
 
@@ -604,7 +756,7 @@ static void uiblock_layer_pass_arrow_buttons(uiLayout *layout, Image *image, Ren
 	but = uiDefIconBut(block, UI_BTYPE_BUT, 0, ICON_TRIA_RIGHT,  0, 0, 0.90f * UI_UNIT_X, UI_UNIT_Y, NULL, 0, 0, 0, 0, TIP_("Next Layer"));
 	UI_but_func_set(but, image_multi_inclay_cb, rr, iuser);
 
-	uiblock_layer_pass_buttons(row, image, rr, iuser, 230 * dpi_fac, render_slot);
+	uiblock_layer_pass_buttons(row, image, rr, iuser, menus_width, render_slot);
 
 	/* decrease, increase arrows */
 	but = uiDefIconBut(block, UI_BTYPE_BUT, 0, ICON_TRIA_LEFT,   0, 0, 0.85f * UI_UNIT_X, UI_UNIT_Y, NULL, 0, 0, 0, 0, TIP_("Previous Pass"));
@@ -638,10 +790,8 @@ static void rna_update_cb(bContext *C, void *arg_cb, void *UNUSED(arg))
 	RNA_property_update(C, &cb->ptr, cb->prop);
 }
 
-void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char *propname, PointerRNA *userptr, int compact)
+void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char *propname, PointerRNA *userptr, int compact, int multiview)
 {
-#define MAX_INFO_LEN  128
-
 	PropertyRNA *prop;
 	PointerRNA imaptr;
 	RNAUpdateCb *cb;
@@ -650,7 +800,7 @@ void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char
 	Scene *scene = CTX_data_scene(C);
 	uiLayout *row, *split, *col;
 	uiBlock *block;
-	char str[MAX_INFO_LEN];
+	char str[MAX_IMAGE_INFO_LEN];
 
 	void *lock;
 
@@ -687,14 +837,14 @@ void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char
 	uiLayoutSetContextPointer(layout, "edit_image_user", userptr);
 
 	if (!compact)
-		uiTemplateID(layout, C, ptr, propname, "IMAGE_OT_new", "IMAGE_OT_open", NULL);
+		uiTemplateID(layout, C, ptr, propname, ima ? NULL : "IMAGE_OT_new", "IMAGE_OT_open", NULL);
 
 	if (ima) {
 		UI_block_funcN_set(block, rna_update_cb, MEM_dupallocN(cb), NULL);
 
 		if (ima->source == IMA_SRC_VIEWER) {
 			ImBuf *ibuf = BKE_image_acquire_ibuf(ima, iuser, &lock);
-			image_info(scene, iuser, ima, ibuf, str, MAX_INFO_LEN);
+			image_info(scene, iuser, ima, ibuf, str, MAX_IMAGE_INFO_LEN);
 			BKE_image_release_ibuf(ima, ibuf, lock);
 
 			uiItemL(layout, ima->id.name + 2, ICON_NONE);
@@ -723,10 +873,17 @@ void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char
 			else if (ima->type == IMA_TYPE_R_RESULT) {
 				/* browse layer/passes */
 				RenderResult *rr;
+				const float dpi_fac = UI_DPI_FAC;
+				const int menus_width = 230 * dpi_fac;
 
 				/* use BKE_image_acquire_renderresult  so we get the correct slot in the menu */
 				rr = BKE_image_acquire_renderresult(scene, ima);
-				uiblock_layer_pass_arrow_buttons(layout, ima, rr, iuser, &ima->render_slot);
+				if (rr) {
+					uiblock_layer_pass_arrow_buttons(layout, ima, rr, iuser, menus_width, &ima->render_slot);
+				}
+				else {
+					uiblock_layer_pass_buttons(layout, ima, rr, iuser, menus_width, &ima->render_slot);
+				}
 				BKE_image_release_renderresult(scene, ima);
 			}
 		}
@@ -735,13 +892,13 @@ void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char
 
 			if (ima->source != IMA_SRC_GENERATED) {
 				row = uiLayoutRow(layout, true);
-				if (ima->packedfile)
+				if (BKE_image_has_packedfile(ima))
 					uiItemO(row, "", ICON_PACKAGE, "image.unpack");
 				else
 					uiItemO(row, "", ICON_UGLYPACKAGE, "image.pack");
 				
 				row = uiLayoutRow(row, true);
-				uiLayoutSetEnabled(row, ima->packedfile == NULL);
+				uiLayoutSetEnabled(row, BKE_image_has_packedfile(ima) == false);
 				uiItemR(row, &imaptr, "filepath", 0, "", ICON_NONE);
 				uiItemO(row, "", ICON_FILE_REFRESH, "image.reload");
 			}
@@ -759,14 +916,12 @@ void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char
 
 			/* multilayer? */
 			if (ima->type == IMA_TYPE_MULTILAYER && ima->rr) {
-				uiblock_layer_pass_arrow_buttons(layout, ima, ima->rr, iuser, NULL);
+				const float dpi_fac = UI_DPI_FAC;
+				uiblock_layer_pass_arrow_buttons(layout, ima, ima->rr, iuser, 230 * dpi_fac, NULL);
 			}
 			else if (ima->source != IMA_SRC_GENERATED) {
 				if (compact == 0) {
-					ImBuf *ibuf = BKE_image_acquire_ibuf(ima, iuser, &lock);
-					image_info(scene, iuser, ima, ibuf, str, MAX_INFO_LEN);
-					BKE_image_release_ibuf(ima, ibuf, lock);
-					uiItemL(layout, str, ICON_NONE);
+					uiTemplateImageInfo(layout, C, ima, iuser);
 				}
 			}
 
@@ -780,7 +935,7 @@ void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char
 					bool has_alpha = true;
 
 					if (ibuf) {
-						int imtype = BKE_ftype_to_imtype(ibuf->ftype);
+						int imtype = BKE_image_ftype_to_imtype(ibuf->ftype, &ibuf->foptions);
 						char valid_channels = BKE_imtype_valid_channels(imtype, false);
 
 						has_alpha = (valid_channels & IMA_CHAN_FLAG_ALPHA) != 0;
@@ -788,13 +943,28 @@ void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char
 						BKE_image_release_ibuf(ima, ibuf, NULL);
 					}
 
+					if (multiview) {
+						if ((scene->r.scemode & R_MULTIVIEW) != 0) {
+							uiItemR(layout, &imaptr, "use_multiview", 0, NULL, ICON_NONE);
+
+							if (RNA_boolean_get(&imaptr, "use_multiview")) {
+								uiTemplateImageViews(layout, &imaptr);
+							}
+						}
+					}
+
 					if (has_alpha) {
 						col = uiLayoutColumn(layout, false);
 						uiItemR(col, &imaptr, "use_alpha", 0, NULL, ICON_NONE);
-						uiItemR(col, &imaptr, "alpha_mode", 0, IFACE_("Alpha"), ICON_NONE);
+						row = uiLayoutRow(col, false);
+						uiLayoutSetActive(row, RNA_boolean_get(&imaptr, "use_alpha"));
+						uiItemR(row, &imaptr, "alpha_mode", 0, IFACE_("Alpha"), ICON_NONE);
 					}
 
-					uiItemS(layout);
+					if (ima->source == IMA_SRC_MOVIE) {
+						col = uiLayoutColumn(layout, false);
+						uiItemR(col, &imaptr, "use_deinterlace", 0, IFACE_("Deinterlace"), ICON_NONE);
+					}
 
 					split = uiLayoutSplit(layout, 0.0f, false);
 
@@ -856,8 +1026,6 @@ void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char
 	}
 
 	MEM_freeN(cb);
-
-#undef MAX_INFO_LEN
 }
 
 void uiTemplateImageSettings(uiLayout *layout, PointerRNA *imfptr, int color_management)
@@ -961,22 +1129,140 @@ void uiTemplateImageSettings(uiLayout *layout, PointerRNA *imfptr, int color_man
 	}
 }
 
+void uiTemplateImageStereo3d(uiLayout *layout, PointerRNA *stereo3d_format_ptr)
+{
+	Stereo3dFormat *stereo3d_format = stereo3d_format_ptr->data;
+	uiLayout *col;
+
+	col = uiLayoutColumn(layout, false);
+	uiItemR(col, stereo3d_format_ptr, "display_mode", 0, NULL, ICON_NONE);
+
+	switch (stereo3d_format->display_mode) {
+		case S3D_DISPLAY_ANAGLYPH:
+		{
+			uiItemR(col, stereo3d_format_ptr, "anaglyph_type", 0, NULL, ICON_NONE);
+			break;
+		}
+		case S3D_DISPLAY_INTERLACE:
+		{
+			uiItemR(col, stereo3d_format_ptr, "interlace_type", 0, NULL, ICON_NONE);
+			uiItemR(col, stereo3d_format_ptr, "use_interlace_swap", 0, NULL, ICON_NONE);
+			break;
+		}
+		case S3D_DISPLAY_SIDEBYSIDE:
+		{
+			uiItemR(col, stereo3d_format_ptr, "use_sidebyside_crosseyed", 0, NULL, ICON_NONE);
+			/* fall-through */
+		}
+		case S3D_DISPLAY_TOPBOTTOM:
+		{
+			uiItemR(col, stereo3d_format_ptr, "use_squeezed_frame", 0, NULL, ICON_NONE);
+			break;
+		}
+	}
+}
+
+static void uiTemplateViewsFormat(uiLayout *layout, PointerRNA *ptr, PointerRNA *stereo3d_format_ptr)
+{
+	uiLayout *col, *box;
+
+	col = uiLayoutColumn(layout, false);
+
+	uiItemL(col, IFACE_("Views Format:"), ICON_NONE);
+	uiItemR(uiLayoutRow(col, false), ptr, "views_format", UI_ITEM_R_EXPAND, NULL, ICON_NONE);
+
+	if (stereo3d_format_ptr) {
+		box = uiLayoutBox(col);
+		uiLayoutSetActive(box, RNA_enum_get(ptr, "views_format") == R_IMF_VIEWS_STEREO_3D);
+		uiTemplateImageStereo3d(box, stereo3d_format_ptr);
+	}
+}
+
+void uiTemplateImageViews(uiLayout *layout, PointerRNA *imaptr)
+{
+	Image *ima = imaptr->data;
+
+	if (ima->type != IMA_TYPE_MULTILAYER) {
+		PropertyRNA *prop;
+		PointerRNA stereo3d_format_ptr;
+
+		prop = RNA_struct_find_property(imaptr, "stereo_3d_format");
+		stereo3d_format_ptr = RNA_property_pointer_get(imaptr, prop);
+
+		uiTemplateViewsFormat(layout, imaptr, &stereo3d_format_ptr);
+	}
+	else {
+		uiTemplateViewsFormat(layout, imaptr, NULL);
+	}
+}
+
+void uiTemplateImageFormatViews(uiLayout *layout, PointerRNA *imfptr, PointerRNA *ptr)
+{
+	ImageFormatData *imf = imfptr->data;
+
+	if (ptr == NULL)
+		return;
+
+	uiItemR(layout, ptr, "use_multiview", 0, NULL, ICON_NONE);
+
+	if (RNA_boolean_get(ptr, "use_multiview")) {
+		if (imf->imtype != R_IMF_IMTYPE_MULTILAYER) {
+			PropertyRNA *prop;
+			PointerRNA stereo3d_format_ptr;
+
+			prop = RNA_struct_find_property(imfptr, "stereo_3d_format");
+			stereo3d_format_ptr = RNA_property_pointer_get(imfptr, prop);
+
+			uiTemplateViewsFormat(layout, imfptr, &stereo3d_format_ptr);
+		}
+		else {
+			uiTemplateViewsFormat(layout, imfptr, NULL);
+		}
+	}
+}
+
 void uiTemplateImageLayers(uiLayout *layout, bContext *C, Image *ima, ImageUser *iuser)
 {
 	Scene *scene = CTX_data_scene(C);
 
 	/* render layers and passes */
 	if (ima && iuser) {
-		const float dpi_fac = UI_DPI_FAC;
 		RenderResult *rr;
+		const float dpi_fac = UI_DPI_FAC;
+		const int menus_width = 160 * dpi_fac;
+		const bool is_render_result = (ima->type == IMA_TYPE_R_RESULT);
 
 		/* use BKE_image_acquire_renderresult  so we get the correct slot in the menu */
 		rr = BKE_image_acquire_renderresult(scene, ima);
-		uiblock_layer_pass_buttons(layout, ima, rr, iuser, 160 * dpi_fac, (ima->type == IMA_TYPE_R_RESULT) ? &ima->render_slot : NULL);
+		if (rr && is_render_result) {
+			uiblock_layer_pass_arrow_buttons(layout, ima, rr, iuser, menus_width, &ima->render_slot);
+		}
+		else {
+			uiblock_layer_pass_buttons(layout, ima, rr, iuser, menus_width,
+			                           is_render_result ? &ima->render_slot : NULL);
+		}
 		BKE_image_release_renderresult(scene, ima);
 	}
 }
 
+void uiTemplateImageInfo(uiLayout *layout, bContext *C, Image *ima, ImageUser *iuser)
+{
+	ImBuf *ibuf;
+	char str[MAX_IMAGE_INFO_LEN];
+	void *lock;
+
+	if (!ima || !iuser)
+		return;
+
+	ibuf = BKE_image_acquire_ibuf(ima, iuser, &lock);
+
+	image_info(CTX_data_scene(C), iuser, ima, ibuf, str, MAX_IMAGE_INFO_LEN);
+	BKE_image_release_ibuf(ima, ibuf, lock);
+	uiItemL(layout, str, ICON_NONE);
+}
+
+#undef MAX_IMAGE_INFO_LEN
+
 void image_buttons_register(ARegionType *UNUSED(art))
 {
 	
