@@ -36,87 +36,63 @@
 
 /* internal exports only */
 
-struct wmWindowManager;
 struct wmOperatorType;
 struct TreeStoreElem;
 struct bContext;
 struct Scene;
-struct ARegion;
 struct ID;
 struct Object;
 
 typedef struct TreeElement {
 	struct TreeElement *next, *prev, *parent;
 	ListBase subtree;
-	float xs, ys;       // do selection
-	int store_index;    // offset in tree store
-	short flag;         // flag for non-saved stuff
-	short index;        // index for data arrays
-	short idcode;       // from TreeStore id
-	short xend;         // width of item display, for select
+	int xs, ys;                // do selection
+	TreeStoreElem *store_elem; // element in tree store
+	short flag;                // flag for non-saved stuff
+	short index;               // index for data arrays
+	short idcode;              // from TreeStore id
+	short xend;                // width of item display, for select
 	const char *name;
-	void *directdata;   // Armature Bones, Base, Sequence, Strip...
-	PointerRNA rnaptr;  // RNA Pointer
+	void *directdata;          // Armature Bones, Base, Sequence, Strip...
+	PointerRNA rnaptr;         // RNA Pointer
 }  TreeElement;
 
+#define TREESTORE_ID_TYPE(_id) \
+	(ELEM(GS((_id)->name), ID_SCE, ID_LI, ID_OB, ID_ME, ID_CU, ID_MB, ID_NT, ID_MA, ID_TE, ID_IM, ID_LT, ID_LA, ID_CA) || \
+	 ELEM(GS((_id)->name), ID_KE, ID_WO, ID_SPK, ID_GR, ID_AR, ID_AC, ID_BR, ID_PA, ID_GD, ID_LS) || \
+	 ELEM(GS((_id)->name), ID_SCR, ID_WM, ID_TXT, ID_VF))  /* Only in 'blendfile' mode ... :/ */
+
 /* TreeElement->flag */
 #define TE_ACTIVE       1
 #define TE_ICONROW      2
 #define TE_LAZY_CLOSED  4
 #define TE_FREE_NAME    8
 
-/* TreeStoreElem types */
-#define TSE_NLA             1
-#define TSE_NLA_ACTION      2
-#define TSE_DEFGROUP_BASE   3
-#define TSE_DEFGROUP        4
-#define TSE_BONE            5
-#define TSE_EBONE           6
-#define TSE_CONSTRAINT_BASE 7
-#define TSE_CONSTRAINT      8
-#define TSE_MODIFIER_BASE   9
-#define TSE_MODIFIER        10
-#define TSE_LINKED_OB       11
-#define TSE_SCRIPT_BASE     12
-#define TSE_POSE_BASE       13
-#define TSE_POSE_CHANNEL    14
-#define TSE_ANIM_DATA       15
-#define TSE_DRIVER_BASE     16
-#define TSE_DRIVER          17
-
-#define TSE_PROXY           18
-#define TSE_R_LAYER_BASE    19
-#define TSE_R_LAYER         20
-#define TSE_R_PASS          21
-#define TSE_LINKED_MAT      22
-/* NOTE, is used for light group */
-#define TSE_LINKED_LAMP     23
-#define TSE_POSEGRP_BASE    24
-#define TSE_POSEGRP         25
-#define TSE_SEQUENCE        26
-#define TSE_SEQ_STRIP       27
-#define TSE_SEQUENCE_DUP    28
-#define TSE_LINKED_PSYS     29
-#define TSE_RNA_STRUCT      30
-#define TSE_RNA_PROPERTY    31
-#define TSE_RNA_ARRAY_ELEM  32
-#define TSE_NLA_TRACK       33
-#define TSE_KEYMAP          34
-#define TSE_KEYMAP_ITEM     35
-
 /* button events */
 #define OL_NAMEBUTTON       1
 
+typedef enum {
+	OL_DRAWSEL_NONE    = 0,  /* inactive (regular black text) */
+	OL_DRAWSEL_NORMAL  = 1,  /* active object (draws white text) */
+	OL_DRAWSEL_ACTIVE  = 2,  /* active obdata (draws a circle around the icon) */
+} eOLDrawState;
+
+typedef enum {
+	OL_SETSEL_NONE     = 0,  /* don't change the selection state */
+	OL_SETSEL_NORMAL   = 1,  /* select the item */
+	OL_SETSEL_EXTEND   = 2,  /* select the item and extend (also toggles selection) */
+} eOLSetState;
+
 /* get TreeStoreElem associated with a TreeElement 
  * < a: (TreeElement) tree element to find stored element for
  */
-#define TREESTORE(a) (soops->treestore->data + (a)->store_index)
+#define TREESTORE(a) ((a)->store_elem)
 
 /* size constants */
 #define OL_Y_OFFSET 2
 
-#define OL_TOG_RESTRICT_VIEWX   (UI_UNIT_X * 3)
-#define OL_TOG_RESTRICT_SELECTX (UI_UNIT_X * 2)
+#define OL_TOG_RESTRICT_VIEWX   (UI_UNIT_X * 3.0f)
+#define OL_TOG_RESTRICT_SELECTX (UI_UNIT_X * 2.0f)
 #define OL_TOG_RESTRICT_RENDERX UI_UNIT_X
 
 #define OL_TOGW OL_TOG_RESTRICT_VIEWX
@@ -153,6 +129,7 @@ void outliner_free_tree(ListBase *lb);
 void outliner_cleanup_tree(struct SpaceOops *soops);
 
 TreeElement *outliner_find_tse(struct SpaceOops *soops, TreeStoreElem *tse);
+TreeElement *outliner_find_tree_element(ListBase *lb, TreeStoreElem *store_elem);
 TreeElement *outliner_find_id(struct SpaceOops *soops, ListBase *lb, struct ID *id);
 struct ID *outliner_search_back(SpaceOops *soops, TreeElement *te, short idcode);
 
@@ -164,8 +141,12 @@ void draw_outliner(const struct bContext *C);
 void restrictbutton_gr_restrict_flag(void *poin, void *poin2, int flag);
 
 /* outliner_select.c -------------------------------------------- */
-int tree_element_type_active(struct bContext *C, struct Scene *scene, struct SpaceOops *soops, TreeElement *te, TreeStoreElem *tselem, int set);
-int tree_element_active(struct bContext *C, struct Scene *scene, SpaceOops *soops, TreeElement *te, int set);
+eOLDrawState tree_element_type_active(
+        struct bContext *C, struct Scene *scene, struct SpaceOops *soops,
+        TreeElement *te, TreeStoreElem *tselem, const eOLSetState set, bool recursive);
+eOLDrawState tree_element_active(struct bContext *C, struct Scene *scene, SpaceOops *soops,
+                                 TreeElement *te, const eOLSetState set, const bool handle_all_types);
+int outliner_item_do_activate(struct bContext *C, int x, int y, bool extend, bool recursive);
 
 /* outliner_edit.c ---------------------------------------------- */
 
@@ -174,7 +155,7 @@ void outliner_do_object_operation(struct bContext *C, struct Scene *scene, struc
 
 int common_restrict_check(struct bContext *C, struct Object *ob);
 
-int outliner_has_one_flag(struct SpaceOops *soops, ListBase *lb, short flag, short curlevel);
+int outliner_has_one_flag(struct SpaceOops *soops, ListBase *lb, short flag, const int curlevel);
 void outliner_set_flag(struct SpaceOops *soops, ListBase *lb, short flag, short set);
 
 void object_toggle_visibility_cb(struct bContext *C, struct Scene *scene, TreeElement *te, struct TreeStoreElem *tsep, struct TreeStoreElem *tselem);
@@ -188,9 +169,7 @@ void group_toggle_renderability_cb(struct bContext *C, struct Scene *scene, Tree
 
 void item_rename_cb(struct bContext *C, struct Scene *scene, TreeElement *te, struct TreeStoreElem *tsep, struct TreeStoreElem *tselem);
 
-TreeElement *outliner_dropzone_parent(struct bContext *C, struct wmEvent *event, struct TreeElement *te, float *fmval);
-int outliner_dropzone_parent_clear(struct bContext *C, struct wmEvent *event, struct TreeElement *te, float *fmval);
-TreeElement *outliner_dropzone_scene(struct bContext *C, struct wmEvent *event, struct TreeElement *te, float *fmval);
+TreeElement *outliner_dropzone_find(const struct SpaceOops *soops, const float fmval[2], const bool children);
 /* ...................................................... */
 
 void OUTLINER_OT_item_activate(struct wmOperatorType *ot);
@@ -218,21 +197,26 @@ void OUTLINER_OT_keyingset_remove_selected(struct wmOperatorType *ot);
 void OUTLINER_OT_drivers_add_selected(struct wmOperatorType *ot);
 void OUTLINER_OT_drivers_delete_selected(struct wmOperatorType *ot);
 
+void OUTLINER_OT_orphans_purge(struct wmOperatorType *ot);
+
 void OUTLINER_OT_parent_drop(struct wmOperatorType *ot);
 void OUTLINER_OT_parent_clear(struct wmOperatorType *ot);
 void OUTLINER_OT_scene_drop(struct wmOperatorType *ot);
 void OUTLINER_OT_material_drop(struct wmOperatorType *ot);
+void OUTLINER_OT_group_link(struct wmOperatorType *ot);
 
 /* outliner_tools.c ---------------------------------------------- */
 
 void OUTLINER_OT_operation(struct wmOperatorType *ot);
+void OUTLINER_OT_scene_operation(struct wmOperatorType *ot);
 void OUTLINER_OT_object_operation(struct wmOperatorType *ot);
 void OUTLINER_OT_group_operation(struct wmOperatorType *ot);
 void OUTLINER_OT_id_operation(struct wmOperatorType *ot);
 void OUTLINER_OT_data_operation(struct wmOperatorType *ot);
 void OUTLINER_OT_animdata_operation(struct wmOperatorType *ot);
 void OUTLINER_OT_action_set(struct wmOperatorType *ot);
-
+void OUTLINER_OT_constraint_operation(struct wmOperatorType *ot);
+void OUTLINER_OT_modifier_operation(struct wmOperatorType *ot);
 /* ---------------------------------------------------------------- */
 
 /* outliner_ops.c */
