@@ -33,8 +33,10 @@
 
 #include "DNA_anim_types.h"
 #include "DNA_armature_types.h"
+#include "DNA_gpencil_types.h"
 #include "DNA_group_types.h"
 #include "DNA_lamp_types.h"
+#include "DNA_linestyle_types.h"
 #include "DNA_material_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meta_types.h"
@@ -42,12 +44,15 @@
 #include "DNA_sequence_types.h"
 #include "DNA_world_types.h"
 #include "DNA_object_types.h"
+#include "DNA_constraint_types.h"
+#include "DNA_modifier_types.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_utildefines.h"
 
 #include "BKE_animsys.h"
 #include "BKE_context.h"
+#include "BKE_constraint.h"
 #include "BKE_depsgraph.h"
 #include "BKE_fcurve.h"
 #include "BKE_group.h"
@@ -68,6 +73,7 @@
 
 #include "UI_interface.h"
 #include "UI_view2d.h"
+#include "UI_resources.h"
 
 #include "RNA_access.h"
 #include "RNA_define.h"
@@ -75,6 +81,7 @@
 
 #include "outliner_intern.h"
 
+
 /* ****************************************************** */
 
 /* ************ SELECTION OPERATIONS ********* */
@@ -111,7 +118,7 @@ static void set_operation_types(SpaceOops *soops, ListBase *lb,
 					case ID_LA: case ID_AR: case ID_CA: case ID_SPK:
 					case ID_MA: case ID_TE: case ID_IP: case ID_IM:
 					case ID_SO: case ID_KE: case ID_WO: case ID_AC:
-					case ID_NLA: case ID_TXT: case ID_GR:
+					case ID_NLA: case ID_TXT: case ID_GR: case ID_LS:
 						if (*idlevel == 0) *idlevel = idcode;
 						else if (*idlevel != idcode) *idlevel = -1;
 						break;
@@ -158,11 +165,16 @@ static void unlink_material_cb(bContext *UNUSED(C), Scene *UNUSED(scene), TreeEl
 		totcol = mb->totcol;
 		matar = mb->mat;
 	}
+	else {
+		BLI_assert(0);
+	}
 
-	for (a = 0; a < totcol; a++) {
-		if (a == te->index && matar[a]) {
-			matar[a]->id.us--;
-			matar[a] = NULL;
+	if (LIKELY(matar != NULL)) {
+		for (a = 0; a < totcol; a++) {
+			if (a == te->index && matar[a]) {
+				matar[a]->id.us--;
+				matar[a] = NULL;
+			}
 		}
 	}
 }
@@ -185,8 +197,14 @@ static void unlink_texture_cb(bContext *UNUSED(C), Scene *UNUSED(scene), TreeEle
 		World *wrld = (World *)tsep->id;
 		mtex = wrld->mtex;
 	}
-	else return;
-	
+	else if (GS(tsep->id->name) == ID_LS) {
+		FreestyleLineStyle *ls = (FreestyleLineStyle *)tsep->id;
+		mtex = ls->mtex;
+	}
+	else {
+		return;
+	}
+
 	for (a = 0; a < MAX_MTEX; a++) {
 		if (a == te->index && mtex[a]) {
 			if (mtex[a]->tex) {
@@ -245,7 +263,90 @@ static void outliner_do_libdata_operation(bContext *C, Scene *scene, SpaceOops *
 	}
 }
 
-/* */
+/* ******************************************** */
+typedef enum eOutliner_PropSceneOps {
+	OL_SCENE_OP_DELETE = 1
+} eOutliner_PropSceneOps;
+
+static EnumPropertyItem prop_scene_op_types[] = {
+	{OL_SCENE_OP_DELETE, "DELETE", ICON_X, "Delete", ""},
+	{0, NULL, 0, NULL, NULL}
+};
+
+static bool outliner_do_scene_operation(
+        bContext *C, eOutliner_PropSceneOps event, ListBase *lb,
+        bool (*operation_cb)(bContext *, eOutliner_PropSceneOps, TreeElement *, TreeStoreElem *))
+{
+	TreeElement *te;
+	TreeStoreElem *tselem;
+	bool success = false;
+
+	for (te = lb->first; te; te = te->next) {
+		tselem = TREESTORE(te);
+		if (tselem->flag & TSE_SELECTED) {
+			if (operation_cb(C, event, te, tselem)) {
+				success = true;
+			}
+		}
+	}
+
+	return success;
+}
+
+static bool scene_cb(bContext *C, eOutliner_PropSceneOps event, TreeElement *UNUSED(te), TreeStoreElem *tselem)
+{
+	Scene *scene = (Scene *)tselem->id;
+
+	if (event == OL_SCENE_OP_DELETE) {
+		if (ED_screen_delete_scene(C, scene)) {
+			WM_event_add_notifier(C, NC_SCENE | NA_REMOVED, scene);
+		}
+		else {
+			return false;
+		}
+	}
+
+	return true;
+}
+
+static int outliner_scene_operation_exec(bContext *C, wmOperator *op)
+{
+	SpaceOops *soops = CTX_wm_space_outliner(C);
+	const eOutliner_PropSceneOps event = RNA_enum_get(op->ptr, "type");
+
+	if (outliner_do_scene_operation(C, event, &soops->tree, scene_cb) == false) {
+		return OPERATOR_CANCELLED;
+	}
+
+	if (event == OL_SCENE_OP_DELETE) {
+		outliner_cleanup_tree(soops);
+		ED_undo_push(C, "Delete Scene(s)");
+	}
+	else {
+		BLI_assert(0);
+		return OPERATOR_CANCELLED;
+	}
+
+	return OPERATOR_FINISHED;
+}
+
+void OUTLINER_OT_scene_operation(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Outliner Scene Operation";
+	ot->idname = "OUTLINER_OT_scene_operation";
+	ot->description = "Context menu for scene operations";
+
+	/* callbacks */
+	ot->invoke = WM_menu_invoke;
+	ot->exec = outliner_scene_operation_exec;
+	ot->poll = ED_operator_outliner_active;
+
+	ot->flag = 0;
+
+	ot->prop = RNA_def_enum(ot->srna, "type", prop_scene_op_types, 0, "Scene Operation", "");
+}
+/* ******************************************** */
 
 static void object_select_cb(bContext *UNUSED(C), Scene *scene, TreeElement *te,
                              TreeStoreElem *UNUSED(tsep), TreeStoreElem *tselem)
@@ -259,6 +360,17 @@ static void object_select_cb(bContext *UNUSED(C), Scene *scene, TreeElement *te,
 	}
 }
 
+static void object_select_hierarchy_cb(bContext *C, Scene *UNUSED(scene), TreeElement *UNUSED(te),
+                             TreeStoreElem *UNUSED(tsep), TreeStoreElem *UNUSED(tselem))
+{
+	/* From where do i get the x,y coordinate of the mouse event ? */
+	wmWindow *win = CTX_wm_window(C);
+	int x = win->eventstate->mval[0];
+	int y = win->eventstate->mval[1];
+	outliner_item_do_activate(C, x, y, true, true);
+}
+
+
 static void object_deselect_cb(bContext *UNUSED(C), Scene *scene, TreeElement *te,
                                TreeStoreElem *UNUSED(tsep), TreeStoreElem *tselem)
 {
@@ -281,7 +393,7 @@ static void object_delete_cb(bContext *C, Scene *scene, TreeElement *te,
 	if (base) {
 		// check also library later
 		if (scene->obedit == base->object)
-			ED_object_exit_editmode(C, EM_FREEDATA | EM_FREEUNDO | EM_WAITCURSOR | EM_DO_UNDO);
+			ED_object_editmode_exit(C, EM_FREEDATA | EM_FREEUNDO | EM_WAITCURSOR | EM_DO_UNDO);
 		
 		ED_base_object_free_and_unlink(CTX_data_main(C), scene, base);
 		te->directdata = NULL;
@@ -295,7 +407,7 @@ static void id_local_cb(bContext *C, Scene *UNUSED(scene), TreeElement *UNUSED(t
 	if (tselem->id->lib && (tselem->id->flag & LIB_EXTERN)) {
 		/* if the ID type has no special local function,
 		 * just clear the lib */
-		if (id_make_local(tselem->id, FALSE) == FALSE) {
+		if (id_make_local(tselem->id, false) == false) {
 			Main *bmain = CTX_data_main(C);
 			id_clear_lib_data(bmain, tselem->id);
 		}
@@ -384,7 +496,7 @@ static void group_linkobs2scene_cb(bContext *UNUSED(C), Scene *scene, TreeElemen
 			/* link to scene */
 			base = MEM_callocN(sizeof(Base), "add_base");
 			BLI_addhead(&scene->base, base);
-			base->lay = (1 << 20) - 1; /*v3d->lay;*/ /* would be nice to use the 3d layer but the include's not here */
+			base->lay = gob->ob->lay;
 			gob->ob->flag |= SELECT;
 			base->flag = gob->ob->flag;
 			base->object = gob->ob;
@@ -398,8 +510,7 @@ static void group_instance_cb(bContext *C, Scene *scene, TreeElement *UNUSED(te)
 {
 	Group *group = (Group *)tselem->id;
 
-	Object *ob = ED_object_add_type(C, OB_EMPTY, scene->cursor, NULL, FALSE, scene->layact);
-	rename_id(&ob->id, group->id.name + 2);
+	Object *ob = ED_object_add_type(C, OB_EMPTY, group->id.name + 2, scene->cursor, NULL, false, scene->layact);
 	ob->dup_group = group;
 	ob->transflag |= OB_DUPLIGROUP;
 	id_lib_extern(&group->id);
@@ -435,6 +546,13 @@ void outliner_do_object_operation(bContext *C, Scene *scene_act, SpaceOops *soop
 
 /* ******************************************** */
 
+static void clear_animdata_cb(int UNUSED(event), TreeElement *UNUSED(te),
+                              TreeStoreElem *tselem, void *UNUSED(arg))
+{
+	BKE_animdata_free(tselem->id);
+}
+
+
 static void unlinkact_animdata_cb(int UNUSED(event), TreeElement *UNUSED(te),
                                   TreeStoreElem *tselem, void *UNUSED(arg))
 {
@@ -468,19 +586,39 @@ static void refreshdrivers_animdata_cb(int UNUSED(event), TreeElement *UNUSED(te
 
 /* --------------------------------- */
 
+typedef enum eOutliner_PropDataOps {
+	OL_DOP_SELECT = 1,
+	OL_DOP_DESELECT,
+	OL_DOP_HIDE,
+	OL_DOP_UNHIDE,
+	OL_DOP_SELECT_LINKED,
+} eOutliner_PropDataOps;
+
+typedef enum eOutliner_PropConstraintOps {
+	OL_CONSTRAINTOP_ENABLE = 1,
+	OL_CONSTRAINTOP_DISABLE,
+	OL_CONSTRAINTOP_DELETE
+} eOutliner_PropConstraintOps;
+
+typedef enum eOutliner_PropModifierOps {
+	OL_MODIFIER_OP_TOGVIS = 1,
+	OL_MODIFIER_OP_TOGREN,
+	OL_MODIFIER_OP_DELETE
+} eOutliner_PropModifierOps;
+
 static void pchan_cb(int event, TreeElement *te, TreeStoreElem *UNUSED(tselem), void *UNUSED(arg))
 {
 	bPoseChannel *pchan = (bPoseChannel *)te->directdata;
 	
-	if (event == 1)
+	if (event == OL_DOP_SELECT)
 		pchan->bone->flag |= BONE_SELECTED;
-	else if (event == 2)
+	else if (event == OL_DOP_DESELECT)
 		pchan->bone->flag &= ~BONE_SELECTED;
-	else if (event == 3) {
+	else if (event == OL_DOP_HIDE) {
 		pchan->bone->flag |= BONE_HIDDEN_P;
 		pchan->bone->flag &= ~BONE_SELECTED;
 	}
-	else if (event == 4)
+	else if (event == OL_DOP_UNHIDE)
 		pchan->bone->flag &= ~BONE_HIDDEN_P;
 }
 
@@ -488,15 +626,15 @@ static void bone_cb(int event, TreeElement *te, TreeStoreElem *UNUSED(tselem), v
 {
 	Bone *bone = (Bone *)te->directdata;
 	
-	if (event == 1)
+	if (event == OL_DOP_SELECT)
 		bone->flag |= BONE_SELECTED;
-	else if (event == 2)
+	else if (event == OL_DOP_DESELECT)
 		bone->flag &= ~BONE_SELECTED;
-	else if (event == 3) {
+	else if (event == OL_DOP_HIDE) {
 		bone->flag |= BONE_HIDDEN_P;
 		bone->flag &= ~BONE_SELECTED;
 	}
-	else if (event == 4)
+	else if (event == OL_DOP_UNHIDE)
 		bone->flag &= ~BONE_HIDDEN_P;
 }
 
@@ -504,35 +642,49 @@ static void ebone_cb(int event, TreeElement *te, TreeStoreElem *UNUSED(tselem),
 {
 	EditBone *ebone = (EditBone *)te->directdata;
 	
-	if (event == 1)
+	if (event == OL_DOP_SELECT)
 		ebone->flag |= BONE_SELECTED;
-	else if (event == 2)
+	else if (event == OL_DOP_DESELECT)
 		ebone->flag &= ~BONE_SELECTED;
-	else if (event == 3) {
+	else if (event == OL_DOP_HIDE) {
 		ebone->flag |= BONE_HIDDEN_A;
 		ebone->flag &= ~BONE_SELECTED | BONE_TIPSEL | BONE_ROOTSEL;
 	}
-	else if (event == 4)
+	else if (event == OL_DOP_UNHIDE)
 		ebone->flag &= ~BONE_HIDDEN_A;
 }
 
 static void sequence_cb(int event, TreeElement *te, TreeStoreElem *tselem, void *scene_ptr)
 {
 	Sequence *seq = (Sequence *)te->directdata;
-	if (event == 1) {
+	if (event == OL_DOP_SELECT) {
 		Scene *scene = (Scene *)scene_ptr;
-		Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
+		Editing *ed = BKE_sequencer_editing_get(scene, false);
 		if (BLI_findindex(ed->seqbasep, seq) != -1) {
-			ED_sequencer_select_sequence_single(scene, seq, TRUE);
+			ED_sequencer_select_sequence_single(scene, seq, true);
 		}
 	}
 
 	(void)tselem;
 }
 
+static void gp_layer_cb(int event, TreeElement *te, TreeStoreElem *UNUSED(tselem), void *UNUSED(arg))
+{
+	bGPDlayer *gpl = (bGPDlayer *)te->directdata;
+	
+	if (event == OL_DOP_SELECT)
+		gpl->flag |= GP_LAYER_SELECT;
+	else if (event == OL_DOP_DESELECT)
+		gpl->flag &= ~GP_LAYER_SELECT;
+	else if (event == OL_DOP_HIDE)
+		gpl->flag |= GP_LAYER_HIDE;
+	else if (event == OL_DOP_UNHIDE)
+		gpl->flag &= ~GP_LAYER_HIDE;
+}
+
 static void data_select_linked_cb(int event, TreeElement *te, TreeStoreElem *UNUSED(tselem), void *C_v)
 {
-	if (event == 5) {
+	if (event == OL_DOP_SELECT_LINKED) {
 		if (RNA_struct_is_ID(te->rnaptr.type)) {
 			bContext *C = (bContext *) C_v;
 			ID *id = te->rnaptr.data;
@@ -542,6 +694,68 @@ static void data_select_linked_cb(int event, TreeElement *te, TreeStoreElem *UNU
 	}
 }
 
+static void constraint_cb(int event, TreeElement *te, TreeStoreElem *UNUSED(tselem), void *C_v)
+{
+	bContext *C = C_v;
+	SpaceOops *soops = CTX_wm_space_outliner(C);
+	bConstraint *constraint = (bConstraint *)te->directdata;
+	Object *ob = (Object *)outliner_search_back(soops, te, ID_OB);
+
+	if (event == OL_CONSTRAINTOP_ENABLE) {
+		constraint->flag &= ~CONSTRAINT_OFF;
+		ED_object_constraint_update(ob);
+		WM_event_add_notifier(C, NC_OBJECT | ND_CONSTRAINT, ob);
+	}
+	else if (event == OL_CONSTRAINTOP_DISABLE) {
+		constraint->flag = CONSTRAINT_OFF;
+		ED_object_constraint_update(ob);
+		WM_event_add_notifier(C, NC_OBJECT | ND_CONSTRAINT, ob);
+	}
+	else if (event == OL_CONSTRAINTOP_DELETE) {
+		ListBase *lb = NULL;
+
+		if (TREESTORE(te->parent->parent)->type == TSE_POSE_CHANNEL) {
+			lb = &((bPoseChannel *)te->parent->parent->directdata)->constraints;
+		}
+		else {
+			lb = &ob->constraints;
+		}
+
+		if (BKE_constraint_remove_ex(lb, ob, constraint, true)) {
+			/* there's no active constraint now, so make sure this is the case */
+			BKE_constraints_active_set(&ob->constraints, NULL);
+			ED_object_constraint_update(ob); /* needed to set the flags on posebones correctly */
+			WM_event_add_notifier(C, NC_OBJECT | ND_CONSTRAINT | NA_REMOVED, ob);
+			te->store_elem->flag &= ~TSE_SELECTED;
+		}
+	}
+}
+
+static void modifier_cb(int event, TreeElement *te, TreeStoreElem *UNUSED(tselem), void *Carg)
+{
+	bContext *C = (bContext *)Carg;
+	Main *bmain = CTX_data_main(C);
+	SpaceOops *soops = CTX_wm_space_outliner(C);
+	ModifierData *md = (ModifierData *)te->directdata;
+	Object *ob = (Object *)outliner_search_back(soops, te, ID_OB);
+
+	if (event == OL_MODIFIER_OP_TOGVIS) {
+		md->mode ^= eModifierMode_Realtime;
+		DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
+		WM_event_add_notifier(C, NC_OBJECT | ND_MODIFIER, ob);
+	}
+	else if (event == OL_MODIFIER_OP_TOGREN) {
+		md->mode ^= eModifierMode_Render;
+		DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
+		WM_event_add_notifier(C, NC_OBJECT | ND_MODIFIER, ob);
+	}
+	else if (event == OL_MODIFIER_OP_DELETE) {
+		ED_object_modifier_remove(NULL, bmain, ob, md);
+		WM_event_add_notifier(C, NC_OBJECT | ND_MODIFIER | NA_REMOVED, ob);
+		te->store_elem->flag &= ~TSE_SELECTED;
+	}
+}
+
 static void outliner_do_data_operation(SpaceOops *soops, int type, int event, ListBase *lb,
                                        void (*operation_cb)(int, TreeElement *, TreeStoreElem *, void *),
                                        void *arg)
@@ -562,16 +776,77 @@ static void outliner_do_data_operation(SpaceOops *soops, int type, int event, Li
 	}
 }
 
+static Base *outline_delete_hierarchy(bContext *C, Scene *scene, Base *base)
+{
+	Base *child_base, *base_next;
+	Object *parent;
+
+	if (!base) {
+	    return NULL;
+	}
+
+	for (child_base = scene->base.first; child_base; child_base = base_next) {
+		base_next = child_base->next;
+		for (parent = child_base->object->parent; parent && (parent != base->object); parent = parent->parent);
+		if (parent) {
+			base_next = outline_delete_hierarchy(C, scene, child_base);
+		}
+	}
+
+	base_next = base->next;
+	ED_base_object_free_and_unlink(CTX_data_main(C), scene, base);
+	return base_next;
+}
+
+static void object_delete_hierarchy_cb(
+        bContext *C, Scene *scene, TreeElement *te, TreeStoreElem *UNUSED(tsep), TreeStoreElem *tselem)
+{
+	Base *base = (Base *)te->directdata;
+	Object *obedit = scene->obedit;
+
+	if (!base) {
+		base = BKE_scene_base_find(scene, (Object *)tselem->id);
+	}
+	if (base) {
+		/* Check also library later. */
+		for (; obedit && (obedit != base->object); obedit = obedit->parent);
+		if (obedit == base->object) {
+			ED_object_editmode_exit(C, EM_FREEDATA | EM_FREEUNDO | EM_WAITCURSOR | EM_DO_UNDO);
+		}
+
+		outline_delete_hierarchy(C, scene, base);
+		te->directdata = NULL;
+		tselem->id = NULL;
+	}
+
+	WM_event_add_notifier(C, NC_SCENE | ND_OB_ACTIVE, scene);
+}
+
 /* **************************************** */
 
+enum {
+	OL_OP_SELECT = 1,
+	OL_OP_DESELECT,
+	OL_OP_SELECT_HIERARCHY,
+	OL_OP_DELETE,
+	OL_OP_DELETE_HIERARCHY,
+	OL_OP_LOCALIZED,  /* disabled, see below */
+	OL_OP_TOGVIS,
+	OL_OP_TOGSEL,
+	OL_OP_TOGREN,
+	OL_OP_RENAME
+};
+
 static EnumPropertyItem prop_object_op_types[] = {
-	{1, "SELECT", 0, "Select", ""},
-	{2, "DESELECT", 0, "Deselect", ""},
-	{4, "DELETE", 0, "Delete", ""},
-	{6, "TOGVIS", 0, "Toggle Visible", ""},
-	{7, "TOGSEL", 0, "Toggle Selectable", ""},
-	{8, "TOGREN", 0, "Toggle Renderable", ""},
-	{9, "RENAME", 0, "Rename", ""},
+	{OL_OP_SELECT, "SELECT", 0, "Select", ""},
+	{OL_OP_DESELECT, "DESELECT", 0, "Deselect", ""},
+	{OL_OP_SELECT_HIERARCHY, "SELECT_HIERARCHY", 0, "Select Hierarchy", ""},
+	{OL_OP_DELETE, "DELETE", 0, "Delete", ""},
+	{OL_OP_DELETE_HIERARCHY, "DELETE_HIERARCHY", 0, "Delete Hierarchy", ""},
+	{OL_OP_TOGVIS, "TOGVIS", 0, "Toggle Visible", ""},
+	{OL_OP_TOGSEL, "TOGSEL", 0, "Toggle Selectable", ""},
+	{OL_OP_TOGREN, "TOGREN", 0, "Toggle Renderable", ""},
+	{OL_OP_RENAME, "RENAME", 0, "Rename", ""},
 	{0, NULL, 0, NULL, NULL}
 };
 
@@ -589,7 +864,7 @@ static int outliner_object_operation_exec(bContext *C, wmOperator *op)
 	
 	event = RNA_enum_get(op->ptr, "type");
 
-	if (event == 1) {
+	if (event == OL_OP_SELECT) {
 		Scene *sce = scene;  // to be able to delete, scenes are set...
 		outliner_do_object_operation(C, scene, soops, &soops->tree, object_select_cb);
 		if (scene != sce) {
@@ -599,12 +874,21 @@ static int outliner_object_operation_exec(bContext *C, wmOperator *op)
 		str = "Select Objects";
 		WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, scene);
 	}
-	else if (event == 2) {
+	else if (event == OL_OP_SELECT_HIERARCHY) {
+		Scene *sce = scene;  // to be able to delete, scenes are set...
+		outliner_do_object_operation(C, scene, soops, &soops->tree, object_select_hierarchy_cb);
+		if (scene != sce) {
+			ED_screen_set_scene(C, CTX_wm_screen(C), sce);
+		}	
+		str = "Select Object Hierarchy";
+		WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, scene);
+	}
+	else if (event == OL_OP_DESELECT) {
 		outliner_do_object_operation(C, scene, soops, &soops->tree, object_deselect_cb);
 		str = "Deselect Objects";
 		WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, scene);
 	}
-	else if (event == 4) {
+	else if (event == OL_OP_DELETE) {
 		outliner_do_object_operation(C, scene, soops, &soops->tree, object_delete_cb);
 
 		/* XXX: tree management normally happens from draw_outliner(), but when
@@ -614,33 +898,47 @@ static int outliner_object_operation_exec(bContext *C, wmOperator *op)
 		 *      cleanup tree here to prevent such cases. */
 		outliner_cleanup_tree(soops);
 
-		DAG_scene_sort(bmain, scene);
+		DAG_relations_tag_update(bmain);
 		str = "Delete Objects";
 		WM_event_add_notifier(C, NC_SCENE | ND_OB_ACTIVE, scene);
 	}
-	else if (event == 5) {    /* disabled, see above enum (ton) */
+	else if (event == OL_OP_DELETE_HIERARCHY) {
+		outliner_do_object_operation(C, scene, soops, &soops->tree, object_delete_hierarchy_cb);
+
+		/* XXX: See OL_OP_DELETE comment above. */
+		outliner_cleanup_tree(soops);
+
+		DAG_relations_tag_update(bmain);
+		str = "Delete Object Hierarchy";
+		WM_event_add_notifier(C, NC_SCENE | ND_OB_ACTIVE, scene);
+	}
+	else if (event == OL_OP_LOCALIZED) {    /* disabled, see above enum (ton) */
 		outliner_do_object_operation(C, scene, soops, &soops->tree, id_local_cb);
 		str = "Localized Objects";
 	}
-	else if (event == 6) {
+	else if (event == OL_OP_TOGVIS) {
 		outliner_do_object_operation(C, scene, soops, &soops->tree, object_toggle_visibility_cb);
 		str = "Toggle Visibility";
 		WM_event_add_notifier(C, NC_SCENE | ND_OB_VISIBLE, scene);
 	}
-	else if (event == 7) {
+	else if (event == OL_OP_TOGSEL) {
 		outliner_do_object_operation(C, scene, soops, &soops->tree, object_toggle_selectability_cb);
 		str = "Toggle Selectability";
 		WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, scene);
 	}
-	else if (event == 8) {
+	else if (event == OL_OP_TOGREN) {
 		outliner_do_object_operation(C, scene, soops, &soops->tree, object_toggle_renderability_cb);
 		str = "Toggle Renderability";
 		WM_event_add_notifier(C, NC_SCENE | ND_OB_RENDER, scene);
 	}
-	else if (event == 9) {
+	else if (event == OL_OP_RENAME) {
 		outliner_do_object_operation(C, scene, soops, &soops->tree, item_rename_cb);
 		str = "Rename Object";
 	}
+	else {
+		BLI_assert(0);
+		return OPERATOR_CANCELLED;
+	}
 
 	ED_undo_push(C, str);
 	
@@ -667,15 +965,26 @@ void OUTLINER_OT_object_operation(wmOperatorType *ot)
 
 /* **************************************** */
 
+typedef enum eOutliner_PropGroupOps {
+	OL_GROUPOP_UNLINK = 1,
+	OL_GROUPOP_LOCAL,
+	OL_GROUPOP_LINK,
+	OL_GROUPOP_INSTANCE,
+	OL_GROUPOP_TOGVIS,
+	OL_GROUPOP_TOGSEL,
+	OL_GROUPOP_TOGREN,
+	OL_GROUPOP_RENAME,
+} eOutliner_PropGroupOps;
+
 static EnumPropertyItem prop_group_op_types[] = {
-	{0, "UNLINK",   0, "Unlink Group", ""},
-	{1, "LOCAL",    0, "Make Local Group", ""},
-	{2, "LINK",     0, "Link Group Objects to Scene", ""},
-	{3, "INSTANCE", 0, "Instance Groups in Scene", ""},
-	{4, "TOGVIS",   0, "Toggle Visible Group", ""},
-	{5, "TOGSEL",   0, "Toggle Selectable", ""},
-	{6, "TOGREN",   0, "Toggle Renderable", ""},
-	{7, "RENAME",   0, "Rename", ""},
+	{OL_GROUPOP_UNLINK, "UNLINK",     0, "Unlink Group", ""},
+	{OL_GROUPOP_LOCAL, "LOCAL",       0, "Make Local Group", ""},
+	{OL_GROUPOP_LINK, "LINK",         0, "Link Group Objects to Scene", ""},
+	{OL_GROUPOP_INSTANCE, "INSTANCE", 0, "Instance Groups in Scene", ""},
+	{OL_GROUPOP_TOGVIS, "TOGVIS",     0, "Toggle Visible Group", ""},
+	{OL_GROUPOP_TOGSEL, "TOGSEL",     0, "Toggle Selectable", ""},
+	{OL_GROUPOP_TOGREN, "TOGREN",     0, "Toggle Renderable", ""},
+	{OL_GROUPOP_RENAME, "RENAME",     0, "Rename", ""},
 	{0, NULL, 0, NULL, NULL}
 };
 
@@ -692,30 +1001,42 @@ static int outliner_group_operation_exec(bContext *C, wmOperator *op)
 	event = RNA_enum_get(op->ptr, "type");
 
 	switch (event) {
-		case 0: outliner_do_libdata_operation(C, scene, soops, &soops->tree, unlink_group_cb); break;
-		case 1: outliner_do_libdata_operation(C, scene, soops, &soops->tree, id_local_cb); break;
-		case 2: outliner_do_libdata_operation(C, scene, soops, &soops->tree, group_linkobs2scene_cb); break;
-		case 3: outliner_do_libdata_operation(C, scene, soops, &soops->tree, group_instance_cb); break;
-		case 4: outliner_do_libdata_operation(C, scene, soops, &soops->tree, group_toggle_visibility_cb); break;
-		case 5: outliner_do_libdata_operation(C, scene, soops, &soops->tree, group_toggle_selectability_cb); break;
-		case 6: outliner_do_libdata_operation(C, scene, soops, &soops->tree, group_toggle_renderability_cb); break;
-		case 7: outliner_do_libdata_operation(C, scene, soops, &soops->tree, item_rename_cb); break;
+		case OL_GROUPOP_UNLINK:
+			outliner_do_libdata_operation(C, scene, soops, &soops->tree, unlink_group_cb);
+			break;
+		case OL_GROUPOP_LOCAL:
+			outliner_do_libdata_operation(C, scene, soops, &soops->tree, id_local_cb);
+			break;
+		case OL_GROUPOP_LINK:
+			outliner_do_libdata_operation(C, scene, soops, &soops->tree, group_linkobs2scene_cb);
+			break;
+		case OL_GROUPOP_INSTANCE:
+			outliner_do_libdata_operation(C, scene, soops, &soops->tree, group_instance_cb);
+			break;
+		case OL_GROUPOP_TOGVIS:
+			outliner_do_libdata_operation(C, scene, soops, &soops->tree, group_toggle_visibility_cb);
+			break;
+		case OL_GROUPOP_TOGSEL:
+			outliner_do_libdata_operation(C, scene, soops, &soops->tree, group_toggle_selectability_cb);
+			break;
+		case OL_GROUPOP_TOGREN:
+			outliner_do_libdata_operation(C, scene, soops, &soops->tree, group_toggle_renderability_cb);
+			break;
+		case OL_GROUPOP_RENAME:
+			outliner_do_libdata_operation(C, scene, soops, &soops->tree, item_rename_cb);
+			break;
 		default:
 			BLI_assert(0);
-			return OPERATOR_CANCELLED;
 	}
-	
 
 	if (event == 3) { /* instance */
-		Main *bmain = CTX_data_main(C);
-
 		/* works without this except if you try render right after, see: 22027 */
-		DAG_scene_sort(bmain, scene);
+		DAG_relations_tag_update(CTX_data_main(C));
 	}
-	
-	ED_undo_push(C, prop_group_op_types[event].name);
+
+	ED_undo_push(C, prop_group_op_types[event - 1].name);
 	WM_event_add_notifier(C, NC_GROUP, NULL);
-	
+
 	return OPERATOR_FINISHED;
 }
 
@@ -759,7 +1080,7 @@ static EnumPropertyItem prop_id_op_types[] = {
 	{OUTLINER_IDOP_LOCAL, "LOCAL", 0, "Make Local", ""},
 	{OUTLINER_IDOP_SINGLE, "SINGLE", 0, "Make Single User", ""},
 	{OUTLINER_IDOP_FAKE_ADD, "ADD_FAKE", 0, "Add Fake User",
-     "Ensure datablock gets saved even if it isn't in use (e.g. for motion and material libraries)"},
+	 "Ensure datablock gets saved even if it isn't in use (e.g. for motion and material libraries)"},
 	{OUTLINER_IDOP_FAKE_CLEAR, "CLEAR_FAKE", 0, "Clear Fake User", ""},
 	{OUTLINER_IDOP_RENAME, "RENAME", 0, "Rename", ""},
 	{OUTLINER_IDOP_SELECT_LINKED, "SELECT_LINKED", 0, "Select Linked", ""},
@@ -811,20 +1132,18 @@ static int outliner_id_operation_exec(bContext *C, wmOperator *op)
 					ED_undo_push(C, "Unlink world");
 					break;
 				default:
-					BKE_report(op->reports, RPT_WARNING, "Not Yet");
+					BKE_report(op->reports, RPT_WARNING, "Not yet implemented");
 					break;
 			}
+			break;
 		}
-		break;
-			
 		case OUTLINER_IDOP_LOCAL:
 		{
 			/* make local */
 			outliner_do_libdata_operation(C, scene, soops, &soops->tree, id_local_cb);
 			ED_undo_push(C, "Localized Data");
+			break;
 		}
-		break;
-			
 		case OUTLINER_IDOP_SINGLE:
 		{
 			/* make single user */
@@ -844,12 +1163,11 @@ static int outliner_id_operation_exec(bContext *C, wmOperator *op)
 					break;
 					
 				default:
-					BKE_report(op->reports, RPT_WARNING, "Not Yet");
+					BKE_report(op->reports, RPT_WARNING, "Not yet implemented");
 					break;
 			}
+			break;
 		}
-		break;
-			
 		case OUTLINER_IDOP_FAKE_ADD:
 		{
 			/* set fake user */
@@ -857,9 +1175,8 @@ static int outliner_id_operation_exec(bContext *C, wmOperator *op)
 			
 			WM_event_add_notifier(C, NC_ID | NA_EDITED, NULL);
 			ED_undo_push(C, "Add Fake User");
+			break;
 		}
-		break;
-			
 		case OUTLINER_IDOP_FAKE_CLEAR:
 		{
 			/* clear fake user */
@@ -867,8 +1184,8 @@ static int outliner_id_operation_exec(bContext *C, wmOperator *op)
 			
 			WM_event_add_notifier(C, NC_ID | NA_EDITED, NULL);
 			ED_undo_push(C, "Clear Fake User");
+			break;
 		}
-		break;
 		case OUTLINER_IDOP_RENAME:
 		{
 			/* rename */
@@ -876,9 +1193,8 @@ static int outliner_id_operation_exec(bContext *C, wmOperator *op)
 			
 			WM_event_add_notifier(C, NC_ID | NA_EDITED, NULL);
 			ED_undo_push(C, "Rename");
+			break;
 		}
-		break;
-
 		case OUTLINER_IDOP_SELECT_LINKED:
 			outliner_do_libdata_operation(C, scene, soops, &soops->tree, id_select_linked_cb);
 			ED_undo_push(C, "Select");
@@ -974,15 +1290,15 @@ static int outliner_action_set_exec(bContext *C, wmOperator *op)
 	act = BLI_findlink(&CTX_data_main(C)->action, RNA_enum_get(op->ptr, "action"));
 	
 	if (act == NULL) {
-		BKE_report(op->reports, RPT_ERROR, "No valid Action to add");
+		BKE_report(op->reports, RPT_ERROR, "No valid action to add");
 		return OPERATOR_CANCELLED;
 	}
 	else if (act->idroot == 0) {
 		/* hopefully in this case (i.e. library of userless actions), the user knows what they're doing... */
 		BKE_reportf(op->reports, RPT_WARNING,
-		            "Action '%s' does not specify what datablocks it can be used on. "
-		            "Try setting the 'ID Root Type' setting from the Datablocks Editor "
-		            "for this Action to avoid future problems",
+		            "Action '%s' does not specify what datablocks it can be used on "
+		            "(try setting the 'ID Root Type' setting from the Datablocks Editor "
+		            "for this action to avoid future problems)",
 		            act->id.name + 2);
 	}
 	
@@ -1023,6 +1339,7 @@ void OUTLINER_OT_action_set(wmOperatorType *ot)
 	// TODO: this would be nicer as an ID-pointer...
 	prop = RNA_def_enum(ot->srna, "action", DummyRNA_NULL_items, 0, "Action", "");
 	RNA_def_enum_funcs(prop, RNA_action_itemf);
+	RNA_def_property_flag(prop, PROP_ENUM_NO_TRANSLATE);
 	ot->prop = prop;
 }
 
@@ -1031,6 +1348,8 @@ void OUTLINER_OT_action_set(wmOperatorType *ot)
 typedef enum eOutliner_AnimDataOps {
 	OUTLINER_ANIMOP_INVALID = 0,
 	
+	OUTLINER_ANIMOP_CLEAR_ADT,
+	
 	OUTLINER_ANIMOP_SET_ACT,
 	OUTLINER_ANIMOP_CLEAR_ACT,
 	
@@ -1042,6 +1361,7 @@ typedef enum eOutliner_AnimDataOps {
 } eOutliner_AnimDataOps;
 
 static EnumPropertyItem prop_animdata_op_types[] = {
+	{OUTLINER_ANIMOP_CLEAR_ADT, "CLEAR_ANIMDATA", 0, "Clear Animation Data", "Remove this animation data container"},
 	{OUTLINER_ANIMOP_SET_ACT, "SET_ACT", 0, "Set Action", ""},
 	{OUTLINER_ANIMOP_CLEAR_ACT, "CLEAR_ACT", 0, "Unlink Action", ""},
 	{OUTLINER_ANIMOP_REFRESH_DRV, "REFRESH_DRIVERS", 0, "Refresh Drivers", ""},
@@ -1070,6 +1390,14 @@ static int outliner_animdata_operation_exec(bContext *C, wmOperator *op)
 	
 	/* perform the core operation */
 	switch (event) {
+		case OUTLINER_ANIMOP_CLEAR_ADT:
+			/* Remove Animation Data - this may remove the active action, in some cases... */
+			outliner_do_data_operation(soops, datalevel, event, &soops->tree, clear_animdata_cb, NULL);
+			
+			WM_event_add_notifier(C, NC_ANIMATION | ND_NLA_ACTCHANGE, NULL);
+			ED_undo_push(C, "Clear Animation Data");
+			break;
+		
 		case OUTLINER_ANIMOP_SET_ACT:
 			/* delegate once again... */
 			WM_operator_name_call(C, "OUTLINER_OT_action_set", WM_OP_INVOKE_REGION_WIN, NULL);
@@ -1105,14 +1433,8 @@ static int outliner_animdata_operation_exec(bContext *C, wmOperator *op)
 	
 	/* update dependencies */
 	if (updateDeps) {
-		Main *bmain = CTX_data_main(C);
-		Scene *scene = CTX_data_scene(C);
-		
 		/* rebuild depsgraph for the new deps */
-		DAG_scene_sort(bmain, scene);
-		
-		/* force an update of depsgraph */
-		DAG_ids_flush_update(bmain, 0);
+		DAG_relations_tag_update(CTX_data_main(C));
 	}
 	
 	return OPERATOR_FINISHED;
@@ -1138,12 +1460,105 @@ void OUTLINER_OT_animdata_operation(wmOperatorType *ot)
 
 /* **************************************** */
 
+static EnumPropertyItem prop_constraint_op_types[] = {
+	{OL_CONSTRAINTOP_ENABLE, "ENABLE", ICON_RESTRICT_VIEW_OFF, "Enable", ""},
+	{OL_CONSTRAINTOP_DISABLE, "DISABLE", ICON_RESTRICT_VIEW_ON, "Disable", ""},
+	{OL_CONSTRAINTOP_DELETE, "DELETE", ICON_X, "Delete", ""},
+	{0, NULL, 0, NULL, NULL}
+};
+
+static int outliner_constraint_operation_exec(bContext *C, wmOperator *op)
+{
+	SpaceOops *soops = CTX_wm_space_outliner(C);
+	int scenelevel = 0, objectlevel = 0, idlevel = 0, datalevel = 0;
+	eOutliner_PropConstraintOps event;
+
+	event = RNA_enum_get(op->ptr, "type");
+	set_operation_types(soops, &soops->tree, &scenelevel, &objectlevel, &idlevel, &datalevel);
+
+	outliner_do_data_operation(soops, datalevel, event, &soops->tree, constraint_cb, C);
+
+	if (event == OL_CONSTRAINTOP_DELETE) {
+		outliner_cleanup_tree(soops);
+	}
+
+	ED_undo_push(C, "Constraint operation");
+
+	return OPERATOR_FINISHED;
+}
+
+void OUTLINER_OT_constraint_operation(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Outliner Constraint Operation";
+	ot->idname = "OUTLINER_OT_constraint_operation";
+	ot->description = "";
+
+	/* callbacks */
+	ot->invoke = WM_menu_invoke;
+	ot->exec = outliner_constraint_operation_exec;
+	ot->poll = ED_operator_outliner_active;
+
+	ot->flag = 0;
+
+	ot->prop = RNA_def_enum(ot->srna, "type", prop_constraint_op_types, 0, "Constraint Operation", "");
+}
+
+/* ******************** */
+
+static EnumPropertyItem prop_modifier_op_types[] = {
+	{OL_MODIFIER_OP_TOGVIS, "TOGVIS", ICON_RESTRICT_VIEW_OFF, "Toggle viewport use", ""},
+	{OL_MODIFIER_OP_TOGREN, "TOGREN", ICON_RESTRICT_RENDER_OFF, "Toggle render use", ""},
+	{OL_MODIFIER_OP_DELETE, "DELETE", ICON_X, "Delete", ""},
+	{0, NULL, 0, NULL, NULL}
+};
+
+static int outliner_modifier_operation_exec(bContext *C, wmOperator *op)
+{
+	SpaceOops *soops = CTX_wm_space_outliner(C);
+	int scenelevel = 0, objectlevel = 0, idlevel = 0, datalevel = 0;
+	eOutliner_PropModifierOps event;
+
+	event = RNA_enum_get(op->ptr, "type");
+	set_operation_types(soops, &soops->tree, &scenelevel, &objectlevel, &idlevel, &datalevel);
+
+	outliner_do_data_operation(soops, datalevel, event, &soops->tree, modifier_cb, C);
+
+	if (event == OL_MODIFIER_OP_DELETE) {
+		outliner_cleanup_tree(soops);
+	}
+
+	ED_undo_push(C, "Modifier operation");
+
+	return OPERATOR_FINISHED;
+}
+
+void OUTLINER_OT_modifier_operation(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Outliner Modifier Operation";
+	ot->idname = "OUTLINER_OT_modifier_operation";
+	ot->description = "";
+
+	/* callbacks */
+	ot->invoke = WM_menu_invoke;
+	ot->exec = outliner_modifier_operation_exec;
+	ot->poll = ED_operator_outliner_active;
+
+	ot->flag = 0;
+
+	ot->prop = RNA_def_enum(ot->srna, "type", prop_modifier_op_types, 0, "Modifier Operation", "");
+}
+
+/* ******************** */
+
+// XXX: select linked is for RNA structs only
 static EnumPropertyItem prop_data_op_types[] = {
-	{1, "SELECT", 0, "Select", ""},
-	{2, "DESELECT", 0, "Deselect", ""},
-	{3, "HIDE", 0, "Hide", ""},
-	{4, "UNHIDE", 0, "Unhide", ""},
-	{5, "SELECT_LINKED", 0, "Select Linked", ""},
+	{OL_DOP_SELECT, "SELECT", 0, "Select", ""},
+	{OL_DOP_DESELECT, "DESELECT", 0, "Deselect", ""},
+	{OL_DOP_HIDE, "HIDE", 0, "Hide", ""},
+	{OL_DOP_UNHIDE, "UNHIDE", 0, "Unhide", ""},
+	{OL_DOP_SELECT_LINKED, "SELECT_LINKED", 0, "Select Linked", ""},
 	{0, NULL, 0, NULL, NULL}
 };
 
@@ -1151,7 +1566,7 @@ static int outliner_data_operation_exec(bContext *C, wmOperator *op)
 {
 	SpaceOops *soops = CTX_wm_space_outliner(C);
 	int scenelevel = 0, objectlevel = 0, idlevel = 0, datalevel = 0;
-	int event;
+	eOutliner_PropDataOps event;
 	
 	/* check for invalid states */
 	if (soops == NULL)
@@ -1160,48 +1575,55 @@ static int outliner_data_operation_exec(bContext *C, wmOperator *op)
 	event = RNA_enum_get(op->ptr, "type");
 	set_operation_types(soops, &soops->tree, &scenelevel, &objectlevel, &idlevel, &datalevel);
 	
-	if (event <= 0)
-		return OPERATOR_CANCELLED;
-	
 	switch (datalevel) {
 		case TSE_POSE_CHANNEL:
 		{
 			outliner_do_data_operation(soops, datalevel, event, &soops->tree, pchan_cb, NULL);
 			WM_event_add_notifier(C, NC_OBJECT | ND_POSE, NULL);
 			ED_undo_push(C, "PoseChannel operation");
-		}
+
 			break;
-		
+		}
 		case TSE_BONE:
 		{
 			outliner_do_data_operation(soops, datalevel, event, &soops->tree, bone_cb, NULL);
 			WM_event_add_notifier(C, NC_OBJECT | ND_POSE, NULL);
 			ED_undo_push(C, "Bone operation");
-		}
+
 			break;
-			
+		}
 		case TSE_EBONE:
 		{
 			outliner_do_data_operation(soops, datalevel, event, &soops->tree, ebone_cb, NULL);
 			WM_event_add_notifier(C, NC_OBJECT | ND_POSE, NULL);
 			ED_undo_push(C, "EditBone operation");
-		}
+
 			break;
-			
+		}
 		case TSE_SEQUENCE:
 		{
 			Scene *scene = CTX_data_scene(C);
 			outliner_do_data_operation(soops, datalevel, event, &soops->tree, sequence_cb, scene);
-		}
+
 			break;
-			
+		}
+		case TSE_GP_LAYER:
+		{
+			outliner_do_data_operation(soops, datalevel, event, &soops->tree, gp_layer_cb, NULL);
+			WM_event_add_notifier(C, NC_GPENCIL | ND_DATA, NULL);
+			ED_undo_push(C, "Grease Pencil Layer operation");
+
+			break;
+		}
 		case TSE_RNA_STRUCT:
-			if (event == 5) {
+			if (event == OL_DOP_SELECT_LINKED) {
 				outliner_do_data_operation(soops, datalevel, event, &soops->tree, data_select_linked_cb, C);
 			}
+
 			break;
-			
+
 		default:
+			BKE_report(op->reports, RPT_WARNING, "Not yet implemented");
 			break;
 	}
 	
@@ -1231,7 +1653,7 @@ void OUTLINER_OT_data_operation(wmOperatorType *ot)
 
 
 static int do_outliner_operation_event(bContext *C, Scene *scene, ARegion *ar, SpaceOops *soops,
-                                       TreeElement *te, wmEvent *event, const float mval[2])
+                                       TreeElement *te, const wmEvent *event, const float mval[2])
 {
 	ReportList *reports = CTX_wm_reports(C); // XXX...
 	
@@ -1242,7 +1664,7 @@ static int do_outliner_operation_event(bContext *C, Scene *scene, ARegion *ar, S
 		/* select object that's clicked on and popup context menu */
 		if (!(tselem->flag & TSE_SELECTED)) {
 			
-			if (outliner_has_one_flag(soops, &soops->tree, TSE_SELECTED, 1) )
+			if (outliner_has_one_flag(soops, &soops->tree, TSE_SELECTED, 1))
 				outliner_set_flag(soops, &soops->tree, TSE_SELECTED, 0);
 			
 			tselem->flag |= TSE_SELECTED;
@@ -1254,14 +1676,20 @@ static int do_outliner_operation_event(bContext *C, Scene *scene, ARegion *ar, S
 		set_operation_types(soops, &soops->tree, &scenelevel, &objectlevel, &idlevel, &datalevel);
 		
 		if (scenelevel) {
-			//if (objectlevel || datalevel || idlevel) error("Mixed selection");
-			//else pupmenu("Scene Operations%t|Delete");
+			if (objectlevel || datalevel || idlevel) {
+				BKE_report(reports, RPT_WARNING, "Mixed selection");
+			}
+			else {
+				WM_operator_name_call(C, "OUTLINER_OT_scene_operation", WM_OP_INVOKE_REGION_WIN, NULL);
+			}
 		}
 		else if (objectlevel) {
 			WM_operator_name_call(C, "OUTLINER_OT_object_operation", WM_OP_INVOKE_REGION_WIN, NULL);
 		}
 		else if (idlevel) {
-			if (idlevel == -1 || datalevel) BKE_report(reports, RPT_WARNING, "Mixed selection");
+			if (idlevel == -1 || datalevel) {
+				BKE_report(reports, RPT_WARNING, "Mixed selection");
+			}
 			else {
 				if (idlevel == ID_GR)
 					WM_operator_name_call(C, "OUTLINER_OT_group_operation", WM_OP_INVOKE_REGION_WIN, NULL);
@@ -1270,16 +1698,30 @@ static int do_outliner_operation_event(bContext *C, Scene *scene, ARegion *ar, S
 			}
 		}
 		else if (datalevel) {
-			if (datalevel == -1) BKE_report(reports, RPT_WARNING, "Mixed selection");
+			if (datalevel == -1) {
+				BKE_report(reports, RPT_WARNING, "Mixed selection");
+			}
 			else {
 				if (datalevel == TSE_ANIM_DATA)
 					WM_operator_name_call(C, "OUTLINER_OT_animdata_operation", WM_OP_INVOKE_REGION_WIN, NULL);
-				else if (datalevel == TSE_DRIVER_BASE)
-					/* do nothing... no special ops needed yet */;
-				else if (ELEM3(datalevel, TSE_R_LAYER_BASE, TSE_R_LAYER, TSE_R_PASS))
-					/*WM_operator_name_call(C, "OUTLINER_OT_renderdata_operation", WM_OP_INVOKE_REGION_WIN, NULL)*/;
-				else
+				else if (datalevel == TSE_DRIVER_BASE) {
+					/* do nothing... no special ops needed yet */
+				}
+				else if (ELEM(datalevel, TSE_R_LAYER_BASE, TSE_R_LAYER, TSE_R_PASS)) {
+					/*WM_operator_name_call(C, "OUTLINER_OT_renderdata_operation", WM_OP_INVOKE_REGION_WIN, NULL)*/
+				}
+				else if (datalevel == TSE_ID_BASE) {
+					/* do nothing... there are no ops needed here yet */
+				}
+				else if (datalevel == TSE_CONSTRAINT) {
+					WM_operator_name_call(C, "OUTLINER_OT_constraint_operation", WM_OP_INVOKE_REGION_WIN, NULL);
+				}
+				else if (datalevel == TSE_MODIFIER) {
+					WM_operator_name_call(C, "OUTLINER_OT_modifier_operation", WM_OP_INVOKE_REGION_WIN, NULL);
+				}
+				else {
 					WM_operator_name_call(C, "OUTLINER_OT_data_operation", WM_OP_INVOKE_REGION_WIN, NULL);
+				}
 			}
 		}
 		
@@ -1294,18 +1736,25 @@ static int do_outliner_operation_event(bContext *C, Scene *scene, ARegion *ar, S
 }
 
 
-static int outliner_operation(bContext *C, wmOperator *UNUSED(op), wmEvent *event)
+static int outliner_operation(bContext *C, wmOperator *UNUSED(op), const wmEvent *event)
 {
 	Scene *scene = CTX_data_scene(C);
 	ARegion *ar = CTX_wm_region(C);
 	SpaceOops *soops = CTX_wm_space_outliner(C);
+	uiBut *but = UI_context_active_but_get(C);
 	TreeElement *te;
 	float fmval[2];
-	
-	UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1], fmval, fmval + 1);
+
+	if (but) {
+		UI_but_tooltip_timer_remove(C, but);
+	}
+
+	UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1], &fmval[0], &fmval[1]);
 	
 	for (te = soops->tree.first; te; te = te->next) {
-		if (do_outliner_operation_event(C, scene, ar, soops, te, event, fmval)) break;
+		if (do_outliner_operation_event(C, scene, ar, soops, te, event, fmval)) {
+			break;
+		}
 	}
 	
 	return OPERATOR_FINISHED;
