@@ -88,14 +88,14 @@ Controller::Controller()
 	_RootNode = new NodeGroup;
 	_RootNode->addRef();
 
-	_SilhouetteNode = NULL;
 #if 0
+	_SilhouetteNode = NULL;
 	_ProjectedSilhouette = NULL;
 	_VisibleProjectedSilhouette = NULL;
-#endif
 
 	_DebugNode = new NodeGroup;
 	_DebugNode->addRef();
+#endif
 
 	_winged_edge = NULL;
 
@@ -142,6 +142,7 @@ Controller::~Controller()
 			delete _RootNode;
 	}
 
+#if 0
 	if (NULL != _SilhouetteNode) {
 		int ref = _SilhouetteNode->destroy();
 		if (0 == ref)
@@ -153,6 +154,7 @@ Controller::~Controller()
 		if (0 == ref)
 			delete _DebugNode;
 	}
+#endif
 
 	if (_winged_edge) {
 		delete _winged_edge;
@@ -295,7 +297,7 @@ int Controller::LoadMesh(Render *re, SceneRenderLayer *srl)
 		}
 		cam->setProjectionMatrix(proj);
 		_RootNode->AddChild(cam);
-		_RootNode->AddChild(new NodeSceneRenderLayer(*srl));
+		_RootNode->AddChild(new NodeSceneRenderLayer(*re->scene, *srl));
 
 		sceneHashFunc.reset();
 		//blenderScene->accept(sceneHashFunc);
@@ -409,6 +411,7 @@ void Controller::DeleteWingedEdge()
 
 void Controller::DeleteViewMap(bool freeCache)
 {
+#if 0
 	_pView->DetachSilhouette();
 	if (NULL != _SilhouetteNode) {
 		int ref = _SilhouetteNode->destroy();
@@ -418,7 +421,6 @@ void Controller::DeleteViewMap(bool freeCache)
 		}
 	}
 
-#if 0
 	if (NULL != _ProjectedSilhouette) {
 		int ref = _ProjectedSilhouette->destroy();
 		if (0 == ref) {
@@ -433,7 +435,6 @@ void Controller::DeleteViewMap(bool freeCache)
 			_VisibleProjectedSilhouette = NULL;
 		}
 	}
-#endif
 
 	_pView->DetachDebug();
 	if (NULL != _DebugNode) {
@@ -441,6 +442,7 @@ void Controller::DeleteViewMap(bool freeCache)
 		if (0 == ref)
 			_DebugNode->addRef();
 	}
+#endif
 
 	if (NULL != _ViewMap) {
 		if (freeCache || !_EnableViewMapCache) {
@@ -469,7 +471,7 @@ void Controller::ComputeViewMap()
 	// Restore the context of view:
 	// we need to perform all these operations while the 
 	// 3D context is on.
-	Vec3r vp(freestyle_viewpoint[0], freestyle_viewpoint[1], freestyle_viewpoint[2]);
+	Vec3f vp(freestyle_viewpoint[0], freestyle_viewpoint[1], freestyle_viewpoint[2]);
 
 #if 0
 	if (G.debug & G_DEBUG_FREESTYLE) {
@@ -533,7 +535,7 @@ void Controller::ComputeViewMap()
 	}
 	_Chrono.start();
 
-	edgeDetector.setViewpoint(Vec3r(vp));
+	edgeDetector.setViewpoint(vp);
 	edgeDetector.enableOrthographicProjection(proj[3][3] != 0.0);
 	edgeDetector.enableRidgesAndValleysFlag(_ComputeRidges);
 	edgeDetector.enableSuggestiveContours(_ComputeSuggestive);
@@ -557,20 +559,20 @@ void Controller::ComputeViewMap()
 	//----------------------------------------------------------
 	ViewMapBuilder vmBuilder;
 	vmBuilder.setEnableQI(_EnableQI);
-	vmBuilder.setViewpoint(Vec3r(vp));
+	vmBuilder.setViewpoint(vp);
 	vmBuilder.setTransform(mv, proj, viewport, _pView->GetFocalLength(), _pView->GetAspect(), _pView->GetFovyRadian());
 	vmBuilder.setFrustum(_pView->znear(), _pView->zfar());
 	vmBuilder.setGrid(&_Grid);
 	vmBuilder.setRenderMonitor(_pRenderMonitor);
 
+#if 0
 	// Builds a tesselated form of the silhouette for display purpose:
 	//---------------------------------------------------------------
 	ViewMapTesselator3D sTesselator3d;
-#if 0
 	ViewMapTesselator2D sTesselator2d;
 	sTesselator2d.setNature(_edgeTesselationNature);
-#endif
 	sTesselator3d.setNature(_edgeTesselationNature);
+#endif
 
 	if (G.debug & G_DEBUG_FREESTYLE) {
 		cout << "\n===  Building the view map  ===" << endl;
@@ -584,12 +586,12 @@ void Controller::ComputeViewMap()
 		printf("ViewMap edge count : %i\n", _ViewMap->viewedges_size());
 	}
 
+#if 0
 	// Tesselate the 3D edges:
 	_SilhouetteNode = sTesselator3d.Tesselate(_ViewMap);
 	_SilhouetteNode->addRef();
 
 	// Tesselate 2D edges
-#if 0
 	_ProjectedSilhouette = sTesselator2d.Tesselate(_ViewMap);
 	_ProjectedSilhouette->addRef();
 #endif
@@ -599,13 +601,13 @@ void Controller::ComputeViewMap()
 		printf("ViewMap building : %lf\n", duration);
 	}
 
-	_pView->AddSilhouette(_SilhouetteNode);
 #if 0
+	_pView->AddSilhouette(_SilhouetteNode);
 	_pView->AddSilhouette(_WRoot);
 	_pView->Add2DSilhouette(_ProjectedSilhouette);
 	_pView->Add2DVisibleSilhouette(_VisibleProjectedSilhouette);
-#endif
 	_pView->AddDebug(_DebugNode);
+#endif
 
 	// Draw the steerable density map:
 	//--------------------------------
@@ -869,7 +871,7 @@ void Controller::DrawStrokes()
 	real d = _Chrono.stop();
 	if (G.debug & G_DEBUG_FREESTYLE) {
 		cout << "Strokes generation  : " << d << endl;
-		cout << "Stroke count  : " << _Canvas->stroke_count << endl;
+		cout << "Stroke count  : " << _Canvas->getStrokeCount() << endl;
 	}
 	resetModified();
 	DeleteViewMap();
@@ -882,10 +884,13 @@ void Controller::ResetRenderCount()
 
 Render *Controller::RenderStrokes(Render *re, bool render)
 {
+	int totmesh = 0;
 	_Chrono.start();
 	BlenderStrokeRenderer *blenderRenderer = new BlenderStrokeRenderer(re, ++_render_count);
-	if (render)
+	if (render) {
 		_Canvas->Render(blenderRenderer);
+		totmesh = blenderRenderer->GenerateScene();
+	}
 	real d = _Chrono.stop();
 	if (G.debug & G_DEBUG_FREESTYLE) {
 		cout << "Temporary scene generation: " << d << endl;
@@ -904,8 +909,8 @@ Render *Controller::RenderStrokes(Render *re, bool render)
 		float mmap_used_memory = (mmap_in_use) / (1024.0 * 1024.0);
 		float megs_peak_memory = (peak_memory) / (1024.0 * 1024.0);
 
-		printf("%d verts, %d faces, mem %.2fM (%.2fM, peak %.2fM)\n",
-		       freestyle_render->i.totvert, freestyle_render->i.totface,
+		printf("%d objs, %d verts, %d faces, mem %.2fM (%.2fM, peak %.2fM)\n",
+		       totmesh, freestyle_render->i.totvert, freestyle_render->i.totface,
 		       megs_used_memory, mmap_used_memory, megs_peak_memory);
 	}
 	delete blenderRenderer;
@@ -924,13 +929,19 @@ void Controller::InsertStyleModule(unsigned index, const char *iFileName)
 	_Canvas->InsertStyleModule(index, sm);
 }
 
+void Controller::InsertStyleModule(unsigned index, const char *iName, const char *iBuffer)
+{
+	StyleModule *sm = new BufferedStyleModule(iBuffer, iName, _inter);
+	_Canvas->InsertStyleModule(index, sm);
+}
+
 void Controller::InsertStyleModule(unsigned index, const char *iName, struct Text *iText)
 {
 	StyleModule *sm = new BlenderStyleModule(iText, iName, _inter);
 	_Canvas->InsertStyleModule(index, sm);
 }
 
-void Controller::AddStyleModule(const char *iFileName)
+void Controller::AddStyleModule(const char * /*iFileName*/)
 {
 	//_pStyleWindow->Add(iFileName);
 }
@@ -1000,7 +1011,7 @@ void Controller::toggleEdgeTesselationNature(Nature::EdgeNature iNature)
 	ComputeViewMap();
 }
 
-void Controller::setModelsDir(const string& dir)
+void Controller::setModelsDir(const string& /*dir*/)
 {
 	//_current_dirs->setValue("models/dir", dir);
 }
@@ -1012,7 +1023,7 @@ string Controller::getModelsDir() const
 	return dir;
 }
 
-void Controller::setModulesDir(const string& dir)
+void Controller::setModulesDir(const string& /*dir*/)
 {
 	//_current_dirs->setValue("modules/dir", dir);
 }
