@@ -28,6 +28,8 @@
 #ifndef __IMB_IMBUF_TYPES_H__
 #define __IMB_IMBUF_TYPES_H__
 
+#include "DNA_vec_types.h"  /* for rcti */
+
 /**
  * \file IMB_imbuf_types.h
  * \ingroup imbuf
@@ -45,10 +47,8 @@
  * contains an Amiga-format file).
  */
 
-struct ImMetaData;
-
-#define IB_MIPMAP_LEVELS	20
-#define IB_FILENAME_SIZE	1024
+#define IMB_MIPMAP_LEVELS	20
+#define IMB_FILENAME_SIZE	1024
 
 typedef struct DDSData {
 	unsigned int fourcc; /* DDS fourcc info */
@@ -65,6 +65,80 @@ typedef struct DDSData {
  * Also; add new variables to the end to save pain!
  *
  */
+
+/* ibuf->ftype flag, main image types */
+enum eImbTypes {
+	IMB_FTYPE_PNG = 1,
+	IMB_FTYPE_TGA,
+	IMB_FTYPE_JPG,
+	IMB_FTYPE_BMP,
+	IMB_FTYPE_OPENEXR,
+	IMB_FTYPE_IMAGIC,
+#ifdef WITH_OPENIMAGEIO
+	IMB_FTYPE_PSD,
+#endif
+#ifdef WITH_OPENJPEG
+	IMB_FTYPE_JP2,
+#endif
+#ifdef WITH_HDR
+	IMB_FTYPE_RADHDR,
+#endif
+#ifdef WITH_TIFF
+	IMB_FTYPE_TIF,
+#endif
+#ifdef WITH_CINEON
+	IMB_FTYPE_CINEON,
+	IMB_FTYPE_DPX,
+#endif
+
+#ifdef WITH_DDS
+	IMB_FTYPE_DDS,
+#endif
+};
+
+/* ibuf->foptions flag, type specific options.
+ * Some formats include compression rations on some bits */
+
+#define OPENEXR_HALF	(1 << 8 )
+/* careful changing this, it's used in DNA as well */
+#define OPENEXR_COMPRESS (15)
+
+#ifdef WITH_CINEON
+#define CINEON_LOG		(1 << 8)
+#define CINEON_16BIT	(1 << 7)
+#define CINEON_12BIT	(1 << 6)
+#define CINEON_10BIT	(1 << 5)
+#endif
+
+#ifdef WITH_OPENJPEG
+#define JP2_12BIT		(1 << 9)
+#define JP2_16BIT		(1 << 8)
+#define JP2_YCC			(1 << 7)
+#define JP2_CINE		(1 << 6)
+#define JP2_CINE_48FPS	(1 << 5)
+#define JP2_JP2	(1 << 4)
+#define JP2_J2K	(1 << 3)
+#endif
+
+#define PNG_16BIT			(1 << 10)
+
+#define RAWTGA	        1
+
+#define JPG_STD	        0
+#define JPG_VID	        1
+#define JPG_JST	        2
+#define JPG_MAX	        3
+#define JPG_MSK	        0x03
+
+#ifdef WITH_TIFF
+#define TIF_16BIT		(1 << 8 )
+#endif
+
+typedef struct ImbFormatOptions {
+	short flag;
+	char quality; /* quality serves dual purpose as quality number for jpeg or compression amount for png */
+} ImbFormatOptions;
+
 typedef struct ImBuf {
 	struct ImBuf *next, *prev;	/**< allow lists of ImBufs, for caches or flipbooks */
 
@@ -93,7 +167,7 @@ typedef struct ImBuf {
 	/* tiled pixel storage */
 	int tilex, tiley;
 	int xtiles, ytiles;
-	unsigned int **tiles;	
+	unsigned int **tiles;
 
 	/* zbuffer */
 	int	*zbuf;				/* z buffer data, original zbuffer */
@@ -103,19 +177,20 @@ typedef struct ImBuf {
 	float dither;				/* random dither value, for conversion from float -> byte rect */
 
 	/* mipmapping */
-	struct ImBuf *mipmap[IB_MIPMAP_LEVELS]; /* MipMap levels, a series of halved images */
+	struct ImBuf *mipmap[IMB_MIPMAP_LEVELS]; /* MipMap levels, a series of halved images */
 	int miptot, miplevel;
 
 	/* externally used data */
 	int index;						/* reference index for ImBuf lists */
 	int	userflags;					/* used to set imbuf to dirty and other stuff */
-	struct ImMetaData *metadata;	/* image metadata */
-	void *userdata;					/* temporary storage, only used by baking at the moment */
+	struct IDProperty *metadata;	/* image metadata */
+	void *userdata;					/* temporary storage */
 
 	/* file information */
-	int	ftype;							/* file type we are going to save as */
-	char name[IB_FILENAME_SIZE];		/* filename associated with this image */
-	char cachename[IB_FILENAME_SIZE];	/* full filename used for reading from cache */
+	enum eImbTypes	ftype;				/* file type we are going to save as */
+	ImbFormatOptions foptions;			/* file format specific flags */
+	char name[IMB_FILENAME_SIZE];		/* filename associated with this image */
+	char cachename[IMB_FILENAME_SIZE];	/* full filename used for reading from cache */
 
 	/* memory cache limiter */
 	struct MEM_CacheLimiterHandle_s *c_handle; /* handle for cache limiter */
@@ -132,6 +207,7 @@ typedef struct ImBuf {
 	unsigned int *display_buffer_flags;          /* array of per-display display buffers dirty flags */
 	struct ColormanageCache *colormanage_cache;  /* cache used by color management */
 	int colormanage_flag;
+	rcti invalid_rect;
 
 	/* information for compressed textures */
 	struct DDSData dds_data;
@@ -147,6 +223,7 @@ typedef struct ImBuf {
 #define IB_MIPMAP_INVALID		(1 << 2)	/* image mipmaps are invalid, need recreate */
 #define IB_RECT_INVALID			(1 << 3)	/* float buffer changed, needs recreation of byte rect */
 #define IB_DISPLAY_BUFFER_INVALID	(1 << 4)	/* either float or byte buffer changed, need to re-calculate display buffers */
+#define IB_PERSISTENT				(1 << 5)	/* image buffer is persistent in the memory and should never be removed from the cache */
 
 /**
  * \name Imbuf Component flags
@@ -167,63 +244,11 @@ typedef struct ImBuf {
 #define IB_animdeinterlace	(1 << 9)
 #define IB_tiles			(1 << 10)
 #define IB_tilecache		(1 << 11)
-#define IB_premul			(1 << 12)
-#define IB_cm_predivide		(1 << 13)
-
-/*
- * The bit flag is stored in the ImBuf.ftype variable.
- * Note that the lower 10 bits is used for storing custom flags
- */
-#define IB_CUSTOM_FLAGS_MASK 0x3ff
-
-#define PNG				(1 << 30)
-#define TGA				(1 << 28)
-#define JPG				(1 << 27)
-#define BMP				(1 << 26)
-
-#ifdef WITH_QUICKTIME
-#define QUICKTIME		(1 << 25)
-#endif
-
-#ifdef WITH_HDR
-#define RADHDR			(1 << 24)
-#endif
-#ifdef WITH_TIFF
-#define TIF				(1 << 23)
-#define TIF_16BIT		(1 << 8 )
-#endif
-
-#define OPENEXR			(1 << 22)
-#define OPENEXR_HALF	(1 << 8 )
-#define OPENEXR_COMPRESS (7)	
-
-#ifdef WITH_CINEON
-#define CINEON			(1 << 21)
-#define DPX				(1 << 20)
-#endif
-
-#ifdef WITH_DDS
-#define DDS				(1 << 19)
-#endif
-
-#ifdef WITH_OPENJPEG
-#define JP2				(1 << 18)
-#define JP2_12BIT		(1 << 17)
-#define JP2_16BIT		(1 << 16)
-#define JP2_YCC			(1 << 15)
-#define JP2_CINE		(1 << 14)
-#define JP2_CINE_48FPS	(1 << 13) 
-#endif
-
-#define RAWTGA	        (TGA | 1)
-
-#define JPG_STD	        (JPG | (0 << 8))
-#define JPG_VID	        (JPG | (1 << 8))
-#define JPG_JST	        (JPG | (2 << 8))
-#define JPG_MAX	        (JPG | (3 << 8))
-#define JPG_MSK	        (0xffffff00)
-
-#define IMAGIC			0732
+#define IB_alphamode_premul	(1 << 12)  /* indicates whether image on disk have premul alpha */
+#define IB_alphamode_detect	(1 << 13)  /* if this flag is set, alpha mode would be guessed from file */
+#define IB_ignore_alpha		(1 << 14)  /* ignore alpha on load and substitude it with 1.0f */
+#define IB_thumbnail		(1 << 15)
+#define IB_multiview		(1 << 16)
 
 /**
  * \name Imbuf preset profile tags
@@ -236,24 +261,24 @@ typedef struct ImBuf {
 
 /* dds */
 #ifdef WITH_DDS
-#ifndef MAKEFOURCC
-#define MAKEFOURCC(ch0, ch1, ch2, ch3)\
+#ifndef DDS_MAKEFOURCC
+#define DDS_MAKEFOURCC(ch0, ch1, ch2, ch3)\
 	((unsigned long)(unsigned char)(ch0) | \
 	((unsigned long)(unsigned char)(ch1) << 8) | \
 	((unsigned long)(unsigned char)(ch2) << 16) | \
 	((unsigned long)(unsigned char)(ch3) << 24))
-#endif  /* MAKEFOURCC */
+#endif  /* DDS_MAKEFOURCC */
 
 /*
  * FOURCC codes for DX compressed-texture pixel formats
  */
 
-#define FOURCC_DDS   (MAKEFOURCC('D','D','S',' '))
-#define FOURCC_DXT1  (MAKEFOURCC('D','X','T','1'))
-#define FOURCC_DXT2  (MAKEFOURCC('D','X','T','2'))
-#define FOURCC_DXT3  (MAKEFOURCC('D','X','T','3'))
-#define FOURCC_DXT4  (MAKEFOURCC('D','X','T','4'))
-#define FOURCC_DXT5  (MAKEFOURCC('D','X','T','5'))
+#define FOURCC_DDS   (DDS_MAKEFOURCC('D','D','S',' '))
+#define FOURCC_DXT1  (DDS_MAKEFOURCC('D','X','T','1'))
+#define FOURCC_DXT2  (DDS_MAKEFOURCC('D','X','T','2'))
+#define FOURCC_DXT3  (DDS_MAKEFOURCC('D','X','T','3'))
+#define FOURCC_DXT4  (DDS_MAKEFOURCC('D','X','T','4'))
+#define FOURCC_DXT5  (DDS_MAKEFOURCC('D','X','T','5'))
 
 #endif  /* DDS */
 extern const char *imb_ext_image[];
@@ -261,6 +286,9 @@ extern const char *imb_ext_image_qt[];
 extern const char *imb_ext_movie[];
 extern const char *imb_ext_audio[];
 
+/* image formats that can only be loaded via filepath */
+extern const char *imb_ext_image_filepath_only[];
+
 enum {
 	IMB_COLORMANAGE_IS_DATA = (1 << 0)
 };
