@@ -37,9 +37,12 @@
 
 #include "MEM_guardedalloc.h"
 
+#include "BLI_utildefines.h"
 #include "BLI_string.h"
 #include "BLI_fileops.h"
 
+#include "BKE_idprop.h"
+
 #include "imbuf.h"
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
@@ -52,18 +55,18 @@
 #include "IMB_colormanagement_intern.h"
 
 // #define IS_jpg(x)       (x->ftype & JPG) // UNUSED
-#define IS_stdjpg(x)    ((x->ftype & JPG_MSK) == JPG_STD)
-// #define IS_vidjpg(x)    ((x->ftype & JPG_MSK) == JPG_VID) // UNUSED
-#define IS_jstjpg(x)    ((x->ftype & JPG_MSK) == JPG_JST)
-#define IS_maxjpg(x)    ((x->ftype & JPG_MSK) == JPG_MAX)
+#define IS_stdjpg(x)    ((x->foptions.flag & JPG_MSK) == JPG_STD)
+// #define IS_vidjpg(x)    ((x->foptions & JPG_MSK) == JPG_VID) // UNUSED
+#define IS_jstjpg(x)    ((x->foptions.flag & JPG_MSK) == JPG_JST)
+#define IS_maxjpg(x)    ((x->foptions.flag & JPG_MSK) == JPG_MAX)
 
 /* the types are from the jpeg lib */
-static void jpeg_error(j_common_ptr cinfo);
+static void jpeg_error(j_common_ptr cinfo) ATTR_NORETURN;
 static void init_source(j_decompress_ptr cinfo);
 static boolean fill_input_buffer(j_decompress_ptr cinfo);
 static void skip_input_data(j_decompress_ptr cinfo, long num_bytes);
 static void term_source(j_decompress_ptr cinfo);
-static void memory_source(j_decompress_ptr cinfo, unsigned char *buffer, size_t size);
+static void memory_source(j_decompress_ptr cinfo, const unsigned char *buffer, size_t size);
 static boolean handle_app1(j_decompress_ptr cinfo);
 static ImBuf *ibJpegImageFromCinfo(struct jpeg_decompress_struct *cinfo, int flags);
 
@@ -82,9 +85,9 @@ static ImBuf *ibJpegImageFromCinfo(struct jpeg_decompress_struct *cinfo, int fla
  */
 
 static int jpeg_default_quality;
-static int ibuf_ftype;
+static int ibuf_foptions;
 
-int imb_is_a_jpeg(unsigned char *mem)
+int imb_is_a_jpeg(const unsigned char *mem)
 {
 	if ((mem[0] == 0xFF) && (mem[1] == 0xD8)) return 1;
 	return 0;
@@ -120,15 +123,17 @@ static void jpeg_error(j_common_ptr cinfo)
  * INPUT HANDLER FROM MEMORY
  *---------------------------------------------------------- */
 
+#if 0
 typedef struct {
 	unsigned char  *buffer;
 	int             filled;
 } buffer_struct;
+#endif
 
 typedef struct {
 	struct jpeg_source_mgr pub; /* public fields */
 
-	unsigned char  *buffer;
+	const unsigned char  *buffer;
 	int             size;
 	JOCTET          terminal[2];
 } my_source_mgr;
@@ -154,7 +159,7 @@ static boolean fill_input_buffer(j_decompress_ptr cinfo)
 	src->terminal[0] = (JOCTET) 0xFF;
 	src->terminal[1] = (JOCTET) JPEG_EOI;
 
-	return TRUE;
+	return true;
 }
 
 
@@ -177,7 +182,7 @@ static void term_source(j_decompress_ptr cinfo)
 	(void)cinfo; /* unused */
 }
 
-static void memory_source(j_decompress_ptr cinfo, unsigned char *buffer, size_t size)
+static void memory_source(j_decompress_ptr cinfo, const unsigned char *buffer, size_t size)
 {
 	my_src_ptr src;
 
@@ -231,27 +236,26 @@ static void memory_source(j_decompress_ptr cinfo, unsigned char *buffer, size_t
 
 
 /* Read a byte into variable V.
- * If must suspend, take the specified action (typically "return FALSE").
+ * If must suspend, take the specified action (typically "return false").
  */
 #define INPUT_BYTE(cinfo, V, action)  \
-	MAKESTMT(MAKE_BYTE_AVAIL(cinfo,action); \
-		  bytes_in_buffer--; \
-		  V = GETJOCTET(*next_input_byte++); )
+	MAKESTMT(MAKE_BYTE_AVAIL(cinfo, action); \
+	         bytes_in_buffer--; \
+	         V = GETJOCTET(*next_input_byte++); )
 
 /* As above, but read two bytes interpreted as an unsigned 16-bit integer.
  * V should be declared unsigned int or perhaps INT32.
  */
 #define INPUT_2BYTES(cinfo, V, action)  \
-	MAKESTMT(MAKE_BYTE_AVAIL(cinfo,action); \
-		  bytes_in_buffer--; \
-		  V = ((unsigned int) GETJOCTET(*next_input_byte++)) << 8; \
-		  MAKE_BYTE_AVAIL(cinfo, action); \
-		  bytes_in_buffer--; \
-		  V += GETJOCTET(*next_input_byte++); )
+	MAKESTMT(MAKE_BYTE_AVAIL(cinfo, action); \
+	      bytes_in_buffer--; \
+	      V = ((unsigned int) GETJOCTET(*next_input_byte++)) << 8; \
+	      MAKE_BYTE_AVAIL(cinfo, action); \
+	      bytes_in_buffer--; \
+	      V += GETJOCTET(*next_input_byte++); )
 
 
-static boolean
-handle_app1(j_decompress_ptr cinfo)
+static boolean handle_app1(j_decompress_ptr cinfo)
 {
 	INT32 length; /* initialized by the macro */
 	INT32 i;
@@ -259,18 +263,18 @@ handle_app1(j_decompress_ptr cinfo)
 	
 	INPUT_VARS(cinfo);
 
-	INPUT_2BYTES(cinfo, length, return FALSE);
+	INPUT_2BYTES(cinfo, length, return false);
 	length -= 2;
 	
 	if (length < 16) {
-		for (i = 0; i < length; i++) INPUT_BYTE(cinfo, neogeo[i], return FALSE);
+		for (i = 0; i < length; i++) INPUT_BYTE(cinfo, neogeo[i], return false);
 		length = 0;
-		if (strncmp(neogeo, "NeoGeo", 6) == 0) memcpy(&ibuf_ftype, neogeo + 6, 4);
-		ibuf_ftype = BIG_LONG(ibuf_ftype);
+		if (STREQLEN(neogeo, "NeoGeo", 6)) memcpy(&ibuf_foptions, neogeo + 6, 4);
+		ibuf_foptions = BIG_LONG(ibuf_foptions);
 	}
 	INPUT_SYNC(cinfo);  /* do before skip_input_data */
 	if (length > 0) (*cinfo->src->skip_input_data)(cinfo, length);
-	return TRUE;
+	return true;
 }
 
 
@@ -286,12 +290,12 @@ static ImBuf *ibJpegImageFromCinfo(struct jpeg_decompress_struct *cinfo, int fla
 	char *str, *key, *value;
 
 	/* install own app1 handler */
-	ibuf_ftype = 0;
+	ibuf_foptions = 0;
 	jpeg_set_marker_processor(cinfo, 0xe1, handle_app1);
 	cinfo->dct_method = JDCT_FLOAT;
 	jpeg_save_markers(cinfo, JPEG_COM, 0xffff);
 
-	if (jpeg_read_header(cinfo, FALSE) == JPEG_HEADER_OK) {
+	if (jpeg_read_header(cinfo, false) == JPEG_HEADER_OK) {
 		x = cinfo->image_width;
 		y = cinfo->image_height;
 		depth = cinfo->num_components;
@@ -300,11 +304,11 @@ static ImBuf *ibJpegImageFromCinfo(struct jpeg_decompress_struct *cinfo, int fla
 
 		jpeg_start_decompress(cinfo);
 
-		if (ibuf_ftype == 0) {
-			ibuf_ftype = JPG_STD;
+		if (ibuf_foptions == 0) {
+			ibuf_foptions = JPG_STD;
 			if (cinfo->max_v_samp_factor == 1) {
-				if (cinfo->max_h_samp_factor == 1) ibuf_ftype = JPG_MAX;
-				else ibuf_ftype = JPG_VID;
+				if (cinfo->max_h_samp_factor == 1) ibuf_foptions = JPG_MAX;
+				else ibuf_foptions = JPG_VID;
 			}
 		}
 
@@ -348,30 +352,18 @@ static ImBuf *ibJpegImageFromCinfo(struct jpeg_decompress_struct *cinfo, int fla
 							g = *buffer++;
 							b = *buffer++;
 							k = *buffer++;
-							
-							k = 255 - k;
-							r -= k;
-							if (r & 0xffffff00) {
-								if (r < 0) r = 0;
-								else r = 255;
-							}
-							g -= k;
-							if (g & 0xffffff00) {
-								if (g < 0) g = 0;
-								else g = 255;
-							}
-							b -= k;
-							if (b & 0xffffff00) {
-								if (b < 0) b = 0;
-								else b = 255;
-							}
-							
-							rect[3] = 255 - k;
+
+							r = (r * k) / 255;
+							g = (g * k) / 255;
+							b = (b * k) / 255;
+
+							rect[3] = 255;
 							rect[2] = b;
 							rect[1] = g;
 							rect[0] = r;
 							rect += 4;
 						}
+						break;
 				}
 			}
 
@@ -380,6 +372,12 @@ static ImBuf *ibJpegImageFromCinfo(struct jpeg_decompress_struct *cinfo, int fla
 				if (marker->marker != JPEG_COM)
 					goto next_stamp_marker;
 
+				/*
+				 * JPEG marker strings are not null-terminated,
+				 * create a null-terminated copy before going further
+				 */
+				str = BLI_strdupn((char *)marker->data, marker->data_length);
+
 				/*
 				 * Because JPEG format don't support the
 				 * pair "key/value" like PNG, we store the
@@ -389,7 +387,7 @@ static ImBuf *ibJpegImageFromCinfo(struct jpeg_decompress_struct *cinfo, int fla
 				 * That is why we need split it to the
 				 * common key/value here.
 				 */
-				if (strncmp((char *) marker->data, "Blender", 7)) {
+				if (!STREQLEN(str, "Blender", 7)) {
 					/*
 					 * Maybe the file have text that
 					 * we don't know "what it's", in that
@@ -399,12 +397,12 @@ static ImBuf *ibJpegImageFromCinfo(struct jpeg_decompress_struct *cinfo, int fla
 					 * the information when we write
 					 * it back to disk.
 					 */
-					IMB_metadata_add_field(ibuf, "None", (char *) marker->data);
+					IMB_metadata_add_field(ibuf, "None", str);
 					ibuf->flags |= IB_metadata;
+					MEM_freeN(str);
 					goto next_stamp_marker;
 				}
 
-				str = BLI_strdup((char *) marker->data);
 				key = strchr(str, ':');
 				/*
 				 * A little paranoid, but the file maybe
@@ -437,14 +435,15 @@ next_stamp_marker:
 		
 		jpeg_destroy((j_common_ptr) cinfo);
 		if (ibuf) {
-			ibuf->ftype = ibuf_ftype;
+			ibuf->ftype = IMB_FTYPE_JPG;
+			ibuf->foptions.flag = ibuf_foptions;
 		}
 	}
 
 	return(ibuf);
 }
 
-ImBuf *imb_load_jpeg(unsigned char *buffer, size_t size, int flags, char colorspace[IM_MAX_SPACE])
+ImBuf *imb_load_jpeg(const unsigned char *buffer, size_t size, int flags, char colorspace[IM_MAX_SPACE])
 {
 	struct jpeg_decompress_struct _cinfo, *cinfo = &_cinfo;
 	struct my_error_mgr jerr;
@@ -482,40 +481,39 @@ static void write_jpeg(struct jpeg_compress_struct *cinfo, struct ImBuf *ibuf)
 	uchar *rect;
 	int x, y;
 	char neogeo[128];
-	ImMetaData *iptr;
 	char *text;
 
-	jpeg_start_compress(cinfo, TRUE);
+	jpeg_start_compress(cinfo, true);
 
 	strcpy(neogeo, "NeoGeo");
-	ibuf_ftype = BIG_LONG(ibuf->ftype);
+	ibuf_foptions = BIG_LONG((((int)ibuf->foptions.flag) << 8) | (int)ibuf->foptions.quality);
 	
-	memcpy(neogeo + 6, &ibuf_ftype, 4);
+	memcpy(neogeo + 6, &ibuf_foptions, 4);
 	jpeg_write_marker(cinfo, 0xe1, (JOCTET *) neogeo, 10);
 
 	if (ibuf->metadata) {
+		IDProperty *prop;
 		/* key + max value + "Blender" */
 		text = MEM_mallocN(530, "stamp info read");
-		iptr = ibuf->metadata;
-		while (iptr) {
-			if (!strcmp(iptr->key, "None")) {
-				jpeg_write_marker(cinfo, JPEG_COM, (JOCTET *) iptr->value, strlen(iptr->value) + 1);
-				goto next_stamp_info;
-			}
+		for (prop = ibuf->metadata->data.group.first; prop; prop = prop->next) {
+			if (prop->type == IDP_STRING) {
+				int text_len;
+				if (!strcmp(prop->name, "None")) {
+					jpeg_write_marker(cinfo, JPEG_COM, (JOCTET *) IDP_String(prop), prop->len + 1);
+				}
 
-			/*
-			 * The JPEG format don't support a pair "key/value"
-			 * like PNG, so we "encode" the stamp in a
-			 * single string:
-			 *	"Blender:key:value"
-			 *
-			 * The first "Blender" is a simple identify to help
-			 * in the read process.
-			 */
-			sprintf(text, "Blender:%s:%s", iptr->key, iptr->value);
-			jpeg_write_marker(cinfo, JPEG_COM, (JOCTET *) text, strlen(text) + 1);
-next_stamp_info:
-			iptr = iptr->next;
+				/*
+				 * The JPEG format don't support a pair "key/value"
+				 * like PNG, so we "encode" the stamp in a
+				 * single string:
+				 *	"Blender:key:value"
+				 *
+				 * The first "Blender" is a simple identify to help
+				 * in the read process.
+				 */
+				text_len = sprintf(text, "Blender:%s:%s", prop->name, IDP_String(prop));
+				jpeg_write_marker(cinfo, JPEG_COM, (JOCTET *) text, text_len + 1);
+			}
 		}
 		MEM_freeN(text);
 	}
@@ -549,7 +547,8 @@ next_stamp_info:
 				break;
 			/* default was missing... intentional ? */
 			default:
-				; /* do nothing */
+				/* do nothing */
+				break;
 		}
 
 		jpeg_write_scanlines(cinfo, row_pointer, 1);
@@ -564,7 +563,7 @@ static int init_jpeg(FILE *outfile, struct jpeg_compress_struct *cinfo, struct I
 {
 	int quality;
 
-	quality = ibuf->ftype & 0xff;
+	quality = ibuf->foptions.quality;
 	if (quality <= 0) quality = jpeg_default_quality;
 	if (quality > 100) quality = 100;
 
@@ -594,14 +593,15 @@ static int init_jpeg(FILE *outfile, struct jpeg_compress_struct *cinfo, struct I
 			break;
 		/* default was missing... intentional ? */
 		default:
-			; /* do nothing */
+			/* do nothing */
+			break;
 	}
 	jpeg_set_defaults(cinfo);
 	
 	/* own settings */
 
 	cinfo->dct_method = JDCT_FLOAT;
-	jpeg_set_quality(cinfo, quality, TRUE);
+	jpeg_set_quality(cinfo, quality, true);
 
 	return(0);
 }
@@ -688,6 +688,7 @@ static int save_jstjpeg(const char *name, struct ImBuf *ibuf)
 
 	tbuf = IMB_allocImBuf(ibuf->x, ibuf->y / 2, 24, IB_rect);
 	tbuf->ftype = ibuf->ftype;
+	tbuf->foptions = ibuf->foptions;
 	tbuf->flags = ibuf->flags;
 	
 	oldy = ibuf->y;
