@@ -32,17 +32,17 @@
 #ifndef __DNA_IMAGE_TYPES_H__
 #define __DNA_IMAGE_TYPES_H__
 
+#include "DNA_defs.h"
 #include "DNA_ID.h"
 #include "DNA_color_types.h"  /* for color management */
 
 struct PackedFile;
 struct Scene;
 struct anim;
-struct ImBuf;
+struct MovieCache;
 struct RenderResult;
 struct GPUTexture;
 
-
 /* ImageUser is in Texture, in Nodes, Background Image, Image Window, .... */
 /* should be used in conjunction with an ID * to Image. */
 typedef struct ImageUser {
@@ -52,38 +52,60 @@ typedef struct ImageUser {
 	int frames;					/* total amount of frames to use */
 	int offset, sfra;			/* offset within movie, start frame in global time */
 	char fie_ima, cycl;		/* fields/image in movie, cyclic flag */
-	char ok, pad;
+	char ok;
 
-	short multi_index, layer, pass;	 /* listbase indices, for menu browsing or retrieve buffer */
+	char multiview_eye;			/* multiview current eye - for internal use of drawing routines */
+	short pass;
+	short pad;
 
+	short multi_index, view, layer;	 /* listbase indices, for menu browsing or retrieve buffer */
 	short flag;
-	
-	int pad2;
-
 } ImageUser;
 
+typedef struct ImageAnim {
+	struct ImageAnim *next, *prev;
+	struct anim *anim;
+} ImageAnim;
+
+typedef struct ImageView {
+	struct ImageView *next, *prev;
+	char name[64];			/* MAX_NAME */
+	char filepath[1024];	/* 1024 = FILE_MAX */
+} ImageView;
+
+typedef struct ImagePackedFile {
+	struct ImagePackedFile *next, *prev;
+	struct PackedFile *packedfile;
+	char filepath[1024];	/* 1024 = FILE_MAX */
+} ImagePackedFile;
+
+typedef struct RenderSlot {
+	char name[64];  /* 64 = MAX_NAME */
+} RenderSlot;
+
 /* iuser->flag */
 #define	IMA_ANIM_ALWAYS		1
 #define IMA_ANIM_REFRESHED	2
 /* #define IMA_DO_PREMUL	4 */
 #define IMA_NEED_FRAME_RECALC	8
+#define IMA_SHOW_STEREO		16
 
 typedef struct Image {
 	ID id;
 	
 	char name[1024];			/* file path, 1024 = FILE_MAX */
 	
-	ListBase ibufs;					/* not written in file */
+	struct MovieCache *cache;	/* not written in file */
 	struct GPUTexture *gputexture;	/* not written in file */
 	
 	/* sources from: */
-	struct anim *anim;
+	ListBase anims;
 	struct RenderResult *rr;
 
 	struct RenderResult *renders[8]; /* IMA_MAX_RENDER_SLOT */
 	short render_slot, last_render_slot;
-	
-	short ok, flag;
+
+	int flag;
 	short source, type;
 	int lastframe;
 
@@ -94,40 +116,65 @@ typedef struct Image {
 	unsigned int bindcode;	/* only for current image... */
 	unsigned int *repbind;	/* for repeat of parts of images */
 	
-	struct PackedFile *packedfile;
+	struct PackedFile *packedfile DNA_DEPRECATED; /* deprecated */
+	struct ListBase packedfiles;
 	struct PreviewImage *preview;
 
 	/* game engine tile animation */
 	float lastupdate;
 	int lastused;
 	short animspeed;
+
+	short ok;
 	
 	/* for generated images */
-	short gen_x, gen_y;
+	int gen_x, gen_y;
 	char gen_type, gen_flag;
+	short gen_depth;
+	float gen_color[4];
 	
 	/* display aspect - for UV editing images resized for faster openGL display */
 	float aspx, aspy;
 
 	/* color management */
 	ColorManagedColorspaceSettings colorspace_settings;
+	char alpha_mode;
+
+	char pad[5];
+
+	/* Multiview */
+	char eye; /* for viewer node stereoscopy */
+	char views_format;
+	ListBase views;
+	struct Stereo3dFormat *stereo3d_format;
+
+	RenderSlot render_slots[8];  /* 8 = IMA_MAX_RENDER_SLOT */
 } Image;
 
 
 /* **************** IMAGE ********************* */
 
 /* Image.flag */
-#define IMA_FIELDS			1
-#define IMA_STD_FIELD		2
-#define IMA_DO_PREMUL		4
-#define IMA_REFLECT			16
-#define IMA_NOCOLLECT   	32
-#define IMA_DEPRECATED		64
-#define IMA_OLD_PREMUL		128
-#define IMA_CM_PREDIVIDE	256
-#define IMA_USED_FOR_RENDER	512
-#define IMA_USER_FRAME_IN_RANGE	1024 /* for image user, but these flags are mixed */
-#define IMA_VIEW_AS_RENDER	2048
+enum {
+	IMA_FIELDS              = (1 << 0),
+	IMA_STD_FIELD           = (1 << 1),
+#ifdef DNA_DEPRECATED
+	IMA_DO_PREMUL           = (1 << 2),  /* deprecated, should not be used */
+#endif
+	IMA_REFLECT             = (1 << 4),
+	IMA_NOCOLLECT           = (1 << 5),
+	//IMA_DONE_TAG          = (1 << 6),  // UNUSED
+	IMA_OLD_PREMUL          = (1 << 7),
+	// IMA_CM_PREDIVIDE     = (1 << 8),  /* deprecated, should not be used */
+	IMA_USED_FOR_RENDER     = (1 << 9),
+	IMA_USER_FRAME_IN_RANGE = (1 << 10), /* for image user, but these flags are mixed */
+	IMA_VIEW_AS_RENDER      = (1 << 11),
+	IMA_IGNORE_ALPHA        = (1 << 12),
+	IMA_DEINTERLACE         = (1 << 13),
+	IMA_USE_VIEWS           = (1 << 14),
+	IMA_IS_STEREO           = (1 << 15),
+	IMA_IS_MULTIVIEW        = (1 << 16), /* similar to stereo, but a more general case */
+};
 
 /* Image.tpageflag */
 #define IMA_TILES			1
@@ -137,6 +184,7 @@ typedef struct Image {
 #define IMA_CLAMP_U			16 
 #define IMA_CLAMP_V			32
 #define IMA_TPAGE_REFRESH	64
+#define IMA_GLBIND_IS_DATA	128 /* opengl image texture bound as non-color data */
 
 /* ima->type and ima->source moved to BKE_image.h, for API */
 
@@ -147,4 +195,10 @@ typedef struct Image {
 /* gen_flag */
 #define IMA_GEN_FLOAT		1
 
+/* alpha_mode */
+enum {
+	IMA_ALPHA_STRAIGHT = 0,
+	IMA_ALPHA_PREMUL = 1,
+};
+
 #endif
