@@ -71,6 +71,24 @@
 
 #include "WM_api.h"  /* only for WM_main_playanim */
 
+#ifdef WITH_AUDASPACE
+#  include AUD_DEVICE_H
+#  include AUD_HANDLE_H
+#  include AUD_SOUND_H
+#  include AUD_SPECIAL_H
+
+static AUD_Sound *source = NULL;
+static AUD_Handle *playback_handle = NULL;
+static AUD_Handle *scrub_handle = NULL;
+static AUD_Device *audio_device = NULL;
+#endif
+
+/* simple limiter to avoid flooding memory */
+#define USE_FRAME_CACHE_LIMIT
+#ifdef USE_FRAME_CACHE_LIMIT
+#  define PLAY_FRAME_CACHE_MAX 30
+#endif
+
 struct PlayState;
 static void playanim_window_zoom(struct PlayState *ps, const float zoom_offset);
 
@@ -90,6 +108,7 @@ typedef struct PlayState {
 	bool  turbo;
 	bool  pingpong;
 	bool  noskip;
+	bool  indicator;
 	bool  sstep;
 	bool  wait2;
 	bool  stopped;
@@ -160,11 +179,11 @@ static struct WindowStateGlobal {
 	eWS_Qual qual;
 } g_WS = {NULL};
 
-static void playanim_window_get_size(int *width_r, int *height_r)
+static void playanim_window_get_size(int *r_width, int *r_height)
 {
 	GHOST_RectangleHandle bounds = GHOST_GetClientBounds(g_WS.ghost_window);
-	*width_r = GHOST_GetWidthRectangle(bounds);
-	*height_r = GHOST_GetHeightRectangle(bounds);
+	*r_width = GHOST_GetWidthRectangle(bounds);
+	*r_height = GHOST_GetHeightRectangle(bounds);
 	GHOST_DisposeRectangle(bounds);
 }
 
@@ -222,8 +241,17 @@ typedef struct PlayAnimPict {
 } PlayAnimPict;
 
 static struct ListBase picsbase = {NULL, NULL};
+/* frames in memory - store them here to for easy deallocation later */
 static bool fromdisk = false;
 static double ptottime = 0.0, swaptime = 0.04;
+#ifdef WITH_AUDASPACE
+static double fps_movie;
+#endif
+
+#ifdef USE_FRAME_CACHE_LIMIT
+static struct ListBase inmempicsbase = {NULL, NULL};
+static int added_images = 0;
+#endif
 
 static PlayAnimPict *playanim_step(PlayAnimPict *playanim, int step)
 {
@@ -322,6 +350,30 @@ static void playanim_toscreen(PlayState *ps, PlayAnimPict *picture, struct ImBuf
 		BLF_draw(fontid, str, sizeof(str));
 	}
 
+	if (ps->indicator) {
+		float fac = ps->picture->frame / (double)(((PlayAnimPict *)picsbase.last)->frame - ((PlayAnimPict *)picsbase.first)->frame);
+
+		fac = 2.0f * fac - 1.0f;
+		glMatrixMode(GL_PROJECTION);
+		glPushMatrix();
+		glLoadIdentity();
+		glMatrixMode(GL_MODELVIEW);
+		glPushMatrix();
+		glLoadIdentity();
+
+		glColor4f(0.0f, 1.0f, 0.0f, 1.0f);
+
+		glBegin(GL_LINES);
+		glVertex2f(fac, -1.0f);
+		glVertex2f(fac, 1.0f);
+		glEnd();
+
+		glPopMatrix();
+		glMatrixMode(GL_PROJECTION);
+		glPopMatrix();
+		glMatrixMode(GL_MODELVIEW);
+	}
+
 	GHOST_SwapWindowBuffers(g_WS.ghost_window);
 }
 
@@ -429,6 +481,7 @@ static void build_pict_list_ex(PlayState *ps, const char *first, int totframes,
 
 			picture->mem = mem;
 			picture->name = strdup(filepath);
+			picture->frame = count; /* not exact but should work for positioning */
 			close(file);
 			BLI_addtail(&picsbase, picture);
 			count++;
@@ -476,6 +529,76 @@ static void build_pict_list(PlayState *ps, const char *first, int totframes, int
 	ps->loading = false;
 }
 
+static void update_sound_fps(void)
+{
+#ifdef WITH_AUDASPACE
+	if (playback_handle) {
+		/* swaptime stores the 1.0/fps ratio */
+		double speed = 1.0 / (swaptime * fps_movie);
+
+		AUD_Handle_setPitch(playback_handle, speed);
+	}
+#endif
+}
+
+static void change_frame(PlayState *ps, int cx)
+{
+	int sizex, sizey;
+	int i;
+
+	playanim_window_get_size(&sizex, &sizey);
+	ps->picture = picsbase.first;
+	/* TODO - store in ps direct? */
+	i = 0;
+	while (ps->picture) {
+		i++;
+		ps->picture = ps->picture->next;
+	}
+	i = (i * cx) / sizex;
+
+#ifdef WITH_AUDASPACE
+	if (scrub_handle) {
+		AUD_Handle_stop(scrub_handle);
+		scrub_handle = NULL;
+	}
+
+	if (playback_handle) {
+		AUD_Status status = AUD_Handle_getStatus(playback_handle);
+		if (status != AUD_STATUS_PLAYING) {
+			AUD_Handle_stop(playback_handle);
+			playback_handle = AUD_Device_play(audio_device, source, 1);
+			if (playback_handle) {
+				AUD_Handle_setPosition(playback_handle, i / fps_movie);
+				scrub_handle = AUD_pauseAfter(playback_handle, 1 / fps_movie);
+			}
+			update_sound_fps();
+		}
+		else {
+			AUD_Handle_setPosition(playback_handle, i / fps_movie);
+			scrub_handle = AUD_pauseAfter(playback_handle, 1 / fps_movie);
+		}
+	}
+	else if (source) {
+		playback_handle = AUD_Device_play(audio_device, source, 1);
+		if (playback_handle) {
+			AUD_Handle_setPosition(playback_handle, i / fps_movie);
+			scrub_handle = AUD_pauseAfter(playback_handle, 1 / fps_movie);
+		}
+		update_sound_fps();
+	}
+#endif
+
+	ps->picture = picsbase.first;
+	for (; i > 0; i--) {
+		if (ps->picture->next == NULL) break;
+		ps->picture = ps->picture->next;
+	}
+
+	ps->sstep = true;
+	ps->wait2 = false;
+	ps->next_frame = 0;
+}
+
 static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 {
 	PlayState *ps = (PlayState *)ps_void;
@@ -489,7 +612,6 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 	/* convert ghost event into value keyboard or mouse */
 	val = ELEM(type, GHOST_kEventKeyDown, GHOST_kEventButtonDown);
 
-
 	/* first check if we're busy loading files */
 	if (ps->loading) {
 		switch (type) {
@@ -535,6 +657,9 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 				case GHOST_kKeyA:
 					if (val) ps->noskip = !ps->noskip;
 					break;
+				case GHOST_kKeyI:
+					if (val) ps->indicator = !ps->indicator;
+					break;
 				case GHOST_kKeyP:
 					if (val) ps->pingpong = !ps->pingpong;
 					break;
@@ -548,42 +673,70 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 				}
 				case GHOST_kKey1:
 				case GHOST_kKeyNumpad1:
-					if (val) swaptime = ps->fstep / 60.0;
+					if (val) {
+						swaptime = ps->fstep / 60.0;
+						update_sound_fps();
+					}
 					break;
 				case GHOST_kKey2:
 				case GHOST_kKeyNumpad2:
-					if (val) swaptime = ps->fstep / 50.0;
+					if (val) {
+						swaptime = ps->fstep / 50.0;
+						update_sound_fps();
+					}
 					break;
 				case GHOST_kKey3:
 				case GHOST_kKeyNumpad3:
-					if (val) swaptime = ps->fstep / 30.0;
+					if (val) {
+						swaptime = ps->fstep / 30.0;
+						update_sound_fps();
+					}
 					break;
 				case GHOST_kKey4:
 				case GHOST_kKeyNumpad4:
-					if (g_WS.qual & WS_QUAL_SHIFT)
+					if (g_WS.qual & WS_QUAL_SHIFT) {
 						swaptime = ps->fstep / 24.0;
-					else
+						update_sound_fps();
+					}
+					else {
 						swaptime = ps->fstep / 25.0;
+						update_sound_fps();
+					}
 					break;
 				case GHOST_kKey5:
 				case GHOST_kKeyNumpad5:
-					if (val) swaptime = ps->fstep / 20.0;
+					if (val) {
+						swaptime = ps->fstep / 20.0;
+						update_sound_fps();
+					}
 					break;
 				case GHOST_kKey6:
 				case GHOST_kKeyNumpad6:
-					if (val) swaptime = ps->fstep / 15.0;
+					if (val) {
+						swaptime = ps->fstep / 15.0;
+						update_sound_fps();
+					}
 					break;
 				case GHOST_kKey7:
 				case GHOST_kKeyNumpad7:
-					if (val) swaptime = ps->fstep / 12.0;
+					if (val) {
+						swaptime = ps->fstep / 12.0;
+						update_sound_fps();
+					}
 					break;
 				case GHOST_kKey8:
 				case GHOST_kKeyNumpad8:
-					if (val) swaptime = ps->fstep / 10.0;
+					if (val) {
+						swaptime = ps->fstep / 10.0;
+						update_sound_fps();
+					}
 					break;
 				case GHOST_kKey9:
 				case GHOST_kKeyNumpad9:
-					if (val) swaptime = ps->fstep / 6.0;
+					if (val) {
+						swaptime = ps->fstep / 6.0;
+						update_sound_fps();
+					}
 					break;
 				case GHOST_kKeyLeftArrow:
 					if (val) {
@@ -646,6 +799,7 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 						}
 						else {
 							swaptime = ps->fstep / 5.0;
+							update_sound_fps();
 						}
 					}
 					break;
@@ -662,10 +816,63 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 						}
 					}
 					break;
+
+				case GHOST_kKeySpace:
+					if (val) {
+						if (ps->wait2 || ps->sstep) {
+							ps->wait2 = ps->sstep = false;
+#ifdef WITH_AUDASPACE
+							{
+								PlayAnimPict *picture = picsbase.first;
+								/* TODO - store in ps direct? */
+								int i = 0;
+
+								while (picture && picture != ps->picture) {
+									i++;
+									picture = picture->next;
+								}
+								if (playback_handle)
+									AUD_Handle_stop(playback_handle);
+								playback_handle = AUD_Device_play(audio_device, source, 1);
+								if (playback_handle)
+									AUD_Handle_setPosition(playback_handle, i / fps_movie);
+								update_sound_fps();
+							}
+#endif
+						}
+						else {
+							ps->sstep = true;
+							ps->wait2 = true;
+#ifdef WITH_AUDASPACE
+							if (playback_handle) {
+								AUD_Handle_stop(playback_handle);
+								playback_handle = NULL;
+							}
+#endif
+						}
+					}
+					break;
 				case GHOST_kKeyEnter:
 				case GHOST_kKeyNumpadEnter:
 					if (val) {
 						ps->wait2 = ps->sstep = false;
+#ifdef WITH_AUDASPACE
+						{
+							PlayAnimPict *picture = picsbase.first;
+							/* TODO - store in ps direct? */
+							int i = 0;
+							while (picture && picture != ps->picture) {
+								i++;
+								picture = picture->next;
+							}
+							if (playback_handle)
+								AUD_Handle_stop(playback_handle);
+							playback_handle = AUD_Device_play(audio_device, source, 1);
+							if (playback_handle)
+								AUD_Handle_setPosition(playback_handle, i / fps_movie);
+							update_sound_fps();
+						}
+#endif
 					}
 					break;
 				case GHOST_kKeyPeriod:
@@ -677,6 +884,12 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 						else {
 							ps->sstep = true;
 							ps->wait2 = !ps->wait2;
+#ifdef WITH_AUDASPACE
+							if (playback_handle) {
+								AUD_Handle_stop(playback_handle);
+								playback_handle = NULL;
+							}
+#endif
 						}
 					}
 					break;
@@ -688,7 +901,10 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 						playanim_window_zoom(ps, 1.0f);
 					}
 					else {
-						swaptime /= 1.1;
+						if (swaptime > ps->fstep / 60.0) {
+							swaptime /= 1.1;
+							update_sound_fps();
+						}
 					}
 					break;
 				}
@@ -700,7 +916,10 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 						playanim_window_zoom(ps, -1.0f);
 					}
 					else {
-						swaptime *= 1.1;
+						if (swaptime < ps->fstep / 5.0) {
+							swaptime *= 1.1;
+							update_sound_fps();
+						}
 					}
 					break;
 				}
@@ -726,8 +945,10 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 			
 			if (bd->button == GHOST_kButtonMaskLeft) {
 				if (type == GHOST_kEventButtonDown) {
-					if (inside_window)
+					if (inside_window) {
 						g_WS.qual |= WS_QUAL_LMOUSE;
+						change_frame(ps, cx);
+					}
 				}
 				else
 					g_WS.qual &= ~WS_QUAL_LMOUSE;
@@ -753,31 +974,12 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 		case GHOST_kEventCursorMove:
 		{
 			if (g_WS.qual & WS_QUAL_LMOUSE) {
-				int sizex, sizey;
-				int i;
-
 				GHOST_TEventCursorData *cd = GHOST_GetEventData(evt);
 				int cx, cy;
 
 				GHOST_ScreenToClient(g_WS.ghost_window, cd->x, cd->y, &cx, &cy);
 
-				playanim_window_get_size(&sizex, &sizey);
-				ps->picture = picsbase.first;
-				/* TODO - store in ps direct? */
-				i = 0;
-				while (ps->picture) {
-					i++;
-					ps->picture = ps->picture->next;
-				}
-				i = (i * cx) / sizex;
-				ps->picture = picsbase.first;
-				for (; i > 0; i--) {
-					if (ps->picture->next == NULL) break;
-					ps->picture = ps->picture->next;
-				}
-				ps->sstep = true;
-				ps->wait2 = false;
-				ps->next_frame = 0;
+				change_frame(ps, cx);
 			}
 			break;
 		}
@@ -848,6 +1050,7 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 
 static void playanim_window_open(const char *title, int posx, int posy, int sizex, int sizey)
 {
+	GHOST_GLSettings glsettings = {0};
 	GHOST_TUns32 scr_w, scr_h;
 
 	GHOST_GetMainDisplayDimensions(g_WS.ghost_system, &scr_w, &scr_h);
@@ -860,7 +1063,7 @@ static void playanim_window_open(const char *title, int posx, int posy, int size
 	                                       /* could optionally start fullscreen */
 	                                       GHOST_kWindowStateNormal,
 	                                       GHOST_kDrawingContextTypeOpenGL,
-	                                       false /* no stereo */, false);
+	                                       glsettings);
 }
 
 static void playanim_window_zoom(PlayState *ps, const float zoom_offset)
@@ -911,6 +1114,7 @@ static char *wm_main_playanim_intern(int argc, const char **argv)
 	ps.stopped   = false;
 	ps.loading   = false;
 	ps.picture   = NULL;
+	ps.indicator = false;
 	ps.dropped_file[0] = 0;
 	ps.zoom      = 1.0f;
 	/* resetmap = false */
@@ -1068,6 +1272,23 @@ static char *wm_main_playanim_intern(int argc, const char **argv)
 
 	build_pict_list(&ps, filepath, (efra - sfra) + 1, ps.fstep, ps.fontid);
 
+#ifdef WITH_AUDASPACE
+	source = AUD_Sound_file(filepath);
+	{
+		struct anim *anim_movie = ((struct PlayAnimPict *)picsbase.first)->anim;
+		if (anim_movie) {
+			short frs_sec = 25;
+			float frs_sec_base = 1.0;
+
+			IMB_anim_get_fps(anim_movie, &frs_sec, &frs_sec_base, true);
+
+			fps_movie = (double) frs_sec / (double) frs_sec_base;
+			/* enforce same fps for movie as sound */
+			swaptime = ps.fstep / fps_movie;
+		}
+	}
+#endif
+
 	for (i = 2; i < argc; i++) {
 		BLI_strncpy(filepath, argv[i], sizeof(filepath));
 		build_pict_list(&ps, filepath, (efra - sfra) + 1, ps.fstep, ps.fontid);
@@ -1107,6 +1328,13 @@ static char *wm_main_playanim_intern(int argc, const char **argv)
 		}
 		if (ptottime > 0.0) ptottime = 0.0;
 
+#ifdef WITH_AUDASPACE
+		if (playback_handle)
+			AUD_Handle_stop(playback_handle);
+		playback_handle = AUD_Device_play(audio_device, source, 1);
+		update_sound_fps();
+#endif
+
 		while (ps.picture) {
 			int hasevent;
 #ifndef USE_IMB_CACHE
@@ -1129,11 +1357,39 @@ static char *wm_main_playanim_intern(int argc, const char **argv)
 			}
 
 			if (ibuf) {
+#ifdef USE_FRAME_CACHE_LIMIT
+				LinkData *node;
+#endif
 
 #ifdef USE_IMB_CACHE
 				ps.picture->ibuf = ibuf;
 #endif
 
+#ifdef USE_FRAME_CACHE_LIMIT
+				/* really basic memory conservation scheme. Keep frames in a fifo queue */
+				node = inmempicsbase.last;
+
+				while (node && added_images > PLAY_FRAME_CACHE_MAX) {
+					PlayAnimPict *pic = node->data;
+
+					if (pic->ibuf && pic->ibuf != ibuf) {
+						LinkData *node_tmp;
+						IMB_freeImBuf(pic->ibuf);
+						pic->ibuf = NULL;
+						node_tmp = node->prev;
+						BLI_freelinkN(&inmempicsbase, node);
+						added_images--;
+						node = node_tmp;
+					}
+					else {
+						node = node->prev;
+					}
+				}
+
+				BLI_addhead(&inmempicsbase, BLI_genericNodeN(ps.picture));
+				added_images++;
+#endif  /* USE_FRAME_CACHE_LIMIT */
+
 				BLI_strncpy(ibuf->name, ps.picture->name, sizeof(ibuf->name));
 
 				/* why only windows? (from 2.4x) - campbell */
@@ -1240,6 +1496,22 @@ static char *wm_main_playanim_intern(int argc, const char **argv)
 #endif
 
 	BLI_freelistN(&picsbase);
+	BLI_freelistN(&inmempicsbase);
+	added_images = 0;
+
+#ifdef WITH_AUDASPACE
+	if (playback_handle) {
+		AUD_Handle_stop(playback_handle);
+		playback_handle = NULL;
+	}
+	if (scrub_handle) {
+		AUD_Handle_stop(scrub_handle);
+		scrub_handle = NULL;
+	}
+	AUD_Sound_free(source);
+	source = NULL;
+#endif
+
 #if 0 // XXX25
 	free_blender();
 #else
@@ -1275,18 +1547,43 @@ static char *wm_main_playanim_intern(int argc, const char **argv)
 
 void WM_main_playanim(int argc, const char **argv)
 {
+	const char *argv_next[2];
 	bool looping = true;
 
+#ifdef WITH_AUDASPACE
+	{
+		AUD_DeviceSpecs specs;
+
+		specs.rate = AUD_RATE_44100;
+		specs.format = AUD_FORMAT_S16;
+		specs.channels = AUD_CHANNELS_STEREO;
+
+		AUD_initOnce();
+
+		if (!(audio_device = AUD_init("OpenAL", specs, 1024, "Blender"))) {
+			audio_device = AUD_init("Null", specs, 0, "Blender");
+		}
+	}
+#endif
+
 	while (looping) {
 		const char *filepath = wm_main_playanim_intern(argc, argv);
 
 		if (filepath) {	/* use simple args */
-			argv[1] = "-a";
-			argv[2] = filepath;
-			argc = 3;
+			argv_next[0] = argv[0];
+			argv_next[1] = filepath;
+			argc = 2;
+
+			/* continue with new args */
+			argv = argv_next;
 		}
 		else {
 			looping = false;
 		}
 	}
+
+#ifdef WITH_AUDASPACE
+	AUD_exit(audio_device);
+	AUD_exitOnce();
+#endif
 }
