@@ -30,10 +30,9 @@
  *  \ingroup bgeconv
  */
 
-
-#if defined(WIN32) && !defined(FREE_WINDOWS)
-#pragma warning (disable : 4786) 
-#endif //WIN32
+#ifdef _MSC_VER
+#  pragma warning (disable:4786)
+#endif
 
 #include <math.h>
 
@@ -43,8 +42,7 @@
 #include "KX_ConvertActuators.h"
 
 #ifdef WITH_AUDASPACE
-#  include "AUD_C-API.h"
-#  include "AUD_ChannelMapperFactory.h"
+#  include AUD_SOUND_H
 #endif
 
 // Actuators
@@ -71,17 +69,18 @@
 #include "KX_ParentActuator.h"
 #include "KX_SCA_DynamicActuator.h"
 #include "KX_SteeringActuator.h"
+#include "KX_MouseActuator.h"
 
 #include "KX_Scene.h"
 #include "KX_KetsjiEngine.h"
 
-#include "IntValue.h"
+#include "EXP_IntValue.h"
 #include "KX_GameObject.h"
 
 /* This little block needed for linking to Blender... */
 #include "BKE_text.h"
 #include "BLI_blenlib.h"
-#include "BLI_math_base.h"
+#include "BLI_math.h"
 #include "BLI_path_util.h"
 
 #include "KX_NetworkMessageActuator.h"
@@ -105,22 +104,24 @@
 #include "BL_BlenderDataConversion.h"
 
 /** 
- * KX_BLENDERTRUNC needed to round 'almost' zero values to zero, else velocities etc. are incorrectly set
+ * KX_flt_trunc needed to round 'almost' zero values to zero, else velocities etc. are incorrectly set
  */
 
-#define KX_BLENDERTRUNC(x)  (( x < 0.0001f && x > -0.0001f ) ? 0.0f : x)
+BLI_INLINE float KX_flt_trunc(const float x)
+{
+	return ( x < 0.0001f && x > -0.0001f ) ? 0.0f : x;
+}
 
 void BL_ConvertActuators(const char* maggiename,
-						 struct Object* blenderobject,
-						 KX_GameObject* gameobj,
-						 SCA_LogicManager* logicmgr,
-						 KX_Scene* scene,
-						 KX_KetsjiEngine* ketsjiEngine,
-						 int activeLayerBitInfo,
-						 bool isInActiveLayer,
-						 RAS_IRenderTools* rendertools,
-						 KX_BlenderSceneConverter* converter
-						 )
+                         struct Object* blenderobject,
+                         KX_GameObject* gameobj,
+                         SCA_LogicManager* logicmgr,
+                         KX_Scene* scene,
+                         KX_KetsjiEngine* ketsjiEngine,
+                         int activeLayerBitInfo,
+                         bool isInActiveLayer,
+                         KX_BlenderSceneConverter* converter
+                         )
 {
 	
 	int uniqueint = 0;
@@ -134,7 +135,7 @@ void BL_ConvertActuators(const char* maggiename,
 	}
 	gameobj->ReserveActuator(actcount);
 	bact = (bActuator*) blenderobject->actuators.first;
-	while(bact)
+	while (bact)
 	{
 		STR_String uniquename = bact->name;
 		STR_String& objectname = gameobj->GetName();
@@ -146,20 +147,23 @@ void BL_ConvertActuators(const char* maggiename,
 			{
 				bObjectActuator* obact = (bObjectActuator*) bact->data;
 				KX_GameObject* obref = NULL;
-				MT_Vector3 forcevec(KX_BLENDERTRUNC(obact->forceloc[0]),
-					KX_BLENDERTRUNC(obact->forceloc[1]),
-					KX_BLENDERTRUNC(obact->forceloc[2]));
-				MT_Vector3 torquevec(obact->forcerot[0],obact->forcerot[1],obact->forcerot[2]);
-				MT_Vector3 dlocvec ( KX_BLENDERTRUNC(obact->dloc[0]),
-					KX_BLENDERTRUNC(obact->dloc[1]),
-					KX_BLENDERTRUNC(obact->dloc[2]));
-				MT_Vector3 drotvec ( KX_BLENDERTRUNC(obact->drot[0]),obact->drot[1],obact->drot[2]);
-				MT_Vector3 linvelvec ( KX_BLENDERTRUNC(obact->linearvelocity[0]),
-					KX_BLENDERTRUNC(obact->linearvelocity[1]),
-					KX_BLENDERTRUNC(obact->linearvelocity[2]));
-				MT_Vector3 angvelvec ( KX_BLENDERTRUNC(obact->angularvelocity[0]),
-					KX_BLENDERTRUNC(obact->angularvelocity[1]),
-					KX_BLENDERTRUNC(obact->angularvelocity[2]));
+				MT_Vector3 forcevec(KX_flt_trunc(obact->forceloc[0]),
+				                    KX_flt_trunc(obact->forceloc[1]),
+				                    KX_flt_trunc(obact->forceloc[2]));
+				MT_Vector3 torquevec(obact->forcerot[0],
+				                     obact->forcerot[1],
+				                     obact->forcerot[2]);
+				MT_Vector3 dlocvec(KX_flt_trunc(obact->dloc[0]),
+				                   KX_flt_trunc(obact->dloc[1]),
+				                   KX_flt_trunc(obact->dloc[2]));
+				MT_Vector3 drotvec(KX_flt_trunc(obact->drot[0]),
+				                   obact->drot[1],obact->drot[2]);
+				MT_Vector3 linvelvec(KX_flt_trunc(obact->linearvelocity[0]),
+				                     KX_flt_trunc(obact->linearvelocity[1]),
+				                     KX_flt_trunc(obact->linearvelocity[2]));
+				MT_Vector3 angvelvec(KX_flt_trunc(obact->angularvelocity[0]),
+				                     KX_flt_trunc(obact->angularvelocity[1]),
+				                     KX_flt_trunc(obact->angularvelocity[2]));
 				short damping = obact->damping;
 
 				/* Blender uses a bit vector internally for the local-flags. In */
@@ -175,31 +179,34 @@ void BL_ConvertActuators(const char* maggiename,
 				bitLocalFlag.LinearVelocity = bool((obact->flag & ACT_LIN_VEL_LOCAL)!=0);
 				bitLocalFlag.AngularVelocity = bool((obact->flag & ACT_ANG_VEL_LOCAL)!=0);
 				bitLocalFlag.ServoControl = bool(obact->type == ACT_OBJECT_SERVO);
+				bitLocalFlag.CharacterMotion = bool(obact->type == ACT_OBJECT_CHARACTER);
+				bitLocalFlag.CharacterJump = bool((obact->flag & ACT_CHAR_JUMP)!=0);
 				bitLocalFlag.AddOrSetLinV = bool((obact->flag & ACT_ADD_LIN_VEL)!=0);
+				bitLocalFlag.AddOrSetCharLoc = bool((obact->flag & ACT_ADD_CHAR_LOC)!=0);
 				if (obact->reference && bitLocalFlag.ServoControl)
 				{
 					obref = converter->FindGameObject(obact->reference);
 				}
 				
-				KX_ObjectActuator* tmpbaseact = new KX_ObjectActuator(gameobj,
-					obref,
-					forcevec.getValue(),
-					torquevec.getValue(),
-					dlocvec.getValue(),
-					drotvec.getValue(),
-					linvelvec.getValue(),
-					angvelvec.getValue(),
-					damping,
-					bitLocalFlag
-					);
+				KX_ObjectActuator* tmpbaseact = new KX_ObjectActuator(
+				            gameobj,
+				            obref,
+				            forcevec.getValue(),
+				            torquevec.getValue(),
+				            dlocvec.getValue(),
+				            drotvec.getValue(),
+				            linvelvec.getValue(),
+				            angvelvec.getValue(),
+				            damping,
+				            bitLocalFlag);
 				baseact = tmpbaseact;
 				break;
 			}
 		case ACT_ACTION:
 			{
 				bActionActuator* actact = (bActionActuator*) bact->data;
-				STR_String propname = (actact->name ? actact->name : "");
-				STR_String propframe = (actact->frameProp ? actact->frameProp : "");
+				STR_String propname = actact->name;
+				STR_String propframe = actact->frameProp;
 
 				short ipo_flags = 0;
 
@@ -210,22 +217,23 @@ void BL_ConvertActuators(const char* maggiename,
 				if (actact->flag & ACT_IPOCHILD) ipo_flags |= BL_Action::ACT_IPOFLAG_CHILD;
 					
 				BL_ActionActuator* tmpbaseact = new BL_ActionActuator(
-					gameobj,
-					propname,
-					propframe,
-					actact->sta,
-					actact->end,
-					actact->act,
-					actact->type, // + 1, because Blender starts to count at zero,
-					actact->blendin,
-					actact->priority,
-					actact->layer,
-					actact->layer_weight,
-					ipo_flags,
-					actact->end_reset,
-					actact->stridelength
-					// Ketsji at 1, because zero is reserved for "NoDef"
-					);
+				            gameobj,
+				            propname,
+				            propframe,
+				            actact->sta,
+				            actact->end,
+				            actact->act,
+				            actact->type, // + 1, because Blender starts to count at zero,
+				            actact->blend_mode,
+				            actact->blendin,
+				            actact->priority,
+				            actact->layer,
+				            actact->layer_weight,
+				            ipo_flags,
+				            actact->end_reset,
+				            actact->stridelength
+				            // Ketsji at 1, because zero is reserved for "NoDef"
+				            );
 				baseact= tmpbaseact;
 				break;
 			}
@@ -233,22 +241,21 @@ void BL_ConvertActuators(const char* maggiename,
 			{
 				if (blenderobject->type==OB_MESH) {
 					bActionActuator* actact = (bActionActuator*) bact->data;
-					STR_String propname = (actact->name ? actact->name : "");
-					STR_String propframe = (actact->frameProp ? actact->frameProp : "");
+					STR_String propname = actact->name;
+					STR_String propframe = actact->frameProp;
 					
 					BL_ShapeActionActuator* tmpbaseact = new BL_ShapeActionActuator(
-						gameobj,
-						propname,
-						propframe,
-						actact->sta,
-						actact->end,
-						actact->act,
-						actact->type, // + 1, because Blender starts to count at zero,
-						actact->blendin,
-						actact->priority,
-						actact->stridelength
-						// Ketsji at 1, because zero is reserved for "NoDef"
-						);
+					            gameobj,
+					            propname,
+					            propframe,
+					            actact->sta,
+					            actact->end,
+					            actact->act,
+					            actact->type, // + 1, because Blender starts to count at zero,
+					            actact->blendin,
+					            actact->priority,
+					            actact->stridelength);
+					// Ketsji at 1, because zero is reserved for "NoDef"
 					baseact= tmpbaseact;
 					break;
 				}
@@ -267,17 +274,17 @@ void BL_ConvertActuators(const char* maggiename,
 				bool ipo_add = (ipoact->flag & ACT_IPOADD);
 				
 				KX_IpoActuator* tmpbaseact = new KX_IpoActuator(
-					gameobj,
-					propname ,
-					frameProp,
-					ipoact->sta,
-					ipoact->end,
-					ipochild,
-					ipoact->type + 1, // + 1, because Blender starts to count at zero,
-					// Ketsji at 1, because zero is reserved for "NoDef"
-					ipo_as_force,
-					ipo_add,
-					local);
+				            gameobj,
+				            propname ,
+				            frameProp,
+				            ipoact->sta,
+				            ipoact->end,
+				            ipochild,
+				            ipoact->type + 1, // + 1, because Blender starts to count at zero,
+				            // Ketsji at 1, because zero is reserved for "NoDef"
+				            ipo_as_force,
+				            ipo_add,
+				            local);
 				baseact = tmpbaseact;
 				break;
 			}
@@ -293,14 +300,14 @@ void BL_ConvertActuators(const char* maggiename,
 					
 					/* visifac, fac and axis are not copied from the struct...   */ 
 					/* that's some internal state...                             */
-					KX_CameraActuator *tmpcamact
-						= new KX_CameraActuator(gameobj,
-						tmpgob,
-						camact->height,
-						camact->min,
-						camact->max,
-						camact->axis,
-						camact->damping);
+					KX_CameraActuator *tmpcamact = new KX_CameraActuator(
+					            gameobj,
+					            tmpgob,
+					            camact->height,
+					            camact->min,
+					            camact->max,
+					            camact->axis,
+					            camact->damping);
 					baseact = tmpcamact;
 				}
 				break;
@@ -312,35 +319,28 @@ void BL_ConvertActuators(const char* maggiename,
 				/* Get the name of the properties that objects must own that
 				 * we're sending to, if present
 				 */
-				STR_String toPropName = (msgAct->toPropName
-					? (char*) msgAct->toPropName
-					: "");
+				STR_String toPropName = msgAct->toPropName;
 				
 				/* Get the Message Subject to send.
 				 */
-				STR_String subject = (msgAct->subject
-					? (char*) msgAct->subject
-					: "");
+				STR_String subject = msgAct->subject;
 				
 				/* Get the bodyType
 				 */
 				int bodyType = msgAct->bodyType;
-				
+
 				/* Get the body (text message or property name whose value
 				 * we'll be sending, might be empty
 				 */
-				STR_String body = (msgAct->body
-					? (char*) msgAct->body
-					: "");
+				const STR_String body = msgAct->body;
 				
-				KX_NetworkMessageActuator *tmpmsgact = 
-					new KX_NetworkMessageActuator(
-					gameobj,					// actuator controlling object
-					scene->GetNetworkScene(),	// needed for replication
-					toPropName,
-					subject,
-					bodyType,
-					body);
+				KX_NetworkMessageActuator *tmpmsgact = new KX_NetworkMessageActuator(
+				            gameobj,					// actuator controlling object
+				            scene->GetNetworkScene(),	// needed for replication
+				            toPropName,
+				            subject,
+				            bodyType,
+				            body);
 				baseact = tmpmsgact;
 				break;
 			}
@@ -355,7 +355,7 @@ void BL_ConvertActuators(const char* maggiename,
 				KX_SoundActuator::KX_SOUNDACT_TYPE
 					soundActuatorType = KX_SoundActuator::KX_SOUNDACT_NODEF;
 				
-				switch(soundact->type) {
+				switch (soundact->type) {
 				case ACT_SND_PLAY_STOP_SOUND:
 					soundActuatorType = KX_SoundActuator::KX_SOUNDACT_PLAYSTOP;
 					break;
@@ -384,10 +384,10 @@ void BL_ConvertActuators(const char* maggiename,
 				{
 					bSound* sound = soundact->sound;
 					bool is3d = soundact->flag & ACT_SND_3D_SOUND ? true : false;
-					AUD_Reference<AUD_IFactory> snd_sound;
+					AUD_Sound* snd_sound = NULL;
 					KX_3DSoundSettings settings;
-					settings.cone_inner_angle = soundact->sound3D.cone_inner_angle;
-					settings.cone_outer_angle = soundact->sound3D.cone_outer_angle;
+					settings.cone_inner_angle = RAD2DEGF(soundact->sound3D.cone_inner_angle);
+					settings.cone_outer_angle = RAD2DEGF(soundact->sound3D.cone_outer_angle);
 					settings.cone_outer_gain = soundact->sound3D.cone_outer_gain;
 					settings.max_distance = soundact->sound3D.max_distance;
 					settings.max_gain = soundact->sound3D.max_gain;
@@ -403,27 +403,12 @@ void BL_ConvertActuators(const char* maggiename,
 					}
 					else
 					{
-						snd_sound = *reinterpret_cast<AUD_Reference<AUD_IFactory>*>(sound->playback_handle);
+						snd_sound = sound->playback_handle;
 
 						// if sound shall be 3D but isn't mono, we have to make it mono!
 						if (is3d)
 						{
-							try
-							{
-								AUD_Reference<AUD_IReader> reader = snd_sound->createReader();
-								if (reader->getSpecs().channels != AUD_CHANNELS_MONO)
-								{
-									AUD_DeviceSpecs specs;
-									specs.channels = AUD_CHANNELS_MONO;
-									specs.rate = AUD_RATE_INVALID;
-									specs.format = AUD_FORMAT_INVALID;
-									snd_sound = new AUD_ChannelMapperFactory(snd_sound, specs);
-								}
-							}
-							catch(AUD_Exception&)
-							{
-								// sound cannot be played... ignore
-							}
+							snd_sound = AUD_Sound_rechannel(snd_sound, AUD_CHANNELS_MONO);
 						}
 					}
 					KX_SoundActuator* tmpsoundact =
@@ -435,6 +420,10 @@ void BL_ConvertActuators(const char* maggiename,
 						settings,
 						soundActuatorType);
 
+					// if we made it mono, we have to free it
+					if(snd_sound != sound->playback_handle && snd_sound != NULL)
+						AUD_Sound_free(snd_sound);
+
 					tmpsoundact->SetName(bact->name);
 					baseact = tmpsoundact;
 				}
@@ -457,11 +446,11 @@ void BL_ConvertActuators(const char* maggiename,
 					destinationObj = converter->FindGameObject(propact->ob);
 				
 				SCA_PropertyActuator* tmppropact = new SCA_PropertyActuator(
-					gameobj,
-					destinationObj,
-					propact->name,
-					propact->value,
-					propact->type+1); // + 1 because Ketsji Logic starts
+				            gameobj,
+				            destinationObj,
+				            propact->name,
+				            propact->value,
+				            propact->type + 1); // + 1 because Ketsji Logic starts
 				// with 0 for KX_ACT_PROP_NODEF
 				baseact = tmppropact;
 				break;
@@ -491,18 +480,16 @@ void BL_ConvertActuators(const char* maggiename,
 							}
 						}
 						
-						KX_SCA_AddObjectActuator* tmpaddact = 
-							new KX_SCA_AddObjectActuator(
-								gameobj, 
-								originalval,
-								editobact->time,
-								scene,
-								editobact->linVelocity,
-								(editobact->localflag & ACT_EDOB_LOCAL_LINV)!=0,
-								editobact->angVelocity,
-								(editobact->localflag & ACT_EDOB_LOCAL_ANGV)!=0
-								);
-								
+						KX_SCA_AddObjectActuator* tmpaddact = new KX_SCA_AddObjectActuator(
+						            gameobj,
+						            originalval,
+						            editobact->time,
+						            scene,
+						            editobact->linVelocity,
+						            (editobact->localflag & ACT_EDOB_LOCAL_LINV) != 0,
+						            editobact->angVelocity,
+						            (editobact->localflag & ACT_EDOB_LOCAL_ANGV) != 0);
+
 								//editobact->ob to gameobj
 								baseact = tmpaddact;
 					}
@@ -516,26 +503,16 @@ void BL_ConvertActuators(const char* maggiename,
 					break;
 				case ACT_EDOB_REPLACE_MESH:
 					{
-						RAS_MeshObject *tmpmesh = NULL;
-						if (editobact->me)
-							tmpmesh = BL_ConvertMesh(
-								editobact->me,
-								blenderobject,
-								scene,
-								converter
-								);
+						RAS_MeshObject *tmpmesh = converter->FindGameMesh(editobact->me);
 
-						KX_SCA_ReplaceMeshActuator* tmpreplaceact
-							= new KX_SCA_ReplaceMeshActuator(
-								gameobj,
-								tmpmesh,
-								scene,
-								(editobact->flag & ACT_EDOB_REPLACE_MESH_NOGFX)==0,
-								(editobact->flag & ACT_EDOB_REPLACE_MESH_PHYS)!=0
-								
-								);
-							
-							baseact = tmpreplaceact;
+						KX_SCA_ReplaceMeshActuator* tmpreplaceact = new KX_SCA_ReplaceMeshActuator(
+						            gameobj,
+						            tmpmesh,
+						            scene,
+						            (editobact->flag & ACT_EDOB_REPLACE_MESH_NOGFX) == 0,
+						            (editobact->flag & ACT_EDOB_REPLACE_MESH_PHYS) != 0);
+
+						baseact = tmpreplaceact;
 					}
 					break;
 				case ACT_EDOB_TRACK_TO:
@@ -544,25 +521,23 @@ void BL_ConvertActuators(const char* maggiename,
 						if (editobact->ob)
 							originalval = converter->FindGameObject(editobact->ob);
 							
-						KX_TrackToActuator* tmptrackact 
-							= new KX_TrackToActuator(gameobj, 
-								originalval,
-								editobact->time,
-								editobact->flag,
-								blenderobject->trackflag,
-								blenderobject->upflag
-								);
-							baseact = tmptrackact;
+						KX_TrackToActuator* tmptrackact = new KX_TrackToActuator(
+						            gameobj,
+						            originalval,
+						            editobact->time,
+						            editobact->flag,
+						            editobact->trackflag,
+						            editobact->upflag);
+						baseact = tmptrackact;
 						break;
 					}
 				case ACT_EDOB_DYNAMICS:
 					{
-						KX_SCA_DynamicActuator* tmpdynact 
-							= new KX_SCA_DynamicActuator(gameobj, 
-								editobact->dyn_operation,
-								editobact->mass
-								);
-							baseact = tmpdynact;
+						KX_SCA_DynamicActuator* tmpdynact = new KX_SCA_DynamicActuator(
+						            gameobj,
+						            editobact->dyn_operation,
+						            editobact->mass);
+						baseact = tmpdynact;
 					}
 				}
 				break;
@@ -577,8 +552,8 @@ void BL_ConvertActuators(const char* maggiename,
 				/* convert settings... degrees in the ui become radians  */ 
 				/* internally                                            */ 
 				if (conact->type == ACT_CONST_TYPE_ORI) {
-					min = (float)(((float)MT_2_PI * conact->minloc[0]) / 360.0f);
-					max = (float)(((float)MT_2_PI * conact->maxloc[0]) / 360.0f);
+					min = conact->minloc[0];
+					max = conact->maxloc[0];
 					switch (conact->mode) {
 					case ACT_CONST_DIRPX:
 						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_ORIX;
@@ -694,17 +669,17 @@ void BL_ConvertActuators(const char* maggiename,
 						; /* error */ 
 					}
 				}
-				KX_ConstraintActuator *tmpconact 
-					= new KX_ConstraintActuator(gameobj,
-						conact->damp,
-						conact->rotdamp,
-						min,
-						max,
-						conact->maxrot,
-						locrot,
-						conact->time,
-						conact->flag,
-						prop);
+				KX_ConstraintActuator *tmpconact = new KX_ConstraintActuator(
+				            gameobj,
+				            conact->damp,
+				            conact->rotdamp,
+				            min,
+				            max,
+				            conact->maxrot,
+				            locrot,
+				            conact->time,
+				            conact->flag,
+				            prop);
 				baseact = tmpconact;
 				break;
 			}
@@ -754,9 +729,8 @@ void BL_ConvertActuators(const char* maggiename,
 							break;
 						};
 						
-						if (sceneact->scene)
-						{
-							nextSceneName = sceneact->scene->id.name + 2; // this '2' is necessary to remove prefix 'SC'
+						if (sceneact->scene) {
+							nextSceneName = sceneact->scene->id.name + 2;
 						}
 						
 						break;
@@ -765,7 +739,9 @@ void BL_ConvertActuators(const char* maggiename,
 					mode = KX_SceneActuator::KX_SCENE_SET_CAMERA;
 					if (sceneact->camera)
 					{
-						cam = (KX_Camera*) converter->FindGameObject(sceneact->camera);
+						KX_GameObject *tmp = converter->FindGameObject(sceneact->camera);
+						if (tmp && tmp->GetGameObjectType() == SCA_IObject::OBJ_CAMERA)
+							cam = (KX_Camera*)tmp;
 					}
 					break;
 				case ACT_SCENE_RESTART:
@@ -777,13 +753,14 @@ void BL_ConvertActuators(const char* maggiename,
 				default:
 					; /* flag error */
 				}
-				tmpsceneact = new KX_SceneActuator(gameobj,
-						mode,
-						scene,
-						ketsjiEngine,
-						nextSceneName,
-						cam);
-					baseact = tmpsceneact;
+				tmpsceneact = new KX_SceneActuator(
+				            gameobj,
+				            mode,
+				            scene,
+				            ketsjiEngine,
+				            nextSceneName,
+				            cam);
+				baseact = tmpsceneact;
 				break;
 			}
 		case ACT_GAME:
@@ -832,13 +809,14 @@ void BL_ConvertActuators(const char* maggiename,
 				default:
 					; /* flag error */
 				}
-					tmpgameact = new KX_GameActuator(gameobj,
-						mode,
-						filename,
-						loadinganimationname,
-						scene,
-						ketsjiEngine);
-					baseact = tmpgameact;
+				tmpgameact = new KX_GameActuator(
+				            gameobj,
+				            mode,
+				            filename,
+				            loadinganimationname,
+				            scene,
+				            ketsjiEngine);
+				baseact = tmpgameact;
 
 				break;
 			}
@@ -905,12 +883,13 @@ void BL_ConvertActuators(const char* maggiename,
 				default:
 					; /* error */
 				}
-				tmprandomact = new SCA_RandomActuator(gameobj,
-					seedArg,
-					modeArg, 
-					paraArg1,
-					paraArg2,
-					randAct->propname);
+				tmprandomact = new SCA_RandomActuator(
+				            gameobj,
+				            seedArg,
+				            modeArg,
+				            paraArg1,
+				            paraArg2,
+				            randAct->propname);
 				baseact = tmprandomact;
 			}
 			break;
@@ -947,8 +926,7 @@ void BL_ConvertActuators(const char* maggiename,
 			SCA_2DFilterActuator *tmp = NULL;
 
 			RAS_2DFilterManager::RAS_2DFILTER_MODE filtermode;
-			switch(_2dfilter->type)
-			{
+			switch (_2dfilter->type) {
 				case ACT_2DFILTER_MOTIONBLUR:
 					filtermode = RAS_2DFilterManager::RAS_2DFILTER_MOTIONBLUR;
 					break;
@@ -999,8 +977,9 @@ void BL_ConvertActuators(const char* maggiename,
 					break;
 			}
 
-			tmp = new SCA_2DFilterActuator(gameobj, filtermode, _2dfilter->flag,
-				_2dfilter->float_arg,_2dfilter->int_arg,ketsjiEngine->GetRasterizer(),scene);
+			tmp = new SCA_2DFilterActuator(gameobj, filtermode,  _2dfilter->flag,
+			                               _2dfilter->float_arg, _2dfilter->int_arg,
+			                               ketsjiEngine->GetRasterizer(), scene);
 
 			if (_2dfilter->text)
 			{
@@ -1026,8 +1005,7 @@ void BL_ConvertActuators(const char* maggiename,
 				bool ghost = true;
 				KX_GameObject *tmpgob = NULL;
 
-				switch(parAct->type)
-				{
+				switch (parAct->type) {
 					case ACT_PARENT_SET:
 						mode = KX_ParentActuator::KX_PARENT_SET;
 						tmpgob = converter->FindGameObject(parAct->ob);
@@ -1055,7 +1033,15 @@ void BL_ConvertActuators(const char* maggiename,
 				bArmatureActuator* armAct = (bArmatureActuator*) bact->data;
 				KX_GameObject *tmpgob = converter->FindGameObject(armAct->target);
 				KX_GameObject *subgob = converter->FindGameObject(armAct->subtarget);
-				BL_ArmatureActuator* tmparmact = new BL_ArmatureActuator(gameobj, armAct->type, armAct->posechannel, armAct->constraint, tmpgob, subgob, armAct->weight, armAct->influence);
+				BL_ArmatureActuator* tmparmact = new BL_ArmatureActuator(
+				            gameobj,
+				            armAct->type,
+				            armAct->posechannel,
+				            armAct->constraint,
+				            tmpgob,
+				            subgob,
+				            armAct->weight,
+				            armAct->influence);
 				baseact = tmparmact;
 				break;
 			}
@@ -1073,8 +1059,7 @@ void BL_ConvertActuators(const char* maggiename,
 				KX_GameObject *targetob = converter->FindGameObject(stAct->target);
 
 				int mode = KX_SteeringActuator::KX_STEERING_NODEF;
-				switch(stAct->type)
-				{
+				switch (stAct->type) {
 				case ACT_STEERING_SEEK:
 					mode = KX_SteeringActuator::KX_STEERING_SEEK;
 					break;
@@ -1090,19 +1075,64 @@ void BL_ConvertActuators(const char* maggiename,
 				bool enableVisualization = (stAct->flag & ACT_STEERING_ENABLEVISUALIZATION) !=0;
 				short facingMode = (stAct->flag & ACT_STEERING_AUTOMATICFACING) ? stAct->facingaxis : 0;
 				bool normalup = (stAct->flag & ACT_STEERING_NORMALUP) !=0;
+				bool lockzvel = (stAct->flag & ACT_STEERING_LOCKZVEL) !=0;
 				KX_SteeringActuator *tmpstact
 					= new KX_SteeringActuator(gameobj, mode, targetob, navmeshob,stAct->dist, 
 					stAct->velocity, stAct->acceleration, stAct->turnspeed, 
 					selfTerminated, stAct->updateTime,
-					scene->GetObstacleSimulation(), facingMode, normalup, enableVisualization);
+					scene->GetObstacleSimulation(), facingMode, normalup, enableVisualization, lockzvel);
 				baseact = tmpstact;
 				break;
 			}
+		case ACT_MOUSE:
+			{
+				bMouseActuator* mouAct = (bMouseActuator*) bact->data;
+				int mode = KX_MouseActuator::KX_ACT_MOUSE_NODEF;
+
+				switch (mouAct->type) {
+					case ACT_MOUSE_VISIBILITY:
+					{
+						mode = KX_MouseActuator::KX_ACT_MOUSE_VISIBILITY;
+						break;
+					}
+					case ACT_MOUSE_LOOK:
+					{
+						mode = KX_MouseActuator::KX_ACT_MOUSE_LOOK;
+						break;
+					}
+				}
+
+				bool visible = (mouAct->flag & ACT_MOUSE_VISIBLE) != 0;
+				bool use_axis[2] = {(mouAct->flag & ACT_MOUSE_USE_AXIS_X) != 0, (mouAct->flag & ACT_MOUSE_USE_AXIS_Y) != 0};
+				bool reset[2] = {(mouAct->flag & ACT_MOUSE_RESET_X) != 0, (mouAct->flag & ACT_MOUSE_RESET_Y) != 0};
+				bool local[2] = {(mouAct->flag & ACT_MOUSE_LOCAL_X) != 0, (mouAct->flag & ACT_MOUSE_LOCAL_Y) != 0};
+
+				SCA_MouseManager* eventmgr = (SCA_MouseManager*) logicmgr->FindEventManager(SCA_EventManager::MOUSE_EVENTMGR);
+				if (eventmgr) {
+					KX_MouseActuator* tmpbaseact = new KX_MouseActuator(gameobj,
+																		ketsjiEngine,
+																		eventmgr,
+																		mode,
+																		visible,
+																		use_axis,
+																		mouAct->threshold,
+																		reset,
+																		mouAct->object_axis,
+																		local,
+																		mouAct->sensitivity,
+																		mouAct->limit_x,
+																		mouAct->limit_y);
+					baseact = tmpbaseact;
+				} else {
+					//cout << "\n Couldn't find mouse event manager..."; - should throw an error here...
+				}
+				break;
+			}
 		default:
 			; /* generate some error */
 		}
 		
-		if (baseact)
+		if (baseact && !(bact->flag & ACT_DEACTIVATE))
 		{
 			baseact->SetExecutePriority(executePriority++);
 			uniquename += "#ACT#";
@@ -1118,6 +1148,8 @@ void BL_ConvertActuators(const char* maggiename,
 			// done with baseact, release it
 			baseact->Release();
 		}
+		else if (baseact)
+			baseact->Release();
 		
 		bact = bact->next;
 	}
